CREATE SCHEMA METRICS AUTHORIZATION DBA;

SET SCHEMA METRICS;

CREATE CACHED TABLE METRIC (
	ID INT NOT NULL IDENTITY PRIMARY KEY,
    NAME VARCHAR(2500) NOT NULL,
	PARENT INT,
	
	CONSTRAINT "FK_METRIC_SQLC$PARENT" FOREIGN KEY (PARENT) REFERENCES METRIC (ID) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX "IX_METRIC_SQLC$E_NAME" ON METRIC (PARENT, NAME);

CREATE CACHED TABLE SLICE (
	ID INTEGER NOT NULL IDENTITY PRIMARY KEY,
	METRIC_ID INTEGER NOT NULL,
	TIME_FROM BIGINT NOT NULL,
	TIME_TO BIGINT NOT NULL,
	MEASUREMENTS DOUBLE NOT NULL,
	TOTAL DOUBLE NOT NULL,
	MIN_VALUE DOUBLE NOT NULL,
	MAX_VALUE DOUBLE NOT NULL,
	DEVIATION DOUBLE NOT NULL,

	CONSTRAINT "FK_SLICE_SQLC$METRIC" FOREIGN KEY (METRIC_ID) REFERENCES METRIC (ID) ON DELETE CASCADE
);

CREATE INDEX "IX_SLICE_SQLC$M_TIME_TO" ON SLICE (TIME_TO);

CREATE CACHED TABLE MEASUREMENT (
	ID INTEGER NOT NULL IDENTITY PRIMARY KEY,
	SLICE_ID INTEGER NOT NULL,
	MEASUREMENT_VALUE DOUBLE NOT NULL,
	MEASUREMENT_TIME BIGINT NOT NULL,

	CONSTRAINT FK_MEASUREMENT_SLICE FOREIGN KEY (SLICE_ID) REFERENCES SLICE (ID) ON DELETE CASCADE
);

CREATE INDEX IX_MEASUREMENT_VALUE ON MEASUREMENT (MEASUREMENT_VALUE);

CREATE CACHED TABLE KINDRED (
	ANCESTOR INTEGER NOT NULL,
	DESCENDANT INTEGER NOT NULL,
	PRIMARY KEY (ANCESTOR, DESCENDANT),
	
	CONSTRAINT FK_KINDRED_ANCESTOR FOREIGN KEY (ANCESTOR) REFERENCES METRIC (ID) ON DELETE CASCADE,
	CONSTRAINT FK_KINDRED_DESCENDANT FOREIGN KEY (DESCENDANT) REFERENCES METRIC (ID) ON DELETE CASCADE
);

