/*
@license.text@
 */
package com.pavelvlasov.metrics.sample;

import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.server.UnicastRemoteObject;

import biz.hammurapi.metrics.RemoteSliceConsumer;
import biz.hammurapi.metrics.Slice;

/**
 * Simple remote slice consumer.
 * Binds to //localhost/sliceConsumer. Creates RMI registry if necessary.
 * Outputs to System.out.  
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public class RemoteConsumer extends UnicastRemoteObject implements RemoteSliceConsumer {
	
	public RemoteConsumer(int port, String bindName) throws RemoteException, MalformedURLException {
        try {
            LocateRegistry.createRegistry(port);
            System.out.println("RMI registry created");
        } catch (Exception e) {
            System.out.println("Connecting to RMI registry");
        }
		
		String fullBindName = "//localhost:"+port+"/"+bindName;
		Naming.rebind(fullBindName, this);
	}

	public boolean consumeSlice(String category, Slice slice) throws RemoteException {
		System.out.println("["+category+"] "+slice);
		return true;
	}

	public static void main(String[] args) throws Exception {
		new RemoteConsumer(1099, "sliceConsumer");
	}
}
