/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.ant;

import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.apache.tools.ant.BuildException;

import biz.hammurapi.config.Context;
import biz.hammurapi.sql.ConnectionPerThreadDataSource;
import biz.hammurapi.sql.SQLProcessor;


/**
 * Defines database connection.
 * @ant.element name="Database connection" 
 * @author Pavel Vlasov
 * @version $Revision: 1.8 $
 */
public class ConnectionEntry {
	private String driverClass;
	private String url;
	private String user;
	private String password;
	private Collection onConnects=new ArrayList();
		
	/**
	 * Driver class.
	 * @ant.required
	 */
	public void setDriverClass(String driverClass) {
		this.driverClass = driverClass;
	}
	
	/**
	 * Database password.
	 * @ant.non-required
	 * @param password
	 */
	public void setPassword(String password) {
		this.password = password;
	}
	
	/**
	 * Connection url
	 * @ant.required
	 * @param url
	 */
	public void setUrl(String url) {
		this.url = url;
	}
	
	/**
	 * Database user
	 * @ant.non-required
	 * @param user
	 */
	public void setUser(String user) {
		this.user = user;
	}
	
	public SQLProcessor getProcessor(Context nameMap) {
		SQLProcessor ret = new SQLProcessor(getDataSource(), nameMap);
		processOnConnects(ret);
		return ret;
	}

	/**
	 * @param ret
	 */
	private void processOnConnects(SQLProcessor ret) {
		Iterator it=onConnects.iterator();
		while (it.hasNext()) {
			try {
				((Script) it.next()).execute(ret);
			} catch (IOException e) {
				throw new BuildException(e);
			} catch (SQLException e) {
				throw new BuildException(e);
			}
		}
	}
	
	private Collection dataSources=new ArrayList();

	/**
	 * @return
	 * @throws ClassNotFoundException
	 */
	public ConnectionPerThreadDataSource getDataSource() {
		try {
			ConnectionPerThreadDataSource ds = new ConnectionPerThreadDataSource(driverClass, url, user, password, null);
			dataSources.add(ds);
            return ds;
		} catch (ClassNotFoundException e) {
			throw new BuildException("Cannot load driver class", e);
		}
	}
	
	public Connection getConnection() throws SQLException, ClassNotFoundException {
		Class.forName(driverClass);
		return DriverManager.getConnection(url, user, password);

	}
	
	public Script createOnConnect() {
		Script ret = new Script();
		onConnects.add(ret);
		return ret;
	}

	public void shutdown() {
	    Iterator it=dataSources.iterator();
	    while (it.hasNext()) {
	        ((ConnectionPerThreadDataSource) it.next()).shutdown();
	    }	        
	}
}
