/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.ant;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;

import biz.hammurapi.config.Command;
import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.config.Context;
import biz.hammurapi.config.DomConfigFactory;
import biz.hammurapi.config.DomConfigurableContainer;
import biz.hammurapi.logging.AntLogger;


/**
 * PV Container wrapped in Ant task.
 * Instantiates container, mounts /logger and /taks and then
 * executes nested &lt;command&gt; entries
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public class ContainerTask extends Task {
	private Collection executes=new ArrayList();
	
	/**
	 * Context to "execute".
	 * @author Pavel Vlasov
	 * @revision $Revision$
	 */
	public class Execute extends ObjectEntry {
		private String path;
		
		public void setPath(String path) {
			this.path=path;
		}
	}
	
	public Execute createExecute() {
		Execute ret=new Execute();
		executes.add(ret);
		return ret;
	}
	
	public ContainerTask() {
		super();
	}
	
	private String url;
	private File file;
	private String resource;
	
	/**
	 * URL to load configuration from. 
	 * One of URL, file or resource is mandatory. 
	 * These attributes are mutually exclusive.
	 * @ant.not-required
	 * @param url
	 */
	public void setUrl(String url) {
		checkAlreadySet();
		this.url=url;
	}
	
	/**
	 * File to load configuration from. 
	 * One of URL, file or resource is mandatory. 
	 * These attributes are mutually exclusive.
	 * @ant.not-required
	 * @param url
	 */
	public void setFile(File file) {
		checkAlreadySet();
		this.file=file;
	}
	
	/**
	 * Classloader resource to load configuration from. 
	 * One of URL, file or resource is mandatory. 
	 * These attributes are mutually exclusive.
	 * @ant.not-required
	 * @param url
	 */
	public void setResource(String resource) {
		checkAlreadySet();
		this.resource=resource;
	}
	
	private void checkAlreadySet() {
		if (url!=null) {
			throw new BuildException("url already set");
		}
		
		if (file!=null) {
			throw new BuildException("file already set");
		}
		
		if (resource!=null) {
			throw new BuildException("resource already set");
		}
	}
	
	public void execute() throws BuildException {
		try {
			ClassLoader classLoader = getClass().getClassLoader();
			DomConfigFactory factory=new DomConfigFactory(classLoader, (Context) null);
			Object o;
			if (url!=null) {
				o=factory.create(new URL(url), null);
			} else if (file!=null) {
				o=factory.create(file, null);			
			} else if (resource!=null) {
				InputStream in=classLoader.getResourceAsStream(resource);
				if (in==null) {
					throw new BuildException("Resource "+resource+" not found");
				}
				o=factory.create(in, null);			
			} else {
				throw new BuildException("One of url, file or resource attributes must be set");
			}
			
			if (o instanceof DomConfigurableContainer) {
				DomConfigurableContainer container=(DomConfigurableContainer) o;
				if (container.get("logger")==null) {
					container.addComponent("logger", new AntLogger(this)); 
				}
				if (container.get("task")==null) {
					container.addComponent("task", this); 
				}
				
				container.start();
				try {
					Iterator it=executes.iterator();
					while (it.hasNext()) {
						Execute execute=(Execute) it.next();
						Object target=container.get(execute.path);
						if (target instanceof Command) {
							((Command) target).execute(execute.getObject(classLoader));
						}
					}
				} finally {
					container.stop();
				}
			}			
		} catch (IOException e) {
			throw new BuildException(e.toString(), e);
		} catch (ConfigurationException e) {
			throw new BuildException(e.toString(), e);
		}
		
	}
	
	
}
