/*
  * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
  */
package biz.hammurapi.ant;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.StringReader;
import java.sql.SQLException;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;

import biz.hammurapi.sql.SQLProcessor;

/**
 * Executes SQL script
 * @author Pavel Vlasov
 *
 * @version $Revision: 1.1 $
 */
public class Script extends Task {
	private String script="";
	private String delimiter=";";
	private ConnectionEntry connectionEntry;
	
	/**
	 * Connection
	 * @ant.non-required
	 * @param connectionEntry
	 */
	public void addConfiguredConnection(ConnectionEntry connectionEntry) {
		this.connectionEntry=connectionEntry;
	}
	
	/**
	 * Statements delimiter (single character). Defaults to ';'
	 * @ant.non-required.
	 * @param delimiter
	 */
	public void setDelimiter(String delimiter) {
		this.delimiter=delimiter;
	}
	
	/**
	 * Script text. Required if file is not set.
	 * @ant.non-required
	 * @param text
	 */
	public void  addText(String text) {
		script+=text;
	}
	
	private File file;
	
	/**
	 * Script file. Required if there is no nested text.
	 * File content is executed after text.
	 * @ant.non-required
	 * @param text
	 */
	public void setFile(File file) {
		this.file = file;
	}
	
	public void execute(SQLProcessor processor) throws IOException, SQLException {
		if (script.trim().length()>0) {
			processor.executeScript(new StringReader(script), delimiter.charAt(0));
		}
		
		if (file!=null) {
			processor.executeScript(new FileReader(file), delimiter.charAt(0));
		}
	}
	
	public void execute() throws BuildException {
		try {
			execute(new SQLProcessor(connectionEntry.getDataSource(),null));
		} catch (IOException e) {
			throw new  BuildException(e);
		} catch (SQLException e) {
			throw new  BuildException(e);
		}
	}
}
