/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.ant;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.FileSet;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import biz.hammurapi.xml.dom.DOMUtils;
import biz.hammurapi.xml.dom.DOMUtils;

/**
 * This task applies a stylesheet to XML files. Output file name is calculated from XPath expression.
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public class StyleTask extends Task {

	private File outputDir;
	
	private String fileName;
	
	private Collection fileSets = new ArrayList();
	
	private File style;
	
	/**
	 * Output directory.
	 * @param outputDir
	 */
	public void setOutputDir(File outputDir) {
		this.outputDir = outputDir;
	}
	
	/**
	 * XPath expression for file name. Use / for file separator.
	 * @param fileName
	 */
	public void setFileName(String fileName) {
		this.fileName = fileName;
	}
	
	public void addFileSet(FileSet fileSet) {
		fileSets.add(fileSet);
	}
	
	public void setStyle(File style) {
		this.style = style;
	}
	
	public void execute() throws BuildException {
		try {
			Iterator it=fileSets.iterator();
			while (it.hasNext()) {
				FileSet fs=(FileSet) it.next();
				DirectoryScanner ds = fs.getDirectoryScanner(getProject());
				String[] files=ds.getIncludedFiles();
				for (int i=0; i<files.length; i++) {
					File in = new File(ds.getBasedir(), files[i]);
					Document doc = DOMUtils.parse(in);
					
					String fn = DOMUtils.eval(doc.getDocumentElement(), fileName).toString().replace('/', File.separatorChar);
					File out = new File(outputDir, fn);
					
					File parentDir = out.getParentFile();
					if (!parentDir.exists()) {
						if (!parentDir.mkdirs()) {
							throw new BuildException("Could not create directory: "+parentDir.getAbsolutePath());
						}
					}
					
					DOMUtils.style(doc, out, new FileInputStream(style), null);
				}
			}
		} catch (Exception e) {
			throw new BuildException(e);
		}
	}
	
}
