/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.ant;

import java.io.File;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;
import org.w3c.dom.Element;

/**
 * Base class for elements, which need to read xml files.
 * @author Pavel Vlasov
 * @version $Revision: 1.2 $
 */
public class XmlSourceEntry extends Task {

    /** Holds value of property failOnError. */
    private boolean failOnError = true;

    /**
     * Fail build if unable to read source. Default is true.
     * @ant.non-required
     */
    public void setFailOnError(boolean failOnError) {
        this.failOnError = failOnError;
    }

    /** Getter for property failOnError.
     * @return Value of property failOnError.
     */
    public boolean isFailOnError() {
        return this.failOnError;
    }

    /**
     * URL to read rules from. 
     * @ant.non-required Yes, unless file is set.
     */
    public void setURL(String url) {
        this.url = url;
    }

    /** Getter for property URL.
     * @return Value of property URL.
     */
    public String getURL() {
        return this.url;
    }

    /**
     * File to read rules from.
     * @ant.required Yes, unless URL is set.
     */
    public void setFile(File file) {
    	this.file=file;
    }

    /** Getter for property file.
     * @return Value of property file.
     */
    public File getFile() {
        return this.file;
    }

    /** Holds value of property URL. */
    private String url;
    /** Holds value of property file. */
    private File file;

    public Element getDocumentElement() {
        if (file!=null && url!=null) {
            throw new BuildException("file and url are mutually exclusive");
        }
        
        try {
            DocumentBuilder builder=DocumentBuilderFactory.newInstance().newDocumentBuilder();
            if (file!=null) {
        		if (failOnError && !(file.exists() && file.isFile())) {
        			throw new BuildException(file.getAbsolutePath()+" does not exist or is not a file");
        		}

            	if (getProject()!=null) {
            		log("Parsing "+file.getAbsolutePath(), Project.MSG_VERBOSE);
            	}
                return builder.parse(file).getDocumentElement(); 
            } else if (url!=null) {
            	if (getProject()!=null) {
            		log("Parsing URL "+url, Project.MSG_VERBOSE);
            	}
                java.net.URL theURL=new java.net.URL(url);
                return builder.parse(theURL.openStream()).getDocumentElement();
            } else {
                throw new BuildException("file or url must be set");
            }
        } catch (Exception e) {
            if (failOnError) {
                throw new BuildException("Cannot load xml document", e);
            }
            
			return null;
        }        
    }    
}
