/**
 * Borrowed from Terence Parr
 */
package biz.hammurapi.antlr;

/* ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/license.html
 *
 * $Id: //depot/code/org.antlr/release/antlr-2.7.5/antlr/debug/misc/ASTFrame.java#1 $
 */

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Frame;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import antlr.collections.AST;

public class ASTFrame extends JFrame {
	
	public static class JTreeASTPanel extends JPanel {
	    JTree tree;

	    public JTreeASTPanel(TreeModel tm, TreeSelectionListener listener) {
	        // use a layout that will stretch tree to panel size
	        setLayout(new BorderLayout());

	        // Create tree
	        tree = new JTree(tm);

	        // Change line style
	        tree.putClientProperty("JTree.lineStyle", "Angled");

	        // Add TreeSelectionListener
	        if (listener != null)
	            tree.addTreeSelectionListener(listener);

	        // Put tree in a scrollable pane's viewport
	        JScrollPane sp = new JScrollPane();
	        sp.getViewport().add(tree);

	        add(sp, BorderLayout.CENTER);
	    }
	}	
	
	public static class JTreeASTModel implements TreeModel {
		
		private class Node {
			AST master;

			/**
			 * @param master
			 */
			public Node(AST master) {
				super();
				// TODO Auto-generated constructor stub
				this.master = master;
			}

			public String toString() {
				return "["+tokenNames[master.getType()]+" "+master.getLine()+":"+master.getColumn()+"] "+ master.getText();
			}			
			
			public int hashCode() {
				return master.hashCode();
			}
			
			public boolean equals(Object obj) {
				return obj instanceof Node && master.equals(((Node) obj).master);
			}
		}

	    AST root = null;
		private String[] tokenNames;

	    public JTreeASTModel(AST t, String[] tokenNames) {
	        if (t == null) {
	            throw new IllegalArgumentException("root is null");
	        }
	        root = t;
	        this.tokenNames=tokenNames;
	    }

	    public void addTreeModelListener(TreeModelListener l) {
	    }

	    public Object getChild(Object parent, int index) {
	        if (parent == null) {
	            return null;
	        }
	        AST p = ((Node) parent).master;
	        AST c = p.getFirstChild();
	        if (c == null) {
	            throw new ArrayIndexOutOfBoundsException("node has no children");
	        }
	        int i = 0;
	        while (c != null && i < index) {
	            c = c.getNextSibling();
	            i++;
	        }
	        
	        return new Node(c);
	    }

	    public int getChildCount(Object parent) {
	        if (parent == null) {
	            throw new IllegalArgumentException("root is null");
	        }
	        AST p = ((Node)parent).master;
	        AST c = p.getFirstChild();
	        int i = 0;
	        while (c != null) {
	            c = c.getNextSibling();
	            i++;
	        }
	        return i;
	    }

	    public int getIndexOfChild(Object parent, Object child) {
	        if (parent == null || child == null) {
	            throw new IllegalArgumentException("root or child is null");
	        }
	        AST p = ((Node) parent).master;
	        AST c = p.getFirstChild();
	        if (c == null) {
	            throw new ArrayIndexOutOfBoundsException("node has no children");
	        }
	        int i = 0;
	        while (c != null && c != ((Node) child).master) {
	            c = c.getNextSibling();
	            i++;
	        }
	        if (c == ((Node) child).master) {
	            return i;
	        }
	        throw new java.util.NoSuchElementException("node is not a child");
	    }

	    public Object getRoot() {
	        return new Node(root);
	    }

	    public boolean isLeaf(Object node) {
	        if (node == null) {
	            throw new IllegalArgumentException("node is null");
	        }
	        AST t = ((Node) node).master;
	        return t.getFirstChild() == null;
	    }

	    public void removeTreeModelListener(TreeModelListener l) {
	    }

	    public void valueForPathChanged(TreePath path, Object newValue) {
	        System.out.println("heh, who is calling this mystery method?");
	    }
	}		
	
    // The initial width and height of the frame
    static final int FRAME_WIDTH = 400;
    static final int FRAME_HEIGHT = 600;

    public ASTFrame(String lab, AST r, String[] tokenNames) {
        super(lab);

        JTreeASTPanel tp = new JTreeASTPanel(new JTreeASTModel(r, tokenNames), null);
        Container content = getContentPane();
        content.add(tp, BorderLayout.CENTER);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                Frame f = (Frame)e.getSource();
                f.setVisible(false);
                f.dispose();
                // System.exit(0);
            }
        });
        setSize(FRAME_WIDTH, FRAME_HEIGHT);
    }
}
