/*
  * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.antlr;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import antlr.collections.AST;

/**
 * Serializes AST to DOM
 * @author Pavel Vlasov
 */
public class AstUtil {
	/**
	 * Serializes AST, its children and its siblings to DOM.
	 * @param node
	 * @param tokenNames
	 * @param holder
	 */
	public static void toDom(antlr.collections.AST node, String[] tokenNames, Element holder) {
		for (AST ast=node; ast!=null; ast=ast.getNextSibling()) {
			holder.appendChild(toDom(ast, tokenNames, holder.getOwnerDocument()));
		}
	}
	
	/**
	 * Serializes AST and its children to DOM.
	 * @param node
	 * @param tokenNames
	 * @param owner
	 */
	public static Element toDom(antlr.collections.AST node, String[] tokenNames, Document owner) {
		Element ret=owner.createElement("ast");
		ret.setAttribute("type", tokenNames[node.getType()]);
		if (node.getText()!=null) {
			ret.appendChild(owner.createTextNode(node.getText()));
		}
		for (AST child=node.getFirstChild(); child!=null; child=child.getNextSibling()) {
			ret.appendChild(toDom(child, tokenNames, owner));
		}
		return ret;
	}
	
	private static String tabs(int t) {
		StringBuffer sb=new StringBuffer();
		for (int i=0; i<t; i++) {
			sb.append("....");			
		}
		return sb.toString();
	}
	
	/**
	 * Dumps node and siblings 
	 * @param node
	 * @param tokenNames
	 */
	public static void dumpAll(antlr.collections.AST node, String[] tokenNames) {
		System.out.println("=== AST Dump ===");
		while (node!=null) {
			dump(node, tokenNames, 0);
			node=node.getNextSibling();
		}
	}
	
	/**
	 * Dumps this node and its children
	 * @param node
	 */
	public static void dump(antlr.collections.AST node, String[] tokenNames) {
		dump(node, tokenNames, 0);
	}
		
	private static void dump(antlr.collections.AST node, String[] tokenNames, int tab) {
		System.out.print(tabs(tab));
		System.out.print(tokenNames[node.getType()]);
		if (node.getText()!=null) {
			System.out.println(" '"+node.getText()+"'");
		}
		for (AST child=node.getFirstChild(); child!=null; child=child.getNextSibling()) {
			dump(child, tokenNames, tab+1);
		}		
	}
}
