/*
  * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz 
 */
package biz.hammurapi.antlr;

import antlr.collections.AST;
import biz.hammurapi.util.PoliteVisitor;
import biz.hammurapi.util.Visitable;
import biz.hammurapi.util.Visitor;


/**
 * Navigates Visitor through DOM tree.
 * @author Pavel Vlasov
 * @version $Revision: 1.2 $
 */
public class AstVisitable implements Visitable {
	private AST node;
	private boolean withSiblings;

	public boolean accept(Visitor visitor) {
		if (withSiblings) {
			for (AST ast=node; ast!=null; ast=ast.getNextSibling()) {
				accept(ast, visitor);
			}			
			return true;
		}
		
		return accept(node, visitor);
	}
	
	/**
	 * @param node
	 * @param visitor
	 */
	private boolean accept(AST node, Visitor visitor) {
		if (visitor.visit(node)) {
			for (AST child=node.getFirstChild(); child!=null; child=child.getNextSibling()) {
				accept(child, visitor);
			}
			if (visitor instanceof PoliteVisitor) {
				((PoliteVisitor) visitor).leave(node);
			}
			return true;
		}
		
		return false;
	}

	/**
	 * @param node
	 */
	public AstVisitable(AST node, boolean withSiblings) {
		super();
		this.node = node;
		this.withSiblings=withSiblings;
	}
}
