package biz.hammurapi.antlr;

import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.MutableTreeNode;

import biz.hammurapi.swing.Visualizable;

public class AstVisualizable implements Visualizable {
	
	private AST ast;
	private String[] tokenNames;

	public AstVisualizable(AST ast, String[] names) {
		super();
		this.ast = ast;
		tokenNames = names;
	}

	public MutableTreeNode toTree(final String title) {
		DefaultMutableTreeNode ret=new DefaultMutableTreeNode(this) {
			public String toString() {
				return title+" ["+tokenNames[ast.getType()]+"] "+ast.getLine()+":"+ast.getColumn()+" "+ast.getText();
			}
		};
		
		for (AST child=(AST) ast.getFirstChild(); child!=null; child=(AST) child.getNextSibling()) {
			ret.add(new AstVisualizable(child, tokenNames).toTree(""));
		}
		
		return ret;
	}

	public TableModel toTable() {
		DefaultTableModel ret=new DefaultTableModel(4, 2);
		
		ret.setColumnIdentifiers(new String[] {"Property", "Value"});
		setRow(ret, 0, "Type", tokenNames[ast.getType()]);
		setRow(ret, 1, "Text", ast.getText());
		setRow(ret, 2, "Line", String.valueOf(ast.getLine()));
		setRow(ret, 3, "Column", String.valueOf(ast.getColumn()));
		
		return ret;
	}

	private void setRow(DefaultTableModel ret, int row, String name, Object value) {
		ret.setValueAt(name, row, 0);
		ret.setValueAt(value, row, 1);
	}		
	
}
