/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */

package biz.hammurapi.antlr;

import antlr.CommonToken;
import biz.hammurapi.legacy.review.SourceMarker;
import biz.hammurapi.util.PoliteVisitor;
import biz.hammurapi.util.Visitable;
import biz.hammurapi.util.Visitor;


/**
 *
 * @author  Pavel Vlasov
 * @version $Revision: 1.4 $
 */
public class Token extends CommonToken implements SourceMarker, Comparable, Visitable {
    public Token() {
        super();
    }
    
    public Token(String s) { 
        super(s); 
    }
    
    public Token(int t, String txt) { 
        super(t,txt);
    }
    
    private Token nextToken;
    private Token prevToken;
    
    public void setNextToken(Token token) {
    	if (nextToken!=token) {
	        nextToken=token;
	        if (nextToken!=null) {
	            nextToken.prevToken=this;
	        }
	        
	        if (listener!=null) {
	        	listener.onNextTokenChanged(this);
	        }
    	}
    }
    
    public Token getNextToken() {
        return nextToken;
    }
    
    public void setPrevToken(Token token) {
    	if (prevToken!=token) {
	        prevToken=token;
	        if (prevToken!=null) {
	            prevToken.nextToken=this;
	        }
	        
	        if (listener!=null) {
	        	listener.onPrevTokenChanged(this);
	        }
    	}
    }
    
    public Token getPrevToken() {
        return prevToken;
    }        
    
    private TokenListener listener;    
    
	/**
	 * @return Returns the listener.
	 */
	public TokenListener getListener() {
		return listener;
	}

	/**
	 * @param listener The listener to set.
	 */
	public void setListener(TokenListener listener) {
		this.listener = listener;
	}

	/* (non-Javadoc)
	 * @see antlr.Token#setText(java.lang.String)
	 */
	public void setText(String newText) {
		super.setText(newText);
		if (listener!=null) {
			listener.onTextChanged(this);
		}
	}
	
	private String sourceURL;

	public String getSourceURL() {
		return sourceURL;
	}
	
	/**
	 * Sets source URL for this and all subsequent tokens.
	 * @param sourceURL
	 */
	public void setSourceURL(String sourceURL) {
		this.sourceURL=sourceURL;
	}

	public int compareTo(Object o) {
		if (o instanceof Token) {
			Token t=(Token) o;
			if (t.getLine()==getLine()) {
				if (t.getColumn()==getColumn()) {
					if (getText()==null) {
						return t.getText()==null ? 0 : 1;
					}
					
					return t.getText()==null ? -1 : getText().compareTo(t.getText());
				}
				
				return getColumn()-t.getColumn();
			}
			
			return getLine()-t.getLine();
		}
		
		return 1;
	}	
	
	public boolean equals(Object obj) {
		if (obj==this) {
			return true;
		}
		
		if (obj==null) {
			return false;			
		}
		
		if (obj instanceof Token && getClass().equals(obj.getClass())) {
			Token t=(Token) obj;
			return getType() == t.getType()
				&& ((getText() == null && t.getText() == null)
					|| (getText() != null
						&& t.getText() != null
						&& getText().equals(t.getText())));
		}
		
		return super.equals(obj);
	}
	
	public int hashCode() {
		int ret=getType();
		ret^=getClass().getName().hashCode();
		
		if (getText()!=null) {
			ret^=getText().hashCode();
		}
		return ret;
	}

	public boolean accept(Visitor visitor) {
		for (Token t=this; t!=null; t=t.getNextToken()) {
			if (visitor.visit(t)) {
				if (visitor instanceof PoliteVisitor) {
					((PoliteVisitor) visitor).leave(t);
				}
			} else {
				return false;
			}
		}
		return true;
	}

	public Integer getSourceId() {
		return null;
	}
}
