/*
  * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz 
 */
package biz.hammurapi.cache;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.util.Acceptor;


/**
 * Simple cache. Uses map internally.
 * @author Pavel Vlasov
 * @version $Revision: 1.3 $
 */
public class SimpleMemoryCache extends AbstractProducer implements Cache {
	private Map cache=new HashMap();
	private Producer producer;
		
	/**
	 * 
	 */
	public SimpleMemoryCache(Producer producer) {
		super();
		
		this.producer=producer;
		if (producer!=null) {
			producer.addCache(this);
		}
		
	}
	
	public void put(Object key, final Object value, final long time, final long expirationTime) {
		synchronized (cache) {
			cache.put(key, new Entry() {

				public long getExpirationTime() {
					return expirationTime;
				}

				public long getTime() {
					return time;
				}

				public Object get() {
					return value;
				}
				
			});
		}
	}

	public Entry get(Object key) {	    
		
		synchronized (cache) {
			Entry entry=(Entry) cache.get(key);
			long now = System.currentTimeMillis();
			if (entry!=null) {
				if (entry.getExpirationTime()>0 && entry.getExpirationTime()<now) {
					cache.remove(key);		
				} else {
					return entry;
				}
			} 
			
			if (key instanceof ProducingKey) {
				entry = ((ProducingKey) key).get();
				cache.put(key, entry);
				return entry;
			}
			
			if (producer!=null) {
				entry = producer.get(key);
				cache.put(key, entry);
				return entry;
			}
			
			return null;
		}
	}
	
	public void clear() {
		synchronized (cache) {
			cache.clear();
		}
	}
	
	public void remove(Object key) {
		synchronized (cache) {
			cache.remove(key);
		}
		onRemove(key);
	}

	public void remove(Acceptor acceptor) {
		synchronized (cache) {
			Iterator it=cache.keySet().iterator();
			while (it.hasNext()) {
				Object key=it.next();
				if (acceptor.accept(key)) {
					it.remove();
					onRemove(key);
				}
			}
		}
	}
	
	public void stop() {
		cache.clear();
	}
	
	public Set keySet() {
		HashSet ret = new HashSet();
		synchronized (cache) {
			ret.addAll(cache.keySet());
		}
		
		if (producer!=null) {
			Set pkeys=producer.keySet();
			if (pkeys!=null) {
				ret.addAll(pkeys);
			}
		}
		
		return ret;
	}

	public boolean isActive() {
		return true;
	}

	public void start() throws ConfigurationException {
		// Nothing		
	}

	public void setOwner(Object owner) {
		// Nothing		
	}
}
