/*
  * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
  */
package biz.hammurapi.config;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import biz.hammurapi.convert.CompositeConverter;

/**
 * Populates a bean with parameters from HttpRequest.
 * @author Pavel Vlasov
 *
 * @version $Revision: 1.2 $
 */
public class Bean2ContextConfigurableAdapter implements ContextConfigurable {
	private Object bean;

	public Bean2ContextConfigurableAdapter(Object bean) {
		super();
		this.bean=bean;
	}
	
	public void configure(Context context, CompositeConverter converter) throws ConfigurationException {
		Method[] ma=bean.getClass().getMethods();
		for (int i=0; i<ma.length; i++) {
			String mname = ma[i].getName();
			if (mname.startsWith("set") && mname.length()>=4 && ma[i].getParameterTypes().length==1 && ma[i].getReturnType().equals(void.class)) {				
				String pName = mname.length()==4 ? mname.toLowerCase().substring(3) : mname.substring(3, 4).toLowerCase() + mname.substring(4);
				Object value=context.get(pName);
				if (value!=null) {
					try {
						ma[i].invoke(bean, new Object[] {converter.convert(value, ma[i].getParameterTypes()[0], false)});
					} catch (IllegalAccessException e) {
						throw new ConfigurationException("Cannot invoke method "+ma[i]+" to set parameter value "+value, e);
					} catch (InvocationTargetException e) {
						throw new ConfigurationException("Cannot invoke method "+ma[i]+" to set parameter value "+value, e);
					}
				}
			}
		}
		
		Field[] fa=bean.getClass().getFields();
		for (int i=0; i<fa.length; i++) {
			Object value=context.get(fa[i].getName());
			if (value!=null) {
				try {
					fa[i].set(bean, converter.convert(value, fa[i].getType(), false));
				} catch (IllegalAccessException e) {
					throw new ConfigurationException("Cannot set field "+fa[i]+" to value "+value, e);
				}
			}
		}		
	}
}
