/*
  * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
  */
package biz.hammurapi.config;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import biz.hammurapi.convert.CompositeConverter;

/**
 * Populates a bean with parameters from HttpRequest.
 * @author Pavel Vlasov
 *
 * @version $Revision: 1.3 $
 */
public class BeanContext implements Context {
	private Object bean;

	public BeanContext(Object bean) {
		super();
		this.bean=bean;
	}
	
	public Object get(String name) {
		String[] ni=split(name);
		Method[] ma=bean.getClass().getMethods();
		
		for (int i=0; i<ma.length; i++) {
			Method candidate = ma[i];
			Class[] cpt = candidate.getParameterTypes();
			if (cpt.length==ni.length-1 && !candidate.getReturnType().equals(void.class)) {
	            String mname = candidate.getName();
				if (mname.startsWith("get") && mname.length()>=4) {
					String pName = mname.length()==4 ? mname.substring(3).toLowerCase() : mname.substring(3, 4).toLowerCase() + mname.substring(4);
					if (ni[0].equals(pName)) {
						try {
							CompositeConverter converter = CompositeConverter.getDefaultConverter();
							Object[] args=new Object[ni.length-1];
							for (int j=0; j<args.length; j++) {
								args[j]=converter.convert(ni[j+1], cpt[j], false);
							}
							return candidate.invoke(bean, args);
						} catch (IllegalAccessException e) {
							throw new RuntimeConfigurationException("Cannot invoke method "+candidate, e);
						} catch (InvocationTargetException e) {
							throw new RuntimeConfigurationException("Cannot invoke method "+candidate, e);
						}
					}
				}
			}
		}

		if (ni.length==1) {
			Field[] fa=bean.getClass().getFields();
			for (int i=0; i<fa.length; i++) {
				if (fa[i].getName().equals(ni[0])) {
					try {
						return fa[i].get(bean);
					} catch (IllegalAccessException e) {
						throw new RuntimeConfigurationException("Cannot get field "+fa[i]+" value", e);
					}
				}
			}
		}
			
		return null;
	}
	
	/**
	 * Splits property name into getter name and indexes.
	 * Override this method to achieve desired functionality.
	 * @param name Property name.
	 * @return Name split into getter name and indexes, new String[] {name} if not overriden.
	 */
	protected String[] split(String name) {
	    return new String [] {name};
	}

}
