/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.config;

import biz.hammurapi.metrics.MeasurementCollector;
import biz.hammurapi.metrics.MeasurementConsumer;

/**
 * Base class for components. Implements some standard functions.
 * Measurement collection is delegated if measurement consumer is set.
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public abstract class ComponentBase implements Component, Context, MeasurementConsumer, MeasurementCollector {

	protected Object owner;
	
	private PathNavigator pathNavigator=new PathNavigator(this) {

		protected Object getParent() {
			return owner;
		}

		protected Object getChild(String name) {
			return ComponentBase.this.getChild(name);
		}
		
	};
	
	/**
	 * Override this method if component has subcomponents.
	 * @param name
	 * @return
	 */
	protected Object getChild(String name) {
		return null;
	}

	public void setOwner(Object owner) {		
		this.owner=owner;
	}

	public Object get(String name) {
		return pathNavigator.get(name);
	}

	private MeasurementConsumer measurementConsumer;
	
	public void setMeasurementConsumer(MeasurementConsumer measurementConsumer) {
		this.measurementConsumer = measurementConsumer;		
		if (measurementConsumer instanceof Component) {
			((Component) getMeasurementConsumer()).setOwner(this);
		}
	}
	
	public MeasurementConsumer getMeasurementConsumer() {
		return measurementConsumer;
	}
	
	public void addMeasurement(String name, double value, long time) {
		if (measurementConsumer!=null) {
			measurementConsumer.addMeasurement(name, value, time==0 ? System.currentTimeMillis() : time);
		}		
	}
	
	/**
	 * Finds component owner of particular type.
	 * @param ownerType
	 * @return Owner which is an instance of specified type or null if no such owner is found.
	 */
	public Object getOwner(Class ownerType) {
		if (owner==null || ownerType.isInstance(owner)) {
			return owner;
		}
		
		if (owner instanceof ComponentBase) {
			return ((ComponentBase) owner).getOwner(ownerType);
		}
		
		if (owner instanceof GenericContainer) {
			return ((GenericContainer) owner).getOwner(ownerType);
		}
		
		return null;
	}
	
	/**
	 * @return Immediate component owner
	 */
	public Object getOwner() {
		return owner;
	}
}
