/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.config;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import biz.hammurapi.eval.CircularReferenceException;

/**
 * Manages dependencies between objects.
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public class DependencyManager {
	private Map dependencyMap=new HashMap();
	
	/**
	 * Adds dependency. Master can be null for independent objects.
	 * @param slave
	 * @param master
	 */
	public void addDependency(Object slave, Object master) {
		Set masters=(Set) dependencyMap.get(slave);
		if (masters==null) {
			masters=new HashSet();
			dependencyMap.put(slave, masters);
		}
		if (master!=null) {
			masters.add(master);
		}
	}
	
	/**
	 * Finds out whether slave depends on master directly or indirectly
	 * @param slave
	 * @param master
	 * @return
	 */
	public boolean isDependent(Object slave, Object master) {
		return isDependent(slave, master, slave);
	}

	private boolean isDependent(Object slave, Object master, Object originalSlave) {
		if (master==originalSlave) {
			throw new CircularReferenceException("Circular reference : "+originalSlave);
		}
		
		Set masters=(Set) dependencyMap.get(slave);		
		if (masters!=null && masters.contains(master)) {
			return true;
		}
		
		Iterator it=masters.iterator();
		while (it.hasNext()) {
			if (isDependent(it.next(), master, originalSlave)) {
				return true;
			}
		}
		
		return false;
	}
	
	/**
	 * @return List of items ordered by dependency - independent items first.
	 */
	public List getOrdered() {
		ArrayList ret=new ArrayList(dependencyMap.keySet());
		Collections.sort(
				ret,
				new Comparator() {

					public int compare(Object o1, Object o2) {
						if (o1==o2) {
							return 0;							
						}
						
						if (isDependent(o1, o2)) {
							return 1;
						}
						
						if (isDependent(o2,o1)) {
							return -1;
						}
						
						return o1.hashCode()-o2.hashCode();
					}
					
				});
		return ret;
	}
 
}
