package biz.hammurapi.config;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

/**
 * This is a dynamic proxy wrapper for one component to obtain a reference to another.
 * @author Pavel Vlasov
 */
public class Reference implements Wrapper, Component {
	
	private Object master;
	
	/**
	 * Proxy to the master. Master may not exist at the time of proxy creation.
	 */
	private Object proxy;
	
	public Object getMaster() {
		return proxy;
	}
	
	private Object owner;

	public void setOwner(Object owner) {
		this.owner = owner;		
	}

	public void start() throws ConfigurationException {
		if (owner instanceof Context) {
			master = ((Context) owner).get(path);
			if (master==null) {
				throw new ConfigurationException("Master is null, path: "+path);
			}
		} else {
			throw new ConfigurationException("Owner does not implement "+Context.class.getName());
		}		
	}

	public void stop() throws ConfigurationException {
		// TODO Auto-generated method stub
		
	}
	
	/**
	 * Interface which dynamic proxy shall implement.
	 * @param type
	 * @throws ClassNotFoundException
	 */
	public void setType(String type) throws ClassNotFoundException {
		InvocationHandler ih = new InvocationHandler() {

			public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
				if (master==null) {
					throw new IllegalStateException("Master is null, path: "+path);
				}
				return method.invoke(master, args);
			}
			
		};
		
		proxy = Proxy.newProxyInstance(this.getClass().getClassLoader(), new Class[] {Class.forName(type)}, ih);
	}
	
	private String path;
	
	/**
	 * Path to the referenced component
	 * @param path
	 */
	public void setPath(String path) {
		this.path = path;
	}

}
