/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.config;

import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

 /**
  * Base class for components exposed through RMI
  * @author Pavel Vlasov
  * @revision $Revision$
  */
public class RmiComponent extends UnicastRemoteObject implements Component, DomConfigurable {

	private int port;
	private String name;
	private String bindName;
	protected Object owner;

	public RmiComponent() throws RemoteException {
		super();
	}

	public void start() throws ConfigurationException {
		bindName = "//localhost:"+port+"/"+name;
		try {
			Naming.rebind(bindName, this);
		} catch (RemoteException e) {
			throw new ConfigurationException("Could not bind to name '"+bindName+"' - "+e, e);
		} catch (MalformedURLException e) {
			throw new ConfigurationException("Could not bind to name '"+bindName+"' - "+e, e);
		}
	}

	public void stop() throws ConfigurationException {
		if (bindName!=null) {
			try {
				Naming.unbind(bindName);
				unexportObject(this, false);
				bindName=null;
			} catch (RemoteException e) {
				throw new ConfigurationException("Could not unbind/unexport '"+bindName+"' - "+e, e);
			} catch (MalformedURLException e) {
				throw new ConfigurationException("Could not unbind name '"+bindName+"' - "+e, e);
			} catch (NotBoundException e) {
				throw new ConfigurationException("Could not unbind name '"+bindName+"' - "+e, e);
			}
		}
	}

	public void setOwner(Object owner) {
		this.owner=owner;
	}

	public void configure(Node configNode, Context context) {
		Element ce = (Element) configNode;
		if (ce.hasAttribute("port")) {
			port=Integer.parseInt(ce.getAttribute("port"));
		}
		
		name=ce.getAttribute("name");
	}

}
