/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.config;

import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;

import javax.rmi.PortableRemoteObject;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * Binds remote object to name tree.
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public class RmiWrapper implements Wrapper, Component, DomConfigurable {
	private Object master;
	private Class narrowTo;
	private String url;

	public Object getMaster() {
		return master;
	}

	public void start() throws ConfigurationException {
		try {
			master=Naming.lookup(url);
			if (narrowTo!=null) {
				master=PortableRemoteObject.narrow(master, narrowTo);
			}
		} catch (MalformedURLException e) {
			throw new ConfigurationException("Could not lookup '"+url+"' - "+e, e);
		} catch (RemoteException e) {
			throw new ConfigurationException("Could not lookup '"+url+"' - "+e, e);
		} catch (NotBoundException e) {
			throw new ConfigurationException("Could not bind '"+url+"' - "+e, e);
		}
	}

	public void stop() {
		// Nothing
	}

	public void setOwner(Object owner) {
		// nothing

	}

	public void configure(Node configNode, Context context) throws ConfigurationException {
		Element ce = (Element) configNode;
		url=ce.getAttribute("url");
		
		if (ce.hasAttribute("narrow-to")) {
			try {
				narrowTo=Class.forName(ce.getAttribute("narrow-to"));
			} catch (ClassNotFoundException e) {
				throw new ConfigurationException("Narrow-to class not found: "+ce.getAttribute("narrow-to"), e);
			}
		}
	}

}
