/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.eval;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;


public class MethodEntry {
    Object object;
    Method method;
    String name;
    
    public MethodEntry(Object object, Method method) {
        super();
        this.object = object;
        this.method = method;
        this.name = method.getName();
    }
    
    public MethodEntry(Object object, Method method, String name) {
        super();
        this.object = object;
        this.method = method;
        this.name = name;
    }
    
    Object invoke(Object[] args) throws EvaluationException {
        try {
            return method.invoke(object, args);
        } catch (IllegalAccessException e) {
            throw new EvaluationException(e);
        } catch (InvocationTargetException e) {
            throw new EvaluationException(e);
        }
    }
    
    /**
     * @param otherEntry
     * @return 0 - neither is more specific, 1 - this one is more specific
     * -1 - the other one is more specific.
     */
    int isMoreSpecific(MethodEntry otherEntry) {
        Class[] otherTypes = otherEntry.method.getParameterTypes();
        Class[] types = method.getParameterTypes();
        if (otherTypes.length!=types.length) {
            return 0;
        }
                    
        for (int i=0; i<types.length; i++) {
            if (types[i].equals(otherTypes[i])) {
                continue;
            } else if (types[i].isAssignableFrom(otherTypes[i])) {
                return -1;
            } else if (otherTypes[i].isAssignableFrom(types[i])) {
                return 1;
            }
        }
        
        return 0;
    }
}