package biz.hammurapi.jdbc;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;

import org.apache.commons.dbcp.BasicDataSource;

import biz.hammurapi.config.Component;
import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.jms.adapter.JmsAdapter;

/**
 * Invokes close() in stop() method.
 * @author Pavel
 *
 */
public class BasicDataSourceComponent extends BasicDataSource implements Component {

	public void setOwner(Object owner) {
		// Nothing to do		
	}

	public void start() throws ConfigurationException {
		// Nothing to do		
	}

	public void stop() throws ConfigurationException {
		try {
			super.close();
		} catch (SQLException e) {
			throw new ConfigurationException("Failed to close data source: "+e, e);
		}		
	}
	
	private String initConnectionStatement;
	
	/**
	 * Statement which is invoked every time connection is obtained from the data source.
	 * @param initConnectionStatement
	 */
	public void setInitConnectionStatement(String initConnectionStatement) {
		this.initConnectionStatement = initConnectionStatement;
	}
	
	public Connection getConnection() throws SQLException {
		Connection ret = super.getConnection();
		if (!JmsAdapter.isBlank(initConnectionStatement)) {
			Statement stmt = ret.createStatement();
			try {
				stmt.execute(initConnectionStatement);
			} finally {
				stmt.close();
			}
		}
		return ret;
	}
		
}
