/*
 * hgee 2
 * Hammurapi Group Enterprise Extensions 
 * Copyright (C) 2006  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/products/enterpriseextensions/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.jms;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Timer;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import biz.hammurapi.config.Component;
import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.metrics.Slice;
import biz.hammurapi.metrics.SliceConsumer;
import biz.hammurapi.xml.dom.DOMUtils;


/**
 * Collects slices for a period of time and send them
 * as JMS text message with XML payload.
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public abstract class JmsSliceConsumer implements SliceConsumer, Component {
	protected Timer timer;
	private boolean isOwnTimer;
	protected long interval;

	/**
	 * @param timer Timer to use for scheduling. If null than internal timer
	 * is created.
	 * @param interval Send interval.
	 */
	public JmsSliceConsumer(Timer timer, long interval) {
		super();
		this.timer=timer;
		this.interval=interval;
	}
	
	private Map slices=new HashMap();

	public boolean consumeSlice(String category, Slice slice) {
		synchronized (slices) {
			Collection bucket=(Collection) slices.get(category);
			if (bucket==null) {
				bucket=new ArrayList();
				slices.put(category, bucket);
			}
			bucket.add(slice);
			return true;
		}
	}
	
	/**
	 * Converts accumulated slices to XML. Clears internal slices map.
	 * @throws ParserConfigurationException 
	 * @throws TransformerException 
	 * @throws IOException 
	 * @return Slices in XML format.
	 */
	protected String getSlicesXml() throws IOException, TransformerException, ParserConfigurationException {
		synchronized (slices) {
			try {
				return DOMUtils.toXmlString(slices, "slices");
			} finally {
				slices.clear();
			}
		}
	}			

	public void start() throws ConfigurationException {
		if (timer==null) {
			timer=new Timer();
			isOwnTimer=true;
		}
		
	}

	public void stop() throws ConfigurationException {
		if (isOwnTimer) {
			timer.cancel();
		}		
	}

	public void setOwner(Object owner) {
		// Nothing		
	}

}
