/*
 * hgee 2
 * Hammurapi Group Enterprise Extensions 
 * Copyright (C) 2006  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/products/enterpriseextensions/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.jms;

import java.io.IOException;
import java.util.Timer;
import java.util.TimerTask;

import javax.jms.JMSException;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicPublisher;
import javax.jms.TopicSession;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import biz.hammurapi.config.ConfigurationException;

/**
 * Sends collected metrics to JMS topic.
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public class JmsTopicSliceConsumer extends JmsSliceConsumer {

	private Topic topic;
	private TopicConnection connection;

	/**
	 * @param timer Shared timer to schedule send. If null then internal timer is used.
	 * @param interval Send interval.
	 * @param connection JMS connection.
	 * @param topic JMS topic.
	 */
	public JmsTopicSliceConsumer(Timer timer, long interval, TopicConnection connection, Topic topic) {
		super(timer, interval);
		this.connection=connection;
		this.topic=topic;
	}
	
	private TimerTask sendTask=new TimerTask() {

		public void run() {
			try {
				TopicSession session=connection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
				try {
					TextMessage msg = session.createTextMessage();
					msg.setText(getSlicesXml());
					TopicPublisher publisher = session.createPublisher(topic);
					try {
						publisher.publish(msg);
					} finally {
						publisher.close();
					}
				} finally {
					session.close();
				}
			} catch (JMSException e) {
				System.err.println("[WARNING] Could not send metrics to topic: "+e);
				e.printStackTrace();
			} catch (IOException e) {
				System.err.println("[WARNING] Could not write slices to message: "+e);
				e.printStackTrace();
			} catch (TransformerException e) {
				System.err.println("[WARNING] Could not write slices to XML: "+e);
				e.printStackTrace();
			} catch (ParserConfigurationException e) {
				System.err.println("[WARNING] Could not write slices to XML: "+e);
				e.printStackTrace();
			}
			
		}
		
	};
	
	public void start() throws ConfigurationException {
		super.start();
		timer.schedule(sendTask, interval, interval);
	}
	
	public void stop() throws ConfigurationException {
		sendTask.cancel();
		// Send remaining metrics.
		sendTask.run();
		super.stop();
	}

}
