/*
  * hgee 2
 * Hammurapi Group Enterprise Extensions 
 * Copyright (C) 2006  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/products/enterpriseextensions/index.html
 * e-Mail: support@hammurapi.biz 
 */
package biz.hammurapi.jms;

import java.io.StringReader;
import java.io.StringWriter;
import java.net.URL;

import javax.jms.Destination;
import javax.jms.Message;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.xml.transform.Result;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import biz.hammurapi.config.ConfigurationException;

/**
 * 
 * @author Tatyana Konukova
 *
 */
public class StylingXmlMessageProcessor extends MessageProcessor {

	private static final String RESOURCE_PREFIX = "resource:";
	private String replyDestinationName;
	private Destination replyDestination;
	private String stylesheet;
	
	/**
	 * Creates result to receive transformed input.
	 * @return Result instance.
	 */
	protected Result createResult() {
		return new StreamResult(new StringWriter());
	}
	
	/**
	 * Processes result. If this method return value is not null then it is sent to reply destination.
	 * @param result
	 * @return
	 */
	protected String processResult(Result result) {
		return ((StringWriter) ((StreamResult) result).getWriter()).toString();
	}

	/**
	 * Parses request text, transforms and 
	 */
	protected void processMessage(Message request, Session session) {
		if (request instanceof TextMessage) {			
			try {
				Result result = createResult();
				Transformer transformer = templates==null ? transformerFactory.newTransformer() : templates.newTransformer();
				transformer.transform(new StreamSource(new StringReader(((TextMessage) request).getText())), result);
				String replyText = processResult(result);
				if (replyText!=null) {
					Session replySession = session==null ? borrowSession() : session;
					try {
						TextMessage msg = replySession.createTextMessage(replyText);
						msg.setJMSCorrelationID(request.getJMSMessageID());
						MessageProducer producer = replySession.createProducer(request.getJMSReplyTo()==null ? replyDestination : request.getJMSReplyTo());
						try {
							producer.send(msg);
						} finally {
							producer.close();
						}
					} finally {
						if (session==null) {
							releaseSession(replySession);
						}
					}					
				}
			} catch (Exception e) {
				logger.error(e, e.toString());
			}
		} else {
			logger.warn(request, "Not a text message");
		}
	}
	
	/**
	 * Front-end reply destination (queue or topic) name.
	 * 
	 * @param destinationName
	 */
	public void setReplyDestination(String destinationName) {
		this.replyDestinationName = destinationName;
	}
	
	/**
	 * Stylesheet URL. If it starts with <code>resource:</code> then stylesheet gets loaded from 
	 * classloader resource.
	 * @param stylesheet
	 */
	public void setStylesheet(String stylesheet) {
		this.stylesheet = stylesheet;
	}
	
	private Templates templates;
	protected TransformerFactory transformerFactory;

	public void start() throws ConfigurationException {
		super.start();
		try {
			if (replyDestinationName!=null) {
				replyDestination = (Destination) initialContext.lookup(replyDestinationName);
			}
			
			transformerFactory = TransformerFactory.newInstance();
			if (stylesheet!=null) {
				if (stylesheet.startsWith(RESOURCE_PREFIX)) {
					templates = transformerFactory.newTemplates(
							new StreamSource(
									getClass()
									.getClassLoader()
									.getResourceAsStream(
											stylesheet.substring(RESOURCE_PREFIX.length()))));
				} else {
					templates = transformerFactory.newTemplates(new StreamSource(new URL(stylesheet).openStream()));
				}
			}
		} catch (Exception e) {
			throw new ConfigurationException("Startup failed: "+e, e);
		}
	}

}
