package biz.hammurapi.jms.adapter;

import java.util.ArrayList;
import java.util.Collection;

/**
 * Broadcasts object to registered processors.
 * Adapter client code can lookup listener and register
 * a processor.
 * @author Pavel
 *
 */
public class BroadcastingProcessor implements Processor {
	
	private Collection processors = new ArrayList();
	
	public synchronized void addProcessor(Processor processor) {
		processors.add(processor);
	}

	public synchronized void removeProcessor(Processor processor) {
		processors.remove(processor);
	}
	
	/**
	 * Collects non-null returns from registered processors and returns them as collection
	 * Returns null if collection is empty.
	 */
	public Object process(Object obj) throws Exception {
		Collection ret = new ArrayList();
		Processor[] pa;
		synchronized (processors) {
			pa = (Processor[]) processors.toArray(new Processor[processors.size()]);
		}
		
		for (int i=0; i<pa.length; ++i) {
			Object r = pa[i].process(obj);
			if (r!=null) {
				ret.add(r);
			}
		}
		
		return ret.isEmpty() ? null : ret;
	}

}
