package biz.hammurapi.jms.adapter;

import java.util.Map;

import javax.jms.Message;
import javax.jms.Session;

/**
 * Binding components shall implement this interface.
 * @author Pavel
 */
public interface Converter {

	/**
	 * Converts message to Object.
	 * @param message Source message
	 * @param properties Configuration properties
	 * @return Object created from message data 
	 */
	Object convert(Message message, Map properties) throws Exception;
	
	/**
	 * Creates message and populates it from object.
	 * @param obj Source object
	 * @param session Session to create message
	 * @param properties Configuration properties
	 * @param request Request message, can be null.
	 * @return Populated and ready to send message
	 */
	Message convert(Object obj, Session session, Map properties, Message request) throws Exception;
	
	/**
	 * Creates message and populates it from exception to send it over JMS.
	 * @param obj Exception
	 * @param session Session to create message
	 * @param properties Configuration properties
	 * @param request Request message, can be null.
	 * @return Populated and ready to send message
	 */
	Message convert(Exception e, Session session, Map properties, Message request) throws Exception;
	
}
