package biz.hammurapi.jms.adapter;

import java.util.Hashtable;

import javax.jms.Destination;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.config.GenericContainer;
import biz.hammurapi.jms.adapter.definition.Context;

public class JndiContext extends GenericContainer implements DestinationResolver {

	private Context definition;

	public JndiContext(JmsAdapter jmsAdapter, biz.hammurapi.jms.adapter.definition.Context definition) throws ConfigurationException {
		this.definition = definition;
		biz.hammurapi.jms.adapter.definition.JndiConnection[] ca = definition.getConnectionArray();
		for (int i=0; i<ca.length; ++i) {
			addComponent(ca[i].getName(), new JndiConnection(jmsAdapter, ca[i]));
		}
	}
	
	private javax.naming.Context jndiContext;

	public void start() throws ConfigurationException {
		try {
			Hashtable properties = JmsAdapter.instantiate(definition.getPropertyArray());
			// TODO Prompt for credentials.
			jndiContext = new InitialContext(properties);
			if (!JmsAdapter.isBlank(definition.getRoot())) {
				jndiContext = (javax.naming.Context) jndiContext.lookup(definition.getRoot());
			}
		} catch (NamingException e) {
			throw new ConfigurationException(e);
		}
		super.start();
	}

	public Destination lookupDestination(String name) throws ConfigurationException {
		try {
			return (Destination) jndiContext.lookup(name);
		} catch (NamingException e) {
			throw new ConfigurationException("Destination lookup failed: "+e, e);
		}
	}
	
	public Object lookup(String name) throws ConfigurationException {
		try {
			return jndiContext.lookup(name);
		} catch (NamingException e) {
			throw new ConfigurationException("Lookup failed: "+e, e);
		}
	}
	
	/**
	 * Bridges to underlying JNDI if super.get() returns null.
	 */
	public Object get(String name) {
		Object ret = super.get(name);
		
		if (ret!=null) {
			return ret;
		}
		
		try {
			return jndiContext.lookup(name);
		} catch (NamingException e) {
			return null;
		}
	}

}
