package biz.hammurapi.jms.adapter;



/**
 * Local calls are routed to instances of this interface.
 * @author Pavel
 */
public interface LocalDelegate {

	/**
	 * Local state to be sent to JMS as part of method invocation.
	 * @return
	 */
	Object getState();
	
	/**
	 * @param state Local state.
	 */
	void setState(Object state);
	
	/**
	 * All proxy calls are routed to this method. For local calls the last parameter is null.
	 * LocalDelegate doesn't have to chain-invoke jmsMethod for remote calls. E.g. it may cache return 
	 * values from previous calls.
	 * @param proxy Proxy instance 
	 * @param method Method
	 * @param args Method arguments
	 * @param jmsMethod Remote method. Null for local method calls. If jmsMethod is asynchronous, local delgate
	 * can execute is synchronously or asynchronously at its discretion.
	 * @return Method return value.
	 * @throws Throwable
	 */
	public Object invoke(
			Object proxy, 
			java.lang.reflect.Method method, 
			Object[] args, 
			Method jmsMethod) throws Throwable;
		
}
