package biz.hammurapi.jms.adapter;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import biz.hammurapi.metrics.MeasurementCollector;
import biz.hammurapi.metrics.MeasurementConsumer;

public class MapLocalDelegate implements LocalDelegate, MeasurementCollector, MeasurementConsumer {
	
	private final static Logger logger = Logger.getLogger(MapLocalDelegate.class.getName());
	
	private Map state = new HashMap();

	private MeasurementConsumer measurementConsumer;

	public Object getState() {
		return state;
	}

	public Object invoke(Object proxy, final java.lang.reflect.Method method, final Object[] args, final Method jmsMethod) throws Throwable {
		if (jmsMethod == null) {
			if (method.getName().length()>3) {
				if (method.getName().startsWith("set") && method.getParameterTypes().length==1 && void.class.equals(method.getReturnType())) {
					state.put(method.getName().substring(3), args[0]);
					return null;
				}
				
				if (method.getName().startsWith("get") && method.getParameterTypes().length==0 && !void.class.equals(method.getReturnType())) {
					return state.get(method.getName().substring(3));
				}
			}
			return method.invoke(state, args);
		}
		
		if (jmsMethod!=null && jmsMethod.isAsync()) {
			jmsMethod.getJmsService().getWorker().post(new Runnable() {

				public void run() {
					try {
						jmsMethod.invoke(state, args, method.getDeclaringClass().getName());
					} catch (Throwable e) {
						String methodSignature = ProxyService.signature(method);
						addMeasurement(methodSignature.toString()+".error", 1, System.currentTimeMillis());
						logger.log(Level.SEVERE, "Invocation of "+methodSignature+" failed: "+e, e);
					}
				}
				
			});
			return null;
		} 

		return jmsMethod.invoke(state, args, method.getDeclaringClass().getName());
	}

	public void setMeasurementConsumer(MeasurementConsumer measurementConsumer) {
		this.measurementConsumer = measurementConsumer;		
	}

	public void addMeasurement(String name, double value, long time) {
		if (measurementConsumer!=null) {
			measurementConsumer.addMeasurement(name, value, time);
		}		
	}

	public void setState(Object state) {
		this.state = (Map) state;		
	}

}
