package biz.hammurapi.jms.adapter;

import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.config.ServiceBase;
import biz.hammurapi.remoting.Invocation;

public class Method extends ServiceBase {
	
	private biz.hammurapi.jms.adapter.definition.Method definition;

	private JmsService jmsService;
	
	public Method(biz.hammurapi.jms.adapter.definition.Method definition) throws ConfigurationException {
		this.definition = definition;
		if (definition.getService()!=null) {
			this.jmsService = new JmsService(definition.getService());
		}
	}
		
	private Class wrapperException;

	protected void startInternal() throws ConfigurationException {
		if (definition.getService()!=null) {
			jmsService.start();
		} else if (definition.getServiceRef()!=null) { 
			jmsService = (JmsService) get(definition.getServiceRef());
		} else {
			jmsService = ((ProxyService) getOwner()).getJmsService();
		}
		
		if (jmsService==null) {
			throw new ConfigurationException("Jms service for method "+definition.getName()+" cannot be located");
		}
				
		if (!JmsAdapter.isBlank(definition.getWrapperException())) {
			try {
				wrapperException = Class.forName(definition.getWrapperException());
			} catch (ClassNotFoundException e) {
				throw new ConfigurationException("Wrapper exception class not found: "+e, e);
			}
			
			if (!Exception.class.isAssignableFrom(wrapperException)) {
				throw new ConfigurationException("Wrapper exception class is not a subclass of java.lang.Exception");
			}
		}
		
	}
	
	public Class getWrapperException() {
		return wrapperException;
	}
	
	protected void stopInternal() throws ConfigurationException {
		if (definition.getService()!=null && jmsService!=null) {
			jmsService.stop();
		}
	}
	
	public boolean isAsync() {
		return definition.getAsynchronous();
	}
		
	/**
	 * Invokes remote method
	 * @param state
	 * @param args
	 * @return
	 */
	public Object invoke(Object state, Object[] args, String declaringClass) throws Exception {
		long start = System.currentTimeMillis();
		try {
			Invocation invocation = new Invocation(state, declaringClass, definition.getName(), definition.getParameterArray(), args, null);
			if (definition.getFireAndForget()) {
				jmsService.send(invocation, definition.getAsynchronous());
				return null;
			} else {
				return jmsService.request(invocation);
			}
		} finally {
			long now = System.currentTimeMillis();
			addMeasurement("request", now - start, now);
		}
	}
	
	public JmsService getJmsService() {
		return jmsService;
	}
}
