package biz.hammurapi.jms.adapter.converters;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Map;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.Session;
import javax.jms.TextMessage;

import biz.hammurapi.jms.adapter.Converter;

/**
 * Pass-through converter. Doesn't perform any conversion.
 * @author Pavel
 */
public class NopConverter implements Converter {

	/**
	 * Returns message.
	 */
	public Object convert(Message message, Map properties) throws JMSException {
		return message;
	}

	/**
	 * Returns object cast to message.
	 */
	public Message convert(Object obj, Session session, Map properties, Message request)	throws JMSException {
		return (Message) obj;
	}

	/**
	 * Returns text message with exception stack trace in the body. Correlates by message ID.
	 */
	public Message convert(Exception e, Session session, Map properties, Message request) throws JMSException {
		TextMessage ret = session.createTextMessage();
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);
		e.getCause().printStackTrace(pw);
		pw.close();
		try {
			sw.close();
			ret.setText(sw.toString());
		} catch (IOException ex) {
			ret.setText("Should never happen: "+ex);
		}
		
		if (request!=null) {
			ret.setJMSCorrelationID(request.getJMSMessageID());
		}
		return ret;
	}

}
