/*
  * hgee 2
 * Hammurapi Group Enterprise Extensions 
 * Copyright (C) 2006  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/products/enterpriseextensions/index.html
 * e-Mail: support@hammurapi.biz 
 */
package biz.hammurapi.jms.adapter.converters;

import java.io.Serializable;
import java.util.Map;

import javax.jms.Message;
import javax.jms.ObjectMessage;
import javax.jms.Session;

import biz.hammurapi.jms.adapter.Converter;

/**
 * Reads and writes objects from/to ObjectMessage.
 * @author Tatyana Konukova
 *
 */
public abstract class ObjectMessageConverter implements Converter {


	/**
	 * Reads object from ObjectMessage. If is-error boolean property is true then 
	 * return object gets cast to Exception and thrown.
	 */
	public Object convert(Message message, Map properties) throws Exception {
		Serializable ret = ((ObjectMessage) message).getObject();
		if (message.getBooleanProperty("is-error")) {
			throw (Exception) ret;
		}
		return ret;
	}

	/**
	 * Writes Object to ObjectMessage. Object shall be serializable.
	 * Correlates by message ID.
	 */
	public Message convert(Object obj, Session session, Map properties, Message request) throws Exception {
		ObjectMessage ret = session.createObjectMessage();
		ret.setObject((Serializable) obj);
		if (request!=null) {
			ret.setJMSCorrelationID(request.getJMSMessageID());
		}
		return ret;
	}

	/**
	 * Writes Exception to ObjectMessage. Exception shall be serializable.
	 * Boolean property is-error is set to true to indicate that payload is exception.
	 * Correlates by message ID.
	 */
	public Message convert(Exception ex, Session session, Map properties, Message request) throws Exception {
		ObjectMessage ret = session.createObjectMessage();
		ret.setObject((Serializable) ex);
		ret.setBooleanProperty("is-error", true);
		if (request!=null) {
			ret.setJMSCorrelationID(request.getJMSMessageID());
		}
		return ret;
	}

}
