package biz.hammurapi.jms.adapter.converters;

import javax.jms.JMSException;
import javax.xml.transform.Result;
import javax.xml.transform.Source;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import biz.hammurapi.config.DomConfigFactory;
import biz.hammurapi.xml.dom.DOMUtils;

/**
 * This class uses CompositeDomSerializer to convert
 * Java objects to XML and DomConfigFactory to convert
 * XML to Java objects. 
 * @author Pavel
 *
 */
public class SimpleXmlConverter extends DomConverter {
	
	/**
	 * Creates ojbect from result. In the case of exception tries to reproduce and throw the same exception
	 */
	protected Object processResult(Result result) throws Exception {
		Element documentElement = ((Document) super.processResult(result)).getDocumentElement();
		if ("error".equals(documentElement.getNodeName())) {
			if (documentElement.hasAttribute("type")) {
				Exception toThrow = null;				
				try {
					Class exceptionClass = Class.forName(documentElement.getAttribute("type"));
					if (Exception.class.isAssignableFrom(exceptionClass)) {
						if (documentElement.hasAttribute("message")) {
							toThrow = (Exception) exceptionClass 
								.getConstructor(new Class[] {String.class})
								.newInstance(new String[] {documentElement.getAttribute("message")});
						} else {
							toThrow = (Exception) exceptionClass.newInstance();
						}
					}
				} catch (Exception e) {
					toThrow = new JMSException("Remote exception "+documentElement.getAttribute("type")+": "+documentElement.getAttribute("message"));
				}					
				throw toThrow == null ? new JMSException("Remote exception "+documentElement.getAttribute("type")+": "+documentElement.getAttribute("message")) : toThrow;
			}
			throw new JMSException("Remote exception: "+documentElement.getAttribute("message"));
		}
		DomConfigFactory factory = new DomConfigFactory();
		return factory.create(documentElement);		
	}
	
	/**
	 * Serializes object to XML using composite dom serializer
	 */
	protected Source convert(Object request) throws Exception {
		Document doc = createDocument();
		Element root = doc.createElement("object");
		doc.appendChild(root);
		DOMUtils.toDom(request, root);
		return super.convert(doc);
	}
}
