package biz.hammurapi.jms.adapter.converters;

import java.io.StringReader;
import java.io.StringWriter;
import java.util.Map;

import javax.jms.Message;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.xmlbeans.XmlObject;

import biz.hammurapi.jms.XmlBeansSaxResult;

/**
 * Uses XmlBeans to bind XML to Java
 * @author Pavel
 *
 */
public class StylingXmlBeansConverter extends StylingXmlConverter {
	
	/**
	 * Request must be XmlObject or Invocation.
	 */
	protected Source convert(Object request) throws Exception {
		StringWriter sw = new StringWriter();
		((XmlObject) request).save(sw);
		sw.close();
		return new StreamSource(new StringReader(sw.toString()));
	}

	/**
	 * Creates stream result backed by string writer.
	 */
	protected Result createResult() {
		return new XmlBeansSaxResult();
	}

	/**
	 * Parses result to an instance of XmlObject.
	 */
	protected Object processResult(Result result) throws Exception {
		return ((XmlBeansSaxResult) result).getObject();
	}

	/**
	 * This method serializes exception using XML Beans and schema defined in <code>http://www.hammurapi.biz/invocation namespace</code>, file <code>invocation.xsd</code>
	 * This method supports <code>error-style</code> optional property - URL of stylesheet to apply after Exception -&gt; Conversion.
	 * If URL starts with <code>resource:</code> then it is loaded from classloader resource.</LI>
	 * This implementation uses ThrowableSerializer to convert exception to XML.
	 * Correlates by message ID.
	 */
	public Message convert(Exception ex, Session session, Map properties, Message request) throws Exception {
		biz.hammurapi.invocation.ExceptionDocument doc = biz.hammurapi.invocation.ExceptionDocument.Factory.newInstance(); 
		biz.hammurapi.invocation.Exception xex = doc.addNewException();
		XmlBeansConverter.exceptionToXml(ex, xex);
				
		StringWriter sw = new StringWriter();
		Transformer transformer = getTransformer((String) properties.get("error-style")); 
		if (transformer==null) {
			doc.save(sw);
		} else {
			transformer.transform(new DOMSource(doc.getDomNode()), new StreamResult(sw));
		}
		sw.close();
		
		TextMessage ret = session.createTextMessage();
		ret.setText(sw.toString());
		if (request!=null) {
			ret.setJMSCorrelationID(request.getJMSMessageID());
		}
		return ret;
	}
	
}
