package biz.hammurapi.jms.adapter.jca;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import biz.hammurapi.jms.adapter.ProxyService;

public class ConnectionImpl implements Connection {
		
	private boolean stateful;
	
	public ConnectionImpl(ManagedConnectionImpl managedConnection, boolean stateful, Object state) {
		this.managedConnection = managedConnection;
		init(stateful, state);
	}

	public synchronized void init(boolean stateful, Object state) {
		this.stateful = stateful;
		cache.clear();		
		if (stateful && state instanceof Map) {
			synchronized (cache) {
				Iterator it = ((Map) state).entrySet().iterator();
				while (it.hasNext()) {
					Map.Entry entry = (Map.Entry) it.next();
					String name = String.valueOf(entry.getKey());
					Object toCache = managedConnection.get(name);
					ProxyService.setProxyState(toCache, entry.getValue());
					cache.put(name, toCache);
				}
			}
		}
	}	
	
	public void close() {
		if (managedConnection!=null) {
			managedConnection.close();
		}
	}
		
	private Map cache = new HashMap();

	public void cleanup() {
		cache.clear();
	}

	private ManagedConnectionImpl managedConnection;

	public Object get(String name) {
		if (managedConnection == null) {
			throw new IllegalStateException("Connection is closed");
		}
		
		if (stateful) {
			synchronized (cache) {
				Object ret = cache.get(name);
				if (ret==null) {
					ret = managedConnection.get(name);
					cache.put(name, ret);
				}
				return ret;
			}
		}
		
		return managedConnection.get(name); 
	}

	public Object getState() {
		Map ret = new HashMap();
		synchronized (cache) {
			Iterator it= cache.entrySet().iterator();
			while (it.hasNext()) {
				Map.Entry entry = (Map.Entry) it.next();
				Object state = ProxyService.getProxyState(entry.getValue());
				if (state!=null) {
					ret.put(entry.getKey(), state);
				}
			}
		}
		return ret;
	}
	
}
