package biz.hammurapi.jms.adapter.jca;

import java.io.PrintWriter;
import java.util.Iterator;
import java.util.Set;
import java.util.logging.Logger;

import javax.resource.ResourceException;
import javax.resource.spi.ConnectionManager;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ManagedConnectionFactory;
import javax.resource.spi.ResourceAdapter;
import javax.resource.spi.ResourceAdapterAssociation;
import javax.security.auth.Subject;

public class ManagedConnectionFactoryImpl implements ManagedConnectionFactory, ResourceAdapterAssociation {
	
	private static final Logger logger = Logger.getLogger(ManagedConnectionFactoryImpl.class.getName());
	
	private ResourceAdapterImpl resourceAdapter ;

	private PrintWriter writer;
	
	public class JmsConnectionRequestInfo implements ConnectionRequestInfo {

		private boolean stateful;
		private Object state;
		
		public JmsConnectionRequestInfo(boolean stateful) {
			this.stateful=stateful;
		}
		
		public JmsConnectionRequestInfo(Object state) {
			this(true);
			this.state=state;
		}

		public boolean isStateful() {
			return stateful;
		}
		
		public Object getState() {
			return state;
		}
	}

	public Object createConnectionFactory(ConnectionManager connectionManager)	throws ResourceException {
		if (resourceAdapter == null) {
			throw new ResourceException("No reference to the resource adapter");
		}
		return new ConnectionFactoryImpl(this, connectionManager);
	}

	public Object createConnectionFactory() throws ResourceException {
		if (resourceAdapter == null) {
			throw new ResourceException("No reference to the resource adapter");
		}
		return new ConnectionFactoryImpl(this, null);
	}

	public synchronized ManagedConnection createManagedConnection(Subject subject, ConnectionRequestInfo cxRequestInfo) throws ResourceException {
		return new ManagedConnectionImpl(this, "JMS Adapter", "1.0", 0);
	}

	public ManagedConnection matchManagedConnections(Set connectionSet,	Subject subject, ConnectionRequestInfo cxRequestInfo) throws ResourceException {
		
		if (cxRequestInfo instanceof JmsConnectionRequestInfo) {
			Iterator iterator = connectionSet.iterator();
			while (iterator.hasNext()) {
				// Anything matches - we just re-initialize connection.
				ManagedConnectionImpl match = (ManagedConnectionImpl) iterator.next();
				Object connection = match.getConnection(subject, cxRequestInfo);
				if (connection instanceof ConnectionImpl) {
					JmsConnectionRequestInfo jcri = (JmsConnectionRequestInfo) cxRequestInfo;
					((ConnectionImpl) connection).init(jcri.isStateful(), jcri.getState());
					return match;
				}
			}
			return null;
		}

		if (cxRequestInfo==null) {
			throw new ResourceException("Request info is null");
		}

		throw new ResourceException("Unrecognized request info type: "+cxRequestInfo.getClass());
	}
	

	/**
	 * @see ManagedConnectionFactory#setLogWriter(PrintWriter)
	 */
	public void setLogWriter(PrintWriter writer) throws ResourceException {
		this.writer = writer;
	}

	/**
	 * @see ManagedConnectionFactory#getLogWriter()
	 */
	public PrintWriter getLogWriter() throws ResourceException {
		return writer;
	}

	/**
	 * Two factories are equal if they belong to the same adapter
	 */
	public boolean equals(Object other) {
		if (other==this) {
			return true;
		}
		
		if (other instanceof ManagedConnectionFactoryImpl) {
			return resourceAdapter == ((ManagedConnectionFactoryImpl) other).resourceAdapter;
		}
		return false;
	}

	public int hashCode() {
		return resourceAdapter==null ? super.hashCode() : resourceAdapter.hashCode();
	}

	/**
	 * @return Returns the resourceAdapter.
	 */
	public ResourceAdapter getResourceAdapter() {
		return resourceAdapter;
	}
	/**
	 * @param resourceAdapter The resourceAdapter to set.
	 */
	public void setResourceAdapter(ResourceAdapter resourceAdapter) {
		if (resourceAdapter instanceof ResourceAdapterImpl) {
			this.resourceAdapter = (ResourceAdapterImpl) resourceAdapter;
		} else {
			logger.severe("Resource adapter object handed from the App server is not of expected type "+ResourceAdapterImpl.class);
			this.resourceAdapter = null ;
		}
			
	}

	SharedAdapter getSharedAdapter() {
		return resourceAdapter.getSharedAdapter();
	}
}