package biz.hammurapi.jms.adapter.jca;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.resource.NotSupportedException;
import javax.resource.ResourceException;
import javax.resource.spi.ConnectionEvent;
import javax.resource.spi.ConnectionEventListener;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.LocalTransaction;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ManagedConnectionMetaData;
import javax.security.auth.Subject;
import javax.transaction.xa.XAResource;

import biz.hammurapi.jms.adapter.jca.ManagedConnectionFactoryImpl.JmsConnectionRequestInfo;

public class ManagedConnectionImpl implements ManagedConnection {

	private static final String TRANSACTIONS_NOT_SUPPORTED_MESSAGE =	"Transactions not supported";
	
	
	private ConnectionImpl connection;
	private Collection listeners = new ArrayList();
	private PrintWriter out;
	private String productName;
	private String version;
	private int maxConnections;
	private ManagedConnectionFactoryImpl factory;


	public ManagedConnectionImpl(ManagedConnectionFactoryImpl factory, String productName, String version, int maxConnections) throws ResourceException {
		super();
		this.factory = factory;
		this.productName = productName;
		this.version = version;
		this.maxConnections = maxConnections;
	}
	
	ManagedConnectionFactoryImpl getFactory() {
		return factory;
	}

	/**
	 * @see ManagedConnection#getConnection(Subject, ConnectionRequestInfo)
	 */
	public Object getConnection(Subject subject, ConnectionRequestInfo cxRequestInfo) throws ResourceException {
		if (cxRequestInfo==null) {
			throw new ResourceException("Request info is null");
		}

		if (cxRequestInfo instanceof JmsConnectionRequestInfo) {
			JmsConnectionRequestInfo jcri = (JmsConnectionRequestInfo) cxRequestInfo;
			if (connection==null) {
				connection = new ConnectionImpl(this, jcri.isStateful(), jcri.getState());
			} else {
				((ConnectionImpl) connection).init(jcri.isStateful(), jcri.getState());
			}
			return connection;
		}

		throw new ResourceException("Unrecognized request info type: "+cxRequestInfo.getClass()+", expected "+JmsConnectionRequestInfo.class);
	}

	/**
	 * @see ManagedConnection#destroy()
	 */
	public synchronized void destroy() throws ResourceException {
		connection = null;
		listeners.clear();
	}
	
	public synchronized void close() {
		Iterator it = listeners.iterator();
		ConnectionEvent event =	new ConnectionEvent(this, ConnectionEvent.CONNECTION_CLOSED);
		event.setConnectionHandle(connection);
		while (it.hasNext()) {
			ConnectionEventListener listener = (ConnectionEventListener) it.next();
			(listener).connectionClosed(event);
		}
		
		if (sharedAdapter!=null) {
			sharedAdapter.close();
			sharedAdapter = null;
		}
	}

	/**
	 * @see ManagedConnection#associateConnection(Object)
	 */
	public void associateConnection(Object connection) throws ResourceException {
		this.connection = (ConnectionImpl) connection;
	}

	/**
	 * @see ManagedConnection#addConnectionEventListener(ConnectionEventListener)
	 */
	public synchronized void addConnectionEventListener(ConnectionEventListener listener) {
		listeners.add(listener);
	}

	/**
	 * @see ManagedConnection#removeConnectionEventListener(ConnectionEventListener)
	 */
	public synchronized void removeConnectionEventListener(ConnectionEventListener listener) {
		listeners.remove(listener);
	}

	/**
	 * @see ManagedConnection#getXAResource()
	 */
	public XAResource getXAResource() throws ResourceException {
		throw new NotSupportedException(TRANSACTIONS_NOT_SUPPORTED_MESSAGE);
	}

	/**
	 * @see ManagedConnection#getLocalTransaction()
	 */
	public LocalTransaction getLocalTransaction() throws ResourceException {
		throw new NotSupportedException(TRANSACTIONS_NOT_SUPPORTED_MESSAGE);
	}

	/**
	 * @see ManagedConnection#getMetaData()
	 */
	public ManagedConnectionMetaData getMetaData() throws ResourceException {
		return new ManagedConnectionMetaDataImpl(productName, version, maxConnections, "(unknown)");
	}

	/**
	 * @see ManagedConnection#setLogWriter(PrintWriter)
	 */
	public void setLogWriter(PrintWriter out) throws ResourceException {
		this.out = out;
	}

	/**
	 * @see ManagedConnection#getLogWriter()
	 */
	public PrintWriter getLogWriter() throws ResourceException {
		return out;
	}

	public void cleanup() throws ResourceException {
		// Nothing to do.		
	}
	
	private SharedAdapter sharedAdapter;

	public Object get(String name) {
		synchronized (this) {
			if (sharedAdapter == null) {
				sharedAdapter = factory.getSharedAdapter();
			}
		}
		return sharedAdapter.get(name);
	}	
}