/*
  * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz 
 */
package biz.hammurapi.legacy.persistence;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * 
 * @author Pavel Vlasov
 * @version $Revision: 1.1 $
 */
public class CompositeStorage implements Storage {
	private List storages=new ArrayList();
	
	private class StorageEntry {
		String key;
		Storage storage;
		
		public boolean equals(Object obj) {
			return obj instanceof StorageEntry && ((StorageEntry) obj).key.equals(key);
		}
	}
	
	public void addStorage(String key, Storage storage) {
		if (key==null) {
			throw new NullPointerException("key==null");
		}
		
		if (storage==null) {
			throw new NullPointerException("storage==null");
		}
		
		StorageEntry se=new StorageEntry();
		se.key=key;
		se.storage=storage;
		storages.remove(se);
		storages.add(se);
	}

	public String put(Object o) throws PersistenceException {
		Iterator it=storages.iterator();
		while (it.hasNext()) {
			StorageEntry se=(StorageEntry) it.next();
			String key=se.storage.put(o);
			if (key!=null) {
				return se.key+":"+key;
			}
		}
		return null;
	}

	public Object get(String key) throws PersistenceException {
		int idx=key.indexOf(':');
		if (idx==-1) {
			throw new PersistenceException("Invalid key format");
		} else {
			String skey=key.substring(0, idx);
			Iterator it=storages.iterator();
			while (it.hasNext()) {
				StorageEntry se=(StorageEntry) it.next();
				if (skey.equals(se.key)) {
					return se.storage.get(key.substring(idx+1));
				}
			}
		}
		return null;
	}

	public void remove(String key) throws PersistenceException {
		int idx=key.indexOf(':');
		if (idx==-1) {
			throw new PersistenceException("Invalid key format");
		} else {
			String skey=key.substring(0, idx);
			Iterator it=storages.iterator();
			while (it.hasNext()) {
				StorageEntry se=(StorageEntry) it.next();
				if (skey.equals(se.key)) {
					se.storage.remove(key.substring(idx+1));
					return;
				}
			}
		}
	}
	
	public Storage getStorage(Class storageClass) {
		Iterator it=storages.iterator();
		while (it.hasNext()) {
			StorageEntry se=(StorageEntry) it.next();
			if (storageClass.isInstance(se.storage)) {
				return se.storage;
			}
		}
		return null;
	}
	
	public Storage getStorage(String storageKey) {
		Iterator it=storages.iterator();
		while (it.hasNext()) {
			StorageEntry se=(StorageEntry) it.next();
			if (storageKey.equals(se.key)) {
				return se.storage;
			}
		}
		return null;		
	}
}
