/*
  * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.logging;

/**
 * @author Pavel Vlasov	
 * @version $Revision: 1.2 $
 */
public class ConsoleLogger implements Logger {
	public static final int DEBUG=0;
	public static final int VERBOSE=1;
	public static final int INFO=2;
	public static final int WARN=3;
	public static final int ERROR=4;
	
	private int level;
	
	/**
	 * Default constructor with INFO level.
	 */
	public ConsoleLogger() {
		level=INFO;
	}
	
	public ConsoleLogger(String level) {
		if ("DEBUG".equalsIgnoreCase(level)) {
			this.level=DEBUG;
		} else if ("VERBOSE".equalsIgnoreCase(level)) {
			this.level=VERBOSE;
		} else if ("INFO".equalsIgnoreCase(level)) {
			this.level=INFO;
		} else if ("WARN".equalsIgnoreCase(level)) {
			this.level=WARN;
		} else if ("ERROR".equalsIgnoreCase(level)) {
			this.level=ERROR;
		} else {
			throw new IllegalArgumentException("Invalid level: "+level);
		}
	
		
	}
	
	public ConsoleLogger(int level) {
		if (level<DEBUG || level>ERROR) {
			throw new IllegalArgumentException("Invalid level: "+level);
		}
		this.level=level;
	}

	public void debug(Object source, String message) {
		if (level<=DEBUG) {
			System.out.println("[DEBUG] "+source+" "+message);
		}
	}

	public void verbose(Object source, String message) {
		if (level<=VERBOSE) {
			System.out.println("[VERBOSE] "+source+" "+message);
		}
	}

	public void info(Object source, String message) {
		if (level<=INFO) {
			System.out.println("[INFO] "+source+" "+message);
		}
	}

	public void warn(Object source, String message) {
		if (level<=WARN) {
			System.out.println("[WARN] "+source+" "+message);
			if (source instanceof Throwable) {
				((Throwable) source).printStackTrace();
			}
		}
	}

	public void error(Object source, String message) {
		if (level<=ERROR) {
			System.out.println("[ERROR] "+source+" "+message);
			if (source instanceof Throwable) {
				((Throwable) source).printStackTrace();
			}
		}
	}
}
