/*
  * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.metrics;

import java.util.Collection;

/**
 * @author Pavel Vlasov	
 * @version $Revision: 1.5 $
 */
public interface Metric {
	public interface Measurement {
		double getValue();
		long getTime();
	}
	
	/**
	 * @return number of measurements
	 */
	int getNumber();
	
	/**
	 * @return Minimum value
	 */
	double getMin();
	
	/**
	 * @return Maximum value
	 */
	double getMax();
	
	/**
	 * @return Average
	 */
	double getAvg();
	
	/**
	 * @return Sum of values from all measurements
	 */
	double getTotal();
	
	/**
	 * @return average distance from average
	 */
	double getDeviation();
	
	/**
	 * Adds measurement 
	 * @param source Source object
	 * @param value value
	 */
	void add(double value, long time);
	
	/**
	 * Adds another metric (aggregates)
	 * @param metric
	 */
	void add(Metric metric);
	
	/**
	 * @return Collection of {@link Measurement}
	 */
	Collection getMeasurements();
	
	String getName();
}