package biz.hammurapi.metrics;

import java.lang.reflect.Method;
import java.util.Stack;

import org.codehaus.aspectwerkz.joinpoint.MethodSignature;
import org.codehaus.aspectwerkz.joinpoint.StaticJoinPoint;

import biz.hammurapi.metrics.MeasurementCategory;
import biz.hammurapi.metrics.MeasurementCategoryFactory;

/**
 * Metrics collecting aspect.
 * @author Pavel
 */
public class MetricsAspect {
	private static MeasurementCategory mc = MeasurementCategoryFactory
			.getCategory(MetricsAspect.class);

	/**
	 * Call stack
	 */
	private ThreadLocal calls = new ThreadLocal() {

		protected Object initialValue() {
			return new Stack();
		}

	};

	/**
	 * This method pushes method start time to a thread local stack.
	 * @param joinPoint
	 */
	public void before(StaticJoinPoint joinPoint) {
		((Stack) calls.get()).push(new Long(System.currentTimeMillis()));
	}

	/**
	 * This method pops method start time from the thread local stack and reports method duration measurement.
	 * @param joinPoint
	 */
	public void after(StaticJoinPoint joinPoint) {
		Method method = ((MethodSignature) joinPoint.getSignature()).getMethod();
		Stack tlStack = (Stack) calls.get();
		if (!tlStack.isEmpty()) {
			Long start = (Long) (tlStack).pop();
	
			if (start != null) {	
				long now = System.currentTimeMillis();	
				mc.addMeasurement(method.toString(), now - start.longValue(), now);	
			}
		}
	}

}
