/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */

package biz.hammurapi.metrics;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

/**
 *
 * @author  Pavel Vlasov
 * @version $Revision: 1.8 $
 */
public class SimpleMetric implements Metric, Comparable, Serializable {
	/**
	 * Comment for <code>serialVersionUID</code>
	 */
	private static final long serialVersionUID = -3636560361707914006L;
	private String name;
	private boolean keepMeasurements;

	public SimpleMetric(String name) {
		this.name=name;
	}
	
	public SimpleMetric(String name, boolean keepMeasurements) {
		this.name=name;
		this.keepMeasurements=keepMeasurements;
	}
	
    private int measurements;
    private double total;
    private double min;
    private double max;
    private double deviation;
    
    public int getNumber() {
        return measurements;
    }
    
    public double getMin() {
        return min;
    }
    
    public double getMax() {
        return max;
    }
    
    public double getAvg() {
        return total/measurements;
    }
    
    public double getTotal() {
        return total;
    }
    
    public void add(final double value, final long time) {
        addMeasurement(new Measurement() {
			public double getValue() {
				return value;
			}
		
			public long getTime() {
				return time;
			}        	
		});
    }
    
    /**
	 * @param value
	 * @param measurement
	 */
	protected void addMeasurement(Measurement measurement) {
		if (measurements==0 || measurement.getValue()<min) {
            min=measurement.getValue();
        }
        
        if (measurements==0 || measurement.getValue()>max) {
            max=measurement.getValue();
        }
        
        measurements++;        
        total+=measurement.getValue();        
        deviation+=Math.abs(measurement.getValue()-total/measurements);
        
        if (keepMeasurements) {
			measurementsList.add(measurement);
        }
	}

	/**
	 * @param value
	 * @param time
	 * @return
	 */
	protected Measurement newMeasurement(final double value, final long time) {
		return new Measurement() {
			public double getValue() {
				return value;
			}

			public long getTime() {
				return time;
			}        	
		};
	}

	public void add(Metric metric) {
        if (measurements==0 || metric.getMin()<min) {
            min=metric.getMin();
        }
        
        if (measurements==0 || metric.getMax()>max) {
            max=metric.getMax();
        }
        
        measurements+=metric.getNumber();
        total+=metric.getTotal();
        deviation+=metric.getDeviation()*metric.getNumber();
        
        if (keepMeasurements) {
        	measurementsList.addAll(metric.getMeasurements());
        }
    }
    
    private List measurementsList=new ArrayList();

	public Collection getMeasurements() {
		return measurementsList;
	}

	public String getName() {
		return name;
	}    
	
	public String toString() {
		StringBuffer ret=new StringBuffer();
		ret.append(measurements)
			.append("/")
			.append(total)
			.append(" <= ")
			.append(name)
			.append("\n");
		Iterator it=measurementsList.iterator();
		while (it.hasNext()) {
			Measurement m=(Measurement) it.next();
			ret.append("\t");
			ret.append(m.getValue());
			ret.append(" <- ");
			ret.append("\n");
		}
		return ret.toString();
	}

	public int compareTo(Object o) {
		if (o instanceof Metric) {
			return Double.compare(((Metric) o).getTotal(), total);  
		}
		
		return 1;
	}

    public double getDeviation() {
        return deviation/measurements;
    }

}
