/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.metrics;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.xml.transform.TransformerException;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import biz.hammurapi.config.Component;
import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.config.Context;
import biz.hammurapi.config.DomConfigFactory;
import biz.hammurapi.config.DomConfigurable;
import biz.hammurapi.xml.dom.DOMUtils;
import biz.hammurapi.xml.dom.AbstractDomObject;
import biz.hammurapi.xml.dom.DOMUtils;


/**
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public class SlicingMeasurementCategoryFactory extends MeasurementCategoryFactory implements Component, DomConfigurable {
    private SlicingMeasurementConsumer consumer;
    
    public void start() throws ConfigurationException {        
        try {           
            NodeList nl = DOMUtils.selectNodeList(configElement, "category");
			for (int i=0, l=nl.getLength(); i<l; ++i) {
				Element ce = (Element) nl.item(i);
            	categories.add(DOMUtils.getElementText(ce));
            }
            
            String tickValue=AbstractDomObject.getElementText(configElement, "tick");
            long tick = tickValue==null ? 60000 : Long.parseLong(tickValue);
            
            String kmValue=AbstractDomObject.getElementText(configElement, "keep-measurements");
            boolean keepMeasurements = kmValue==null ? false : "yes".equalsIgnoreCase(kmValue);
            
            String maxQueueValue=AbstractDomObject.getElementText(configElement, "max-queue");
            int maxQueue = maxQueueValue==null ? 1000 : Integer.parseInt(maxQueueValue);
            
			consumer=createMeasurementConsumer(tick, keepMeasurements, maxQueue);
            consumer.start();
        } catch (Exception e) {
            throw new ConfigurationException(e);
		}
    }

    /**
     * Creates a consumer using DomConfigFactory 
	 * @param cxpa
	 * @param factory
	 * @return
	 * @throws ConfigurationException
	 * @throws TransformerException
	 */
	protected SliceConsumer createSliceConsumer() throws ConfigurationException {
		try {
			return (SliceConsumer) new DomConfigFactory(getClass().getClassLoader()).create(DOMUtils.selectSingleNode(configElement, "slice-consumer"));
		} catch (Exception e) {
			throw new ConfigurationException(e);
		}
	}

	/**
     * Override this method to create a custom consumer.
     * @param tick
     * @param keepMeasurements
     * @param maxQueue
     * @param sliceConsumer
     * @return
	 * @throws ConfigurationException
     */
	protected SlicingMeasurementConsumer createMeasurementConsumer(long tick, boolean keepMeasurements, int maxQueue) throws ConfigurationException {
		return new SlicingMeasurementConsumer (tick, keepMeasurements, maxQueue, createSliceConsumer());
	}

	public void stop() throws ConfigurationException {
        if (consumer!=null) {
            consumer.shutdown();
        }
    }
    
    private List categories=new ArrayList();
	protected Element configElement;
    
	public void configure(Node configNode, Context context) throws ConfigurationException {
		configElement=(Element) configNode;
	}

	public MeasurementConsumer getMeasurementConsumer(String categoryName) {
	    if (categories.isEmpty() || categories.contains(categoryName)) {
	        return consumer.getCategoryInstance(categoryName);
	    }
	    
	    Iterator it=categories.iterator();
	    while (it.hasNext()) {
	    	if (categoryName.startsWith(it.next()+".")) {
		        return consumer.getCategoryInstance(categoryName);    			    		
	    	}
	    }
	    
	    return null;
	}

	public void setOwner(Object owner) {
		// Ignore		
	}	
}
