package biz.hammurapi.metrics;

import java.io.ObjectOutputStream;
import java.net.Socket;
import java.util.List;

import biz.hammurapi.config.ConfigurationException;

/**
 * Sends slices to remote slice consumer server.
 * @author Pavel Vlasov
 */
public class SocketSliceConsumer extends BatchingSliceConsumer {
	
	public static final int DEFAULT_PORT = 9814;
	private static final String LOCALHOST = "localhost";
	
	private String address = LOCALHOST;
	private int port = DEFAULT_PORT;
	private String id = Long.toString(System.currentTimeMillis(), Character.MAX_RADIX);
	
	/**
	 * Sets server address. Default is "localhost"
	 * @param address
	 */
	public void setAddress(String address) {
		this.address = address;
	}
	
	/**
	 * Sets server port. Default is 9814
	 * @param port
	 */
	public void setPort(int port) {
		this.port = port;
	}
	
	/**
	 * Sets client ID to differentiate it from other clients.
	 * Defaults to 
	 * @param id
	 */
	public void setId(String id) {
		this.id = id;
	}
	
	public void start() throws ConfigurationException {
		super.start();
		System.out.println("[SocketSliceConsumer] ID="+id);
	}
	
	/**
	 * Sends slices to a server socket.
	 */
	protected boolean processSlices(List slices) {
		try {
			Socket socket = new Socket(address, port);
			ObjectOutputStream oos = new ObjectOutputStream(socket.getOutputStream());
			oos.writeObject(id);
			oos.writeObject(slices);
			oos.close();
			socket.close();
			return true;
		} catch (Exception e) {
			System.err.println("[Socket slice consumer] ERROR: Could not send slices to the server: "+e);
			return false;
		}				
	}

}
