/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.metrics;

import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import biz.hammurapi.RuntimeException;
import biz.hammurapi.config.Component;
import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.render.RenderRequest;
import biz.hammurapi.render.RenderingException;
import biz.hammurapi.xml.dom.DOMUtils;
import biz.hammurapi.xml.dom.DomSerializable;


/**
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public class XmlMeasurementCategoryFactory extends MeasurementCategoryFactory implements Component, DomSerializable {
	private Map categories=new TreeMap();

	public MeasurementConsumer getMeasurementConsumer(String categoryName) {
		synchronized (categories) {
			MeasurementConsumer ret=(MeasurementConsumer) categories.get(categoryName);
			if (ret==null) {
				ret=new SimpleMeasurementConsumer();
				categories.put(categoryName, ret);
			}
			return ret;
		}
	}

	protected File out;
	private long from;

	/**
	 * 
	 * @param out Output file.
	 */
	public XmlMeasurementCategoryFactory(File out) {
		this.out=out;
	}
	
	/**
	 * 
	 */
	public void start() throws ConfigurationException {
		from=System.currentTimeMillis();
	}

	/**
	 * Saves collected metrics to XML.
	 */
	public void stop() throws ConfigurationException {
		try {
			Document doc=DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
			Element root=doc.createElement("metrics");
			doc.appendChild(root);
			toDom(root);
			DOMUtils.serialize(doc, out);
		} catch (IOException e) {
			throw new ConfigurationException(e);
		} catch (TransformerException e) {
			throw new ConfigurationException(e);
		} catch (ParserConfigurationException e) {
			throw new ConfigurationException(e);
		} catch (FactoryConfigurationError e) {
			throw new ConfigurationException(e);
		} catch (DOMException e) {
			throw new ConfigurationException(e);
		}
	}

	public void toDom(Element holder) {
		holder.setAttribute("from", new Date(from).toString());
		holder.setAttribute("to", new Date().toString());
		Iterator it=categories.entrySet().iterator();
		while (it.hasNext()) {
			Map.Entry entry=(Map.Entry) it.next();
			MetricSourceRenderer mcr = new MetricSourceRenderer(new RenderRequest(entry.getValue()));
			try {
				Element element = mcr.render(holder.getOwnerDocument());
				element.setAttribute("category", (String) entry.getKey());
				holder.appendChild(element);
			} catch (RenderingException e) {
				throw new RuntimeException(e);
			}
		}
	}

	public void setOwner(Object owner) {
		// Ignore		
	}	
}
