/*
 * hgee 2
 * Hammurapi Group Enterprise Extensions 
 * Copyright (C) 2006  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/products/enterpriseextensions/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.metrics.j2ee;

import java.io.IOException;
import java.io.Serializable;
import java.io.StringWriter;
import java.util.Collection;
import java.util.Properties;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.ObjectMessage;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.stream.StreamResult;
import javax.xml.xpath.XPathExpressionException;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.config.Context;
import biz.hammurapi.config.DomConfigurable;
import biz.hammurapi.metrics.BatchingSliceConsumer;
import biz.hammurapi.xml.dom.AbstractDomObject;
import biz.hammurapi.xml.dom.DOMUtils;



abstract class AbstractJmsSliceConsumer extends BatchingSliceConsumer implements DomConfigurable {
    private boolean xmlFormat;
    protected Element configElement;
	protected String root;
    
	public void configure(Node configNode, Context context) throws ConfigurationException {
		try {
			Element formatElement=(Element) DOMUtils.selectSingleNode(configNode, "format");
			xmlFormat="xml".equalsIgnoreCase(DOMUtils.getElementText(formatElement));
            root=AbstractDomObject.getElementText(configElement, "root-category");
            String intervalStr = AbstractDomObject.getElementText(configElement, "interval");
            if (intervalStr!=null) {
            	setInterval(Long.parseLong(intervalStr));
            }
            String maxSlicesStr = AbstractDomObject.getElementText(configElement, "max-slices");
            if (maxSlicesStr!=null) {
            	setMaxSlices(Integer.parseInt(maxSlicesStr));
            }
		} catch (Exception e) {
			throw new ConfigurationException(e);
		}
		configElement=(Element) configNode;
	}
	
	/**
	 * @param cxpa
	 * @return
	 * @throws TransformerException
	 * @throws ConfigurationException
	 * @throws NamingException
	 * @throws XPathExpressionException 
	 */
	protected InitialContext createContext() throws ConfigurationException, NamingException, XPathExpressionException {
		Properties ctxProps=new Properties();
		NodeList nl = DOMUtils.selectNodeList(configElement, "environment-property");
		for (int i=0, l=nl.getLength(); i<l; ++i) {
			Element e = (Element) nl.item(i);
			ctxProps.setProperty(e.getAttribute("name"), DOMUtils.getNonBlankElementText(e));
		}
		InitialContext jndiContext = new InitialContext(ctxProps);
		return jndiContext;
	}	
	
	/**
	 * @param session
	 * @return
	 * @throws JMSException
	 * @throws TransformerException
	 * @throws IOException
	 * @throws FactoryConfigurationError
	 * @throws ParserConfigurationException
	 */
	protected Message createMessage(Session session, Collection slices) throws JMSException, ParserConfigurationException, FactoryConfigurationError, IOException, TransformerException {
		if (xmlFormat) {
			StringWriter sw=new StringWriter();
			DOMUtils.serialize(slices, "slices", new StreamResult(sw));
			sw.close();
			
			TextMessage message = session.createTextMessage();
			
        	message.setStringProperty("root-category", root);
			message.setStringProperty("format", "xml");
			message.setText(sw.toString());

			return message;
		} else {
			ObjectMessage message = session.createObjectMessage();
			message.setObject((Serializable) slices);
        	message.setStringProperty("root-category", root);
			message.setStringProperty("format", "object");
			return message;
		}			
	}

}