/*
 * hgee 2
 * Hammurapi Group Enterprise Extensions 
 * Copyright (C) 2006  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/products/enterpriseextensions/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.metrics.j2ee;

import java.util.List;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.naming.InitialContext;

import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.xml.dom.AbstractDomObject;


/**
 * Use this consumer instead of Topic or Queue consumers with JMS 1.1 compatible providers. 
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public class DestinationSliceConsumer extends AbstractJmsSliceConsumer {
	Connection connection;
	Destination destination;

	public void start() throws ConfigurationException {
		try {
			InitialContext jndiContext = createContext();
			ConnectionFactory connectionFactory = (ConnectionFactory) jndiContext.lookup(AbstractDomObject.getElementText(configElement, "connection-factory"));
			destination = (Destination) jndiContext.lookup(AbstractDomObject.getElementText(configElement, "destination"));
			String user = AbstractDomObject.getElementText(configElement, "user");
			if (user==null) {
				connection = connectionFactory.createConnection(); 
			} else {
				connection = connectionFactory.createConnection(user, AbstractDomObject.getElementText(configElement, "password"));
			}
			jndiContext.close();
		} catch (Exception e) {
			throw new ConfigurationException(e);
		}
	}		

	protected boolean processSlices(List slices) {
		try {
	        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
	        try {
		        MessageProducer producer = session.createProducer(destination);
		        try {
		        	Message message = createMessage(session, slices);
					producer.send(message);
				} finally {
		        	producer.close();
		        }
	        } finally {
	        	session.close();
	        }
		} catch (Exception e) {
			return false;
		}
    	return true;
	}

	public void stop() throws ConfigurationException {
		try {
			connection.close();
		} catch (JMSException e) {
			throw new ConfigurationException();
		}
	}

	public void setOwner(Object owner) {
		// Ignore		
	}	
}