package biz.hammurapi.metrics.jmx;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.management.MBeanServer;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.StandardMBean;

import biz.hammurapi.metrics.MeasurementCategoryFactory;
import biz.hammurapi.metrics.MeasurementConsumer;
import biz.hammurapi.metrics.SimpleSlice;
import biz.hammurapi.metrics.Slice;

public abstract class JmxMeasurementCategoryFactory extends MeasurementCategoryFactory {
	private String category;
	
	public void setCategory(String category) {
		this.category = category;
	}
	
	private Map measurementConsumers = new HashMap();
	
	private static class JmxSliceImpl implements JmxSlice {
		
		private String name;
		private Slice master;

		public JmxSliceImpl(String name) {
			this.name = name;
			reset();
		}

		public double getAverage() {
			return master.getAvg();
		}

		public double getDeviation() {
			return master.getDeviation();
		}

		public Date getFrom() {
			return new Date(master.getFrom());
		}

		public double getMax() {
			return master.getMax();
		}

		public double getMin() {
			return master.getMin();
		}

		public int getNumber() {
			return master.getNumber();
		}

		public Date getTo() {
			return new Date(master.getTo());
		}

		public double getTotal() {
			return master.getTotal();
		}

		public void reset() {
			master = new SimpleSlice(name);
		}
		
		public Slice getMaster() {
			return master;
		}
		
	}

	public synchronized MeasurementConsumer getMeasurementConsumer(final String categoryName) {
		MBeanServer mBeanServer = getMBeanServer();
		if (mBeanServer!=null 
				&& (category==null || category.equals(categoryName) || (categoryName!=null && categoryName.startsWith(category+".")))) {
			MeasurementConsumer ret = (MeasurementConsumer) measurementConsumers.get(categoryName);
			if (ret==null) {								
				ret = new MeasurementConsumer() {
					private Map sliceMap = new HashMap();

					public synchronized void addMeasurement(String name, double value, long time) {
						JmxSliceImpl slice = (JmxSliceImpl) sliceMap.get(name);
						if (slice==null) {
							slice=new JmxSliceImpl(name);
							sliceMap.put(name, slice);
							
							try {
					            MBeanServer mbs = getMBeanServer();
					            if (mbs!=null) {
						            mbs.registerMBean(new StandardMBean(slice, JmxSlice.class), objectName(categoryName, name));
					            }
							} catch (Exception e) {
								System.err.println("[JmxMeasurementCategoryFactory] ERROR: "+e);
								return;
							}
						}
						slice.getMaster().add(value, time);
					}
					
				};				
			}
			return ret;		
		}
		return null;
	}
	
	/**
	 * This implementation tries to use Java 5 management features.
	 * Returns null if JVM is not of version 5.
	 * @return MBean server to register MBeans with
	 */
	protected abstract MBeanServer getMBeanServer();

	/**
	 * Constructs MBean name from category name and measurement name. 
	 * @param categoryName
	 * @param name
	 * @return
	 * @throws NullPointerException 
	 * @throws MalformedObjectNameException 
	 */
	protected ObjectName objectName(final String categoryName, String name) throws MalformedObjectNameException {
		return new ObjectName("biz.hammurapi.metrics:category="+categoryName+",name="+name);
	}

}
