/*
 * hgee 2
 * Hammurapi Group Enterprise Extensions 
 * Copyright (C) 2006  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/products/enterpriseextensions/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.metrics.persistent;

import java.io.File;
import java.io.IOException;
import java.io.OutputStream;

import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;

/**
 * @author Pavel Vlasov
 * @version $Revision: 1.2 $
 */
public class ChartHelper {
	
    /**
     * Output JPG.
     */
    public static final byte FORMAT_JPG=0;
    
    /**
     * Output PNG
     */
    public static final byte FORMAT_PNG=1;

    private byte outputFormat;

	private int height;

	private int width;


	/**
	 * Helper class.
	 * @param outputFormat FORMAT_JPG or FORMAT_PNG.
	 * @param width chart width
	 * @param height chart height
	 */
    public ChartHelper(
			byte outputFormat,
			int width,
			int height) {
        this.outputFormat=outputFormat;
        this.height=height;
        this.width=width;
    }

    /**
     * Outputs chart to file.
     * @param chart
     * @param outputFile
     * @throws IOException
     */
    public void output(JFreeChart chart, File outputFile) throws IOException {
    	if (chart!=null) {
	        switch (outputFormat) {
	        	case FORMAT_PNG:
	        		ChartUtilities.saveChartAsPNG(outputFile, chart, width, height );
	        		break;
	        	case FORMAT_JPG:
	        		ChartUtilities.saveChartAsJPEG(outputFile, chart, width, height );
	        		break;
	        	default:
	        		throw new IllegalStateException("Invalid output format: "+outputFormat);
	        }
    	}
    }

    /**
     * Outputs chart to stream
     * @param chart
     * @param out
     * @throws IOException
     */
    public void output(JFreeChart chart, OutputStream out) throws IOException {
    	if (chart!=null) {
	        switch (outputFormat) {
	        	case FORMAT_PNG:
	        		ChartUtilities.writeChartAsPNG(out, chart, width, height );
	        		break;
	        	case FORMAT_JPG:
	        		ChartUtilities.writeChartAsJPEG(out, chart, width, height );
	        		break;
	        	default:
	        		throw new IllegalStateException("Invalid output format: "+outputFormat);
	        }
    	}
    }
}
