/*
 * hgee 2
 * Hammurapi Group Enterprise Extensions 
 * Copyright (C) 2006  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/products/enterpriseextensions/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.metrics.persistent;

import java.io.File;
import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import biz.hammurapi.config.Component;
import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.config.Context;
import biz.hammurapi.config.DomConfigurable;
import biz.hammurapi.config.MapContext;
import biz.hammurapi.metrics.MeasurementConsumer;
import biz.hammurapi.metrics.MetricsException;
import biz.hammurapi.metrics.SlicingMeasurementCategoryFactory;
import biz.hammurapi.metrics.SlicingMeasurementConsumer;
import biz.hammurapi.sql.SQLProcessor;
import biz.hammurapi.xml.dom.DOMUtils;


/**
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public class HypersonicTmpPersistingSlicingMeasurementCategoryFactory extends SlicingMeasurementCategoryFactory implements Component, DomConfigurable {
	private File outputDir;
	private SlicingMeasurementConsumer consumer;
	private int slices;
	private int width;
	private int height;
	private Collection categories=new ArrayList();

	public MeasurementConsumer getMeasurementConsumer(String categoryName) {
	    if (categories.isEmpty() || categories.contains(categoryName)) {
	        return consumer.getCategoryInstance(categoryName);
	    }
	    
	    Iterator it=categories.iterator();
	    while (it.hasNext()) {
	    	if (categoryName.startsWith(it.next()+".")) {
		        return consumer.getCategoryInstance(categoryName);    			    		
	    	}
	    }
	    
	    return null;
	}

	public void start() throws ConfigurationException {
		if (outputDir.exists()) {
			if (!outputDir.isDirectory()) {
				throw new ConfigurationException(outputDir.getAbsolutePath()+" in not a directory");
			}
		} else {
			if (!outputDir.mkdirs()) {
				throw new ConfigurationException("Cannot create directory "+outputDir.getAbsolutePath());
			}
		}
		
		super.start();
	}

	protected SlicingMeasurementConsumer createMeasurementConsumer(
			long tick,
			boolean keepMeasurements, 
			int maxQueue)
			throws ConfigurationException {
		try {
			final String rootCategory = "Root category";
			final HypersonicTmpSliceConsumer sliceConsumer = new HypersonicTmpSliceConsumer(
							rootCategory, 
							Long.MAX_VALUE, 
							Long.MAX_VALUE);
			
			consumer=new SlicingMeasurementConsumer(
					tick, 
					false, 
					maxQueue, 
					sliceConsumer) {
								
				public void shutdown() {
					super.shutdown();
					
			        try {
						// Generation of html's
						SQLProcessor processor=new SQLProcessor(sliceConsumer.getDataSource(), new MapContext(new HashMap()));
						PeriodFactory factory=new PeriodFactory(processor);
						SyntheticPeriod root;
						if (categories.isEmpty()) {
							root=factory.getRootPeriod(from, to, slices, false);
						} else {
							MetricLocator locator=new MetricLocator(processor);
							Collection categoryIds=new ArrayList();
							Iterator it=categories.iterator();
							while (it.hasNext()) {
								Object id = locator.find(sliceConsumer.getRootId().intValue(), (String) it.next());
								if (id!=null) {
									categoryIds.add(id);
								}
							}
							
							int[] roots=new int[categoryIds.size()];
							it=categoryIds.iterator();
							for (int i=0; it.hasNext(); i++) {
								roots[i]=((Number) it.next()).intValue();
							}
							
							root=factory.newSyntheticPeriod(rootCategory, roots, from, to, slices, false);
						}
						
						PeriodVisualizer.generateHtmlDoc(root, outputDir, width, height);
												
						sliceConsumer.getDataSource().shutdown();
			        } catch (SQLException e) {
			            throw new MetricsException(e);
			        }					
				}
			};
			consumer.start();
			return consumer;
		} catch (ClassNotFoundException e) {
			throw new ConfigurationException(e);
		} catch (IOException e) {
			throw new ConfigurationException(e);
		} catch (SQLException e) {
			throw new ConfigurationException(e);
		}
	}

	public void configure(Node configNode, Context context) throws ConfigurationException {
		super.configure(configNode, context);
		try {
			outputDir=new File(DOMUtils.getSingleNonBlankElementText((Element) configNode, "output-dir")); 
			slices=Integer.parseInt(DOMUtils.getSingleNonBlankElementText((Element) configNode, "slices"));
			width=Integer.parseInt(DOMUtils.getSingleNonBlankElementText((Element) configNode, "width"));
			height=Integer.parseInt(DOMUtils.getSingleNonBlankElementText((Element) configNode, "height"));
		} catch (Exception e) {
			throw new ConfigurationException(e);
		}
	}

}
