/*
 * hgee 2
 * Hammurapi Group Enterprise Extensions 
 * Copyright (C) 2006  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/products/enterpriseextensions/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.metrics.persistent;

import java.sql.SQLException;
import java.util.Iterator;

import biz.hammurapi.sql.SQLProcessor;

/**
 * Normalizes metric names - eliminates duplicates.
 * @author Pavel Vlasov
 * @version $Revision$
 */
public class MetricNormalizer {
    private MetricsEngine engine;
    
    public MetricNormalizer(SQLProcessor processor) {
        engine=new MetricsEngine(processor);
    }
    
    MetricNormalizer(MetricsEngine engine) {
        this.engine=engine;
    }    
    
    public void normalize() throws SQLException {
    	for (boolean needToContinue=true; needToContinue;) {
    		needToContinue=false;
	    	Metric prev=null;
	    	Iterator it=engine.getMetricsToNormalize().iterator();
	    	while (it.hasNext()) {
	    		needToContinue=true;
	    		Metric metric=(Metric) it.next();
	    		if (prev!=null 
	    				&& (metric.getParent()==null ? prev.getParent()==null : metric.getParent().equals(prev.getParent())) 
						&& metric.getName().equals(prev.getName())) {
	    			
	    			engine.moveSlices(prev.getId(), metric.getId());
	    			engine.moveChildren(prev.getId(), metric.getId());
	    			engine.deleteMetric(metric.getId());
	    		} else {
	    			prev=metric;
	    		}
	    	}
    	}
    }
}
