/*
 * hgee 2
 * Hammurapi Group Enterprise Extensions 
 * Copyright (C) 2006  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/products/enterpriseextensions/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.metrics.persistent;

import java.sql.SQLException;
import java.util.Set;

import biz.hammurapi.cache.Cache;
import biz.hammurapi.cache.Entry;
import biz.hammurapi.cache.Producer;
import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.sql.SQLProcessor;
import biz.hammurapi.sql.SQLRuntimeException;


/**
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public class PeriodFactory {

	private MetricsEngine engine;
	private Cache cache;

	public PeriodFactory(SQLProcessor processor) {
		engine=new MetricsEngine(processor);
		cache=createCache();
	}
	
	MetricsEngine getEngine() {
		return engine;
	}
	
	/**
	 * Override this method to create caching period factory
	 * @return
	 */
	protected Cache createCache() {
		return null;
	}
	
	public Period getPeriod(int id, long from, long to, int slices, boolean shrink) throws SQLException {
		if (cache==null) {
			return new Period(this, id, from, to, slices, shrink);
		}
		
		return (Period) cache.get(new Period.Key(id, from, to, slices, shrink));
	}
	
	public SyntheticPeriod getRootPeriod(long from, long to, int slices, boolean shrink) throws SQLException {
		MetricLocator locator=new MetricLocator(engine);
		return newSyntheticPeriod("Root categories", locator.findRoots(), from, to, slices, shrink);
	}
	
	/**
	 * Producer to be used by cache.
	 * @return
	 */
	protected Producer getProducer() {
		return new Producer() {

			public Entry get(Object key) {
				try {
					final Period period=new Period(PeriodFactory.this, (Period.Key) key);
					
					return new Entry() {
	
						public long getExpirationTime() {
							return 0;
						}
	
						public long getTime() {
							return 0;
						}
	
						public Object get() {
							return period;
						}
						
					};
				} catch (SQLException e) {
					throw new SQLRuntimeException(e);
				}
			}

			public void addCache(Cache cache) {
				// TODO Auto-generated method stub
				
			}

			public Set keySet() {
				return null;
			}
			
		};
	}
	
	void register(Period period) {
		if (cache!=null) {
			cache.put(period.toKey(), period, System.currentTimeMillis(), 0);
		}
	}
	
	public SyntheticPeriod newSyntheticPeriod(String name, int[] ids, long from, long to, int slices, boolean shrink) throws SQLException {
		return new SyntheticPeriod(this, name, ids, from, to, slices, shrink);
	}
	
	/**
	 * Shuts down cache, if any.
	 *
	 */
	public void stop() throws ConfigurationException {
		if (cache!=null) {
			cache.stop();
		}
	}
}
