/*
 * hgee 2
 * Hammurapi Group Enterprise Extensions 
 * Copyright (C) 2006  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/products/enterpriseextensions/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.metrics.persistent;

import java.io.File;
import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;

import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.stream.StreamResult;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.xy.DefaultHighLowDataset;
import org.jfree.data.xy.OHLCDataset;

import biz.hammurapi.config.MapContext;
import biz.hammurapi.config.RuntimeConfigurationException;
import biz.hammurapi.metrics.MetricsException;
import biz.hammurapi.sql.SQLProcessor;
import biz.hammurapi.sql.hypersonic.HypersonicServerDataSource;
import biz.hammurapi.util.ClassTransformerFactory;
import biz.hammurapi.util.Visitor;


/**
 * @author Pavel Vlasov
 * @version $Revision: 1.2 $
 */
public class PeriodVisualizer {
	private AbstractPeriod period;
    
	/**
     * Constructor from array of metric names
     * @param processor Processor to retrieve metrics from the database
     * @param metrics Metrics to be retrieved
     * @param from Start of metrics period
     * @param to End of metrics period
     * @param cacheMode Cache mode
     * @param outputFormat FORMAT_PNG or FORMAT_JPG
     * @param width Chart width
     * @param height Chart height
     * @param maxSlices Maximum number of slices on chart. Series having more slices will be downsliced - adjacent slices will be merged. 
     * @throws SQLException
     */
    public PeriodVisualizer(AbstractPeriod period) {
        this.period=period;
    }
	
	private JFreeChart intencityChart;
	
	/**
	 * @return
	 */
	public JFreeChart intensityChart() {
		if (intencityChart==null) {
			TimeSeriesCollection tsc=new TimeSeriesCollection();
	        TimeSeries timeSeries = new TimeSeries(normalizeName(period.getName()), SliceTimePeriod.class);
			SliceTimePeriod sp=null;
	        for (int i=0; i<period.getSlices().length; i++) {
	            sp=new SliceTimePeriod(period.getSlices()[i], sp);
	            timeSeries.add(sp, period.getSlices()[i].getMeasurements());
	        }
	        
	        tsc.addSeries(timeSeries);
	        
	        intencityChart = ChartFactory.createXYBarChart(
	                "Intensity (measurements/slice)",  // Title
	                "Time",           // X-Axis label
	                true,
	                "Intensity",           // Y-Axis label
	                tsc,          // Dataset
	                PlotOrientation.VERTICAL,
	                true,                // Show legend
	                true,
	                true
	               );
		}
		
		return intencityChart;
	}
	
	private JFreeChart totalChart;
	
	/**
	 * @return
	 */
	public JFreeChart totalChart() {
		if (totalChart==null) {
			TimeSeriesCollection tsc=new TimeSeriesCollection();
	        TimeSeries timeSeries = new TimeSeries(normalizeName(period.getName()), SliceTimePeriod.class);
			SliceTimePeriod sp=null;
	        for (int i=0; i<period.getSlices().length; i++) {
	            sp=new SliceTimePeriod(period.getSlices()[i], sp);
	            timeSeries.add(sp, period.getSlices()[i].getTotal());
	        }
	        
	        tsc.addSeries(timeSeries);
	        
	        totalChart = ChartFactory.createXYBarChart(
	                "Total (sum of values per slice)",  // Title
	                "Time",           // X-Axis label
	                true,
	                "Total",           // Y-Axis label
	                tsc,          // Dataset
	                PlotOrientation.VERTICAL,
	                true,                // Show legend
	                true,
	                true
	               );
		}
		
		return totalChart;
	}
	
	private JFreeChart averageChart;
	
	private String normalizeName(String name) {
		return name.length()>25 ? "Category" : name;
	}
	
	/**
	 * @return
	 */
	public JFreeChart averageChart() {
		if (averageChart==null) {
			TimeSeriesCollection tsc=new TimeSeriesCollection();
	        TimeSeries timeSeries = new TimeSeries(normalizeName(period.getName()), SliceTimePeriod.class);
			SliceTimePeriod sp=null;
	        for (int i=0; i<period.getSlices().length; i++) {
	            sp=new SliceTimePeriod(period.getSlices()[i], sp);
	            timeSeries.add(sp, period.getSlices()[i].getTotal()/period.getSlices()[i].getMeasurements());
	        }
	        
	        tsc.addSeries(timeSeries);
	        
		    averageChart = ChartFactory.createTimeSeriesChart(
		            "Average",  // Title
		            "Time",     // X-Axis label
		            "Average",  // Y-Axis label
		            tsc,        // Dataset
		            true,       // Show legend
		            true,
		            true
		           );
		}
		
		return averageChart;
	}
	
	private JFreeChart childrenIntencityChart;
	
	/**
	 * @return
	 * @throws SQLException
	 */
	public JFreeChart childrenIntensityChart() throws SQLException {
		if (childrenIntencityChart==null && period.getSubCategories().size()>1) {
			TimeSeriesCollection tsc=new TimeSeriesCollection();
			Iterator it=period.getSubCategories().iterator();
	        while (it.hasNext()) {
	        	Period p=(Period) it.next();
		        TimeSeries timeSeries = new TimeSeries(period.getLabel(p.getId()), SliceTimePeriod.class);
				SliceTimePeriod sp=null;
		        for (int i=0; i<p.getSlices().length; i++) {
		            sp=new SliceTimePeriod(p.getSlices()[i], sp);
		            timeSeries.add(sp, p.getSlices()[i].getMeasurements());
		        }
		        tsc.addSeries(timeSeries);		        
	        }
	        
		    childrenIntencityChart = ChartFactory.createTimeSeriesChart(
		            "Intensity (measurements/time)",  // Title
		            "Time",     // X-Axis label
		            "Intensity",  // Y-Axis label
		            tsc,        // Dataset
		            true,       // Show legend
		            true,
		            true
		           );
		}
		
		return childrenIntencityChart;
	}
	
	private JFreeChart childrenTotalChart;

	/**
	 * @return
	 * @throws SQLException
	 */
	public JFreeChart childrenTotalChart() throws SQLException {
		if (childrenTotalChart==null && period.getSubCategories().size()>1) {
			TimeSeriesCollection tsc=new TimeSeriesCollection();
			Iterator it=period.getSubCategories().iterator();
	        while (it.hasNext()) {
	        	Period p=(Period) it.next();
		        TimeSeries timeSeries = new TimeSeries(period.getLabel(p.getId()), SliceTimePeriod.class);
				SliceTimePeriod sp=null;
		        for (int i=0; i<p.getSlices().length; i++) {
		            sp=new SliceTimePeriod(p.getSlices()[i], sp);
		            timeSeries.add(sp, p.getSlices()[i].getTotal());
		        }
		        tsc.addSeries(timeSeries);		        
	        }
	        
		    childrenTotalChart = ChartFactory.createTimeSeriesChart(
		            "Total (sum of values per slice)",  // Title
		            "Time",     // X-Axis label
		            "Total",  // Y-Axis label
		            tsc,        // Dataset
		            true,       // Show legend
		            true,
		            true
		           );
		}
		
		return childrenTotalChart;
	}

	private JFreeChart childrenAverageChart;
	
	/**
	 * @return
	 * @throws SQLException
	 */
	public JFreeChart childrenAverageChart() throws SQLException {
		if (childrenAverageChart==null && period.getSubCategories().size()>1) {
			TimeSeriesCollection tsc=new TimeSeriesCollection();
			Iterator it=period.getSubCategories().iterator();
	        while (it.hasNext()) {
	        	Period p=(Period) it.next();
		        TimeSeries timeSeries = new TimeSeries(period.getLabel(p.getId()), SliceTimePeriod.class);
				SliceTimePeriod sp=null;
		        for (int i=0; i<p.getSlices().length; i++) {
		            sp=new SliceTimePeriod(p.getSlices()[i], sp);
		            timeSeries.add(sp, p.getSlices()[i].getTotal()/p.getSlices()[i].getMeasurements());
		        }
		        tsc.addSeries(timeSeries);		        
	        }
	        
		    childrenAverageChart = ChartFactory.createTimeSeriesChart(
		            "Average",  // Title
		            "Time",     // X-Axis label
		            "Average",  // Y-Axis label
		            tsc,        // Dataset
		            true,       // Show legend
		            true,
		            true
		           );
		}
		return childrenAverageChart;
	}

	private JFreeChart mainChart;

    /**
	 * @param i
	 * @return
	 */
	public JFreeChart mainChart() {		
		if (mainChart==null) {
			Slice[] periodSlices = period.getSlices();
			int size = periodSlices.length;
			
			Date[] dates=new Date[size];
			double[] higs=new double[size];
			double[] lows=new double[size];
			double[] opens=new double[size];
			double[] closes=new double[size];
			double[] volumes=new double[size];
			
			for (int k=0; k<size; k++) {
			    Slice slice=periodSlices[k];
			    dates[k]=new Date(slice.getTimeFrom());
			    higs[k]=slice.getMaxValue();
			    lows[k]=slice.getMinValue();		            
			    double avg = slice.getMeasurements()==0 ? 0 : slice.getTotal()/slice.getMeasurements();
			    opens[k]=Math.max(slice.getMeasurements()==0 ? avg : avg-slice.getDeviation(), slice.getMinValue());			    
			    closes[k]=Math.min(slice.getMeasurements()==0 ? avg : opens[k]+slice.getDeviation(), slice.getMaxValue());
			    volumes[k]=slice.getTotal();
			}
			
			OHLCDataset dataSet=new DefaultHighLowDataset(
					normalizeName(period.getName()),
			        dates,
			        higs,
			        lows,
			        opens,
			        closes,
			        volumes);
			
			mainChart = ChartFactory.createCandlestickChart(
					normalizeName(period.getName()),
			        "Time",
			        "Value",
			        dataSet,
			        true
			       );
		}
		
		return mainChart;
	}

 	/**
	 * @return Returns the period.
	 */
	public AbstractPeriod getPeriod() {
		return period;
	}
	
	private Collection children;
	
	public Collection getChildren() throws SQLException {
		if (children==null) {
			children=new ArrayList();
			Iterator it=period.getSubCategories().iterator();
			while (it.hasNext()) {
				children.add(new PeriodVisualizer((Period) it.next()));
			}
		}
		
		return children;
	}

	/**
		 * @param root
		 */
		public static void generateHtmlDoc(AbstractPeriod root, final File outputDir, final int width, final int height) {
			root.accept(new Visitor() {
				
				ClassTransformerFactory ctf=new ClassTransformerFactory(null, null, true);
	
				public boolean visit(Object target) {
					try {
						AbstractPeriod period = (AbstractPeriod) target;
						ctf.transform(period, "category", null, null, null, null, new StreamResult(new File(outputDir, target instanceof Period ? "period_"+((Period) period).getId()+".html" : "index.html")));
	//										DOMUtils.serialize(period, "category", new File(outputDir, "period_"+period.getId()+".xml"));									
						PeriodVisualizer visualizer=new PeriodVisualizer(period);
						String periodId = period instanceof Period ? String.valueOf(((Period) period).getId()) : ((SyntheticPeriod) period).getId();								
						
						ChartHelper chartHelper = new ChartHelper(ChartHelper.FORMAT_JPG, width, height);										
						chartHelper.output(visualizer.averageChart(), new File(outputDir, "average_"+periodId+".jpg"));
						chartHelper.output(visualizer.childrenAverageChart(), new File(outputDir, "childrenAverage_"+periodId+".jpg"));
						chartHelper.output(visualizer.childrenIntensityChart(), new File(outputDir, "childrenIntensity_"+periodId+".jpg"));
						chartHelper.output(visualizer.childrenTotalChart(), new File(outputDir, "childrenTotal_"+periodId+".jpg"));
						chartHelper.output(visualizer.intensityChart(), new File(outputDir, "intensity_"+periodId+".jpg"));
						chartHelper.output(visualizer.mainChart(), new File(outputDir, "main_"+periodId+".jpg"));
						chartHelper.output(visualizer.totalChart(), new File(outputDir, "total_"+periodId+".jpg"));
					} catch (ParserConfigurationException e) {
						throw new RuntimeConfigurationException(e);
					} catch (FactoryConfigurationError e) {
						throw new RuntimeConfigurationException(e);
					} catch (IOException e) {
						throw new RuntimeConfigurationException(e);
					} catch (TransformerException e) {
						throw new RuntimeConfigurationException(e);
					} catch (SQLException e) {
						throw new RuntimeConfigurationException(e);
					}
					return true;
				}
				
			});
		}
		
		public static void main(String args[]) throws Exception {
			File outputDir=new File("_MetricDebug"); 
			if (outputDir.exists()) {
				if (!outputDir.isDirectory()) {
					throw new MetricsException(outputDir.getAbsolutePath()+" in not a directory");
				}
			} else {
				if (!outputDir.mkdirs()) {
					throw new MetricsException("Cannot create directory "+outputDir.getAbsolutePath());
				}
			}
			
			// Generation of html's
			SQLProcessor processor=new SQLProcessor(new HypersonicServerDataSource("localhost/metrics", null), new MapContext(new HashMap()));
			new MetricNormalizer(processor).normalize();
			PeriodFactory factory=new PeriodFactory(processor);			
			MetricLocator locator=new MetricLocator(processor);
			AbstractPeriod root=factory.getPeriod(locator.find("poc.wsusjxla3b3a147.send size").intValue(), 0, Long.MAX_VALUE, 20, true);
			PeriodVisualizer.generateHtmlDoc(root, outputDir, 500, 300);
			
		}
}
