/*
 * hgee 2
 * Hammurapi Group Enterprise Extensions 
 * Copyright (C) 2006  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/products/enterpriseextensions/index.html
 * e-Mail: support@hammurapi.biz  
 */
package biz.hammurapi.metrics.persistent;

import java.io.File;
import java.util.Properties;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import biz.hammurapi.config.Component;
import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.config.Context;
import biz.hammurapi.config.DomConfigurable;
import biz.hammurapi.config.MapContext;
import biz.hammurapi.metrics.Slice;
import biz.hammurapi.metrics.SliceConsumer;
import biz.hammurapi.sql.ConnectionPerThreadDataSource;
import biz.hammurapi.sql.IdentityManager;
import biz.hammurapi.sql.SQLProcessor;
import biz.hammurapi.xml.dom.AbstractDomObject;
import biz.hammurapi.xml.dom.DOMUtils;


/**
 * @author Pavel Vlasov
 */
public class PersistingSliceConsumerDomConfigurableAdapter implements DomConfigurable, Component, SliceConsumer {    
    private SliceConsumer master;
	private ConnectionPerThreadDataSource ds;
	private SQLProcessor processor;
	private long from;
    
    public void start() throws ConfigurationException {
        from=System.currentTimeMillis();
        try {           
            ds=new ConnectionPerThreadDataSource(
            		AbstractDomObject.getElementText(configElement, "driver-class"),
					AbstractDomObject.getElementText(configElement, "database-url"),
					AbstractDomObject.getElementText(configElement, "user"),
					AbstractDomObject.getElementText(configElement, "password"),
					null); // TODO - connection initialization nested elements???
            
            Properties properties=new Properties();
            NodeList nl=DOMUtils.selectNodeList(configElement, "name-map");
    		for (int i=0, l=nl.getLength(); i<l; ++i) {
    			Element nme = (Element) nl.item(i);
            	properties.setProperty(nme.getAttribute("key"), DOMUtils.getElementText(nme));
            }
            
            processor=new SQLProcessor(ds, new MapContext(properties));
                        
//            String kmValue=AbstractDomObject.getElementText(configElement, "keep-measurements", cxpa);
//            boolean keepMeasurements = kmValue==null ? false : "yes".equalsIgnoreCase(kmValue);
//            
//            String maxQueueValue=AbstractDomObject.getElementText(configElement, "max-queue", cxpa);
//            int maxQueue = maxQueueValue==null ? 1000 : Integer.parseInt(maxQueueValue);
            
            String root=AbstractDomObject.getElementText(configElement, "root-category");
            
            String historyValue=AbstractDomObject.getElementText(configElement, "history");            
            long history = historyValue==null ? PersistingSliceConsumer.DAY : toTime(historyValue);
            
            String miValue=AbstractDomObject.getElementText(configElement, "maintenance-interval");            
            long maintenanceInterval = miValue==null ? 10*PersistingSliceConsumer.MINUTE : toTime(miValue);
            
            String imValue=AbstractDomObject.getElementText(configElement, "identity-manager");            
            IdentityManager identityManager = imValue==null ? null : (IdentityManager) Class.forName(imValue).newInstance();
            
            master = new PersistingSliceConsumer(
                	root,
                    history,
                    maintenanceInterval,
                    processor,
					identityManager);            

            if (master instanceof Component) {
            	((Component) master).start();
            }
        } catch (Exception e) {
            throw new ConfigurationException(e);
		}
    }

    /**
	 * @param historyValue
	 * @return
	 */
	static long toTime(String timeValue) throws ConfigurationException {
		String[] tva=timeValue.trim().split(" ");
		long ret=Long.parseLong(tva[0]);
		if (tva.length==1) {
			return ret;
		} else if (tva.length==2) {
			if ("sec".equalsIgnoreCase(tva[1])) {
				return ret*PersistingSliceConsumer.SECOND;
			} else if ("min".equalsIgnoreCase(tva[1])) {
				return ret*PersistingSliceConsumer.MINUTE;
			} else if ("hour".equalsIgnoreCase(tva[1]) || "hours".equalsIgnoreCase(tva[1])) {
				return ret*PersistingSliceConsumer.MONTH;
			} else if ("day".equalsIgnoreCase(tva[1]) || "days".equalsIgnoreCase(tva[1])) {
				return ret*PersistingSliceConsumer.DAY;
			} else if ("week".equalsIgnoreCase(tva[1]) || "weeks".equalsIgnoreCase(tva[1])) {
				return ret*PersistingSliceConsumer.WEEK;
			} else if ("month".equalsIgnoreCase(tva[1]) || "months".equalsIgnoreCase(tva[1])) {
				return ret*PersistingSliceConsumer.MONTH;
			} else if ("year".equalsIgnoreCase(tva[1]) || "years".equalsIgnoreCase(tva[1])) {
				return ret*PersistingSliceConsumer.YEAR;
			} else {
				throw new ConfigurationException("Invalid time: "+timeValue);				
			}
		} else {
			throw new ConfigurationException("Invalid time: "+timeValue);
		}
	}

	public void stop() throws ConfigurationException {
        if (master instanceof Component) {
            ((Component) master).stop();
        }
        
        if (ds!=null) {
	        try {
	        	if (DOMUtils.selectSingleNode(configElement, "output-dir")!=null) {
					PeriodFactory factory=new PeriodFactory(processor);
					SyntheticPeriod root;
					root=factory.getRootPeriod(
							from, 
							System.currentTimeMillis(), 
							Integer.parseInt(DOMUtils.getSingleNonBlankElementText((Element) configElement, "slices")), false);
					
					PeriodVisualizer.generateHtmlDoc(
							root, 
							new File(DOMUtils.getSingleNonBlankElementText((Element) configElement, "output-dir")), 
							Integer.parseInt(DOMUtils.getSingleNonBlankElementText((Element) configElement, "width")), 
							Integer.parseInt(DOMUtils.getSingleNonBlankElementText((Element) configElement, "height")));
	        	}
										
				ds.shutdown();
			} catch (Exception e) {
				throw new ConfigurationException(e);
			}
        }
    }
    
	private Element configElement;
    
	public void configure(Node configNode, Context context) throws ConfigurationException {
		configElement=(Element) configNode;
	}

	public boolean consumeSlice(String category, Slice slice) {
		return master.consumeSlice(category, slice);
	}

	public void setOwner(Object owner) {
		// Ignore		
	}	
}
