/*
 * hgee 2
 * Hammurapi Group Enterprise Extensions 
 * Copyright (C) 2006  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/products/enterpriseextensions/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.metrics.persistent;

import java.sql.SQLException;
import java.util.Iterator;
import java.util.List;

import org.w3c.dom.Element;

import biz.hammurapi.xml.dom.DomSerializable;

/**
 * Measurement period for a particular metric
 * @author Pavel Vlasov
 * @version $Revision$
 */
public class SyntheticPeriod extends AbstractPeriod implements DomSerializable {
	private int[] ids;
	private String id;
	
    /**
     * Actual from and to can differ from the passed if shrink is true slices don't cover all the period.
     * @param processor
     * @param id
     * @param from
     * @param to
     * @param periods
     * @param shrink Shrink period to exclude leading and trailing times without measurements.
     * @throws SQLException
     */
    SyntheticPeriod(PeriodFactory factory, String name, int[] ids, long from, long to, int slices, boolean shrink) throws SQLException {
    	this.ids=ids;
    	this.name=name;
    	MetricsEngine engine = factory.getEngine();
    	this.factory=factory;    	    	
    	
        this.timeFrom=from;
        this.timeTo=to;
        if (shrink) {
        	for (int i=0; i<ids.length; i++) {
	            ActualPeriod actualPeriod = engine.getActualPeriod(ids[i], to, from);
	            if (actualPeriod!=null && actualPeriod.getTimeFrom()!=actualPeriod.getTimeTo()) {
	                this.timeFrom=Math.max(from, actualPeriod.getTimeFrom());
	                this.timeTo=Math.min(to, actualPeriod.getTimeTo());
	            }
        	}
        }
        
        createSlices(slices);
        
        // calculate values for slices and totals here
        for (int i=0; i<ids.length; i++) {
	        Iterator it=engine.getSlice(ids[i], this.timeTo, this.timeFrom).iterator();
	        while (it.hasNext()) {
				aggregate((Slice) it.next());
	        }
        }
                
        normalizeSliceDeviations();
        
        StringBuffer idb=new StringBuffer("s_");
        idb.append(from);
        idb.append("_");
        idb.append(to);
        idb.append("_");
        idb.append(slices);
        idb.append("_");
        idb.append(shrink);
        for (int i=0; i<ids.length; i++) {
        	idb.append("_");
        	idb.append(ids[i]);
        }
        id=idb.toString();
    }
    
	/**
	 * @throws SQLException
	 */
	protected void loadSubCategories(List subCategories) throws SQLException {
		for (int i=0; i<ids.length; i++) {
			Period subPeriod = factory.getPeriod(ids[i], getTimeFrom(), getTimeTo(), slices.length, false); 
			subCategories.add(subPeriod);
		}
	}
	
	public String getId() {
		return id;
	}
	
	protected void setAttributes(Element holder) {
		super.setAttributes(holder);
		holder.setAttribute("id", String.valueOf(id));
	}	
}
