/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.sql;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Map;
import java.util.StringTokenizer;

/**
 * Base class for reflection projectors.
 * @author Pavel Vlasov
 * @version $Revision: 1.2 $
 */
public class BaseReflectionProjector {
	
	private Map typeMap;
	
	/**
	 * @param typeMap type map to be used in java.sql.ResultSet.getField(String/int, Map) method
	 */
	protected BaseReflectionProjector(Map typeMap) {
		this.typeMap=typeMap;
	}

	/**
	 * This implmentation uses ResultSet.getObject() method to obtain field value.
	 * Override this method if needed to provide custom type conversion.
	 * @param rs ResultSet
	 * @param columnName Field name
	 * @return Column value
	 * @throws SQLException
	 */
	protected Object getColumn(ResultSet rs, String columnName) throws SQLException {
		return typeMap==null ? rs.getObject(columnName) : rs.getObject(columnName, typeMap);
	}
	
	/**
	 * This implmentation uses ResultSet.getObject() method to obtain field value.
	 * Override this method if needed to provide custom type conversion.
	 * @param rs ResultSet
	 * @param columnNo Column number
	 * @return Field value
	 * @throws SQLException
	 */
	protected Object getColumn(ResultSet rs, int columnNo) throws SQLException {
		return typeMap==null ? rs.getObject(columnNo) : rs.getObject(columnNo, typeMap);
	}
	
	/**
	 * Converts column name such as "MY_COLUMN" to java property name such as myColumn
	 * @param columnName
	 * @return
	 */
	public static String propertyName(String columnName) {
		StringTokenizer st=new StringTokenizer(columnName,"_");
		StringBuffer ret=new StringBuffer();
		boolean capitalize=false;
		while (st.hasMoreTokens()) {
			String token=st.nextToken();
			if (capitalize) {
				ret.append(Character.toUpperCase(token.charAt(0)));
				if (token.length()>1) {
					ret.append(token.substring(1).toLowerCase());
				}
			} else {
				ret.append(token.toLowerCase());
				capitalize=true;
			}
		}
		return ret.toString();		
	}
	
	/**
	 * MY_COLUMN -&gt; getMyColumn
	 * @param columnName
	 * @return
	 */
	public static String accessorName(String columnName) {
		String propertyName=propertyName(columnName);
		return "get"+ (propertyName.length()<2 ? propertyName.toUpperCase() : propertyName.substring(0, 1).toUpperCase() + propertyName.substring(1)); 
	}

	/**
	 * MY_COLUMN -&gt; setMyColumn
	 * @param columnName
	 * @return
	 */
	public static String mutatorName(String columnName) {
		String propertyName=propertyName(columnName);
		return "set"+ (propertyName.length()<2 ? propertyName.toUpperCase() : propertyName.substring(0, 1).toUpperCase() + propertyName.substring(1)); 
	}
}
