/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.sql;

import java.sql.PreparedStatement;
import java.sql.SQLException;

/**
 * Represents multiple parameterizers as one
 * @author Pavel Vlasov
 * @version $Revision: 1.1 $
 */
public class CompositeParameterizer implements Parameterizer {

	private Parameterizer[] parameterizers;

	/**
	 * Constructs CompositeParameterizer
	 * @param parameterizers array of parameterisers
	 */
	public CompositeParameterizer(Parameterizer[] parameterizers) {
		super();
		this.parameterizers=parameterizers;
	}

	/**
	 * Invokes parameterizers one-by-one
	 */
	public void parameterize(PreparedStatement ps) throws SQLException {
		for (int i=0; i<parameterizers.length; i++) {
			parameterizers[i].parameterize(ps);
		}
	}
}
