/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.sql;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import biz.hammurapi.config.Component;
import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.config.Context;
import biz.hammurapi.config.DomConfigurable;
import biz.hammurapi.config.Wrapper;
import biz.hammurapi.xml.dom.DOMUtils;
import biz.hammurapi.xml.dom.DOMUtils;


public class ConnectionPerThreadDataSourceComponent implements Wrapper, DomConfigurable, Component {

	private ConnectionPerThreadDataSource master;
	private String driverClass;
	private String dbUrl;
	private String user;
	private String password;

	public ConnectionPerThreadDataSourceComponent() {
		super();
	}

	public Object getMaster() {
		if (master==null) {
			throw new IllegalStateException("Not yet started");
		}
		return master;
	}
	
	private Collection initStatements=new ArrayList();

	public void configure(Node configNode, Context context) throws ConfigurationException {
		try {
			driverClass=DOMUtils.getSingleElementText((Element) configNode, "driver-class");
			dbUrl=DOMUtils.getSingleElementText((Element) configNode, "connection-url");
			user=DOMUtils.getSingleElementText((Element) configNode, "user");
			password=DOMUtils.getSingleElementText((Element) configNode, "password");
			NodeList nl=DOMUtils.selectNodeList(configNode, "init-connection");
			for (int i=0, l=nl.getLength(); i<l; ++i) {
				Element el = (Element) nl.item(i);
				initStatements.add(DOMUtils.getElementText(el));
			}
		} catch (Exception e) {
			throw new ConfigurationException("Cannot read parameters", e);
		}

	}

	public void start() throws ConfigurationException {
		try {
			master=new ConnectionPerThreadDataSource(
					driverClass, 
					dbUrl, 
					user, 
					password,
					new Transaction() {

						public boolean execute(SQLProcessor processor) throws SQLException {
							Iterator it=initStatements.iterator();
							while (it.hasNext()) {
								processor.processUpdate((String) it.next(), null);
							}
							return true;
						}
						
					});
		} catch (ClassNotFoundException e) {
			throw new ConfigurationException("Driver class not found", e);
		}
	}

	public void stop() throws ConfigurationException {
		if (master!=null) {
			master.shutdown();
		}

	}

	public void setOwner(Object owner) {
		// Nothing
	}
}
