package biz.hammurapi.sql;

import java.sql.Connection;
import java.sql.SQLException;
import java.text.MessageFormat;

/**
 * Generates identities by executing specified SQL statement
 * @author Pavel Vlasov
 *
 */
public class GenericIdentityGenerator implements IdentityGenerator {
	
	
	private boolean invariant;
	private MessageFormat mf;
	private Object[] args=new Object[] {null};
	private String sql;
	
	/**
	 * Constructor
	 * @param pattern - SQL statement pattern. {0} stands for table name.
	 */
	public GenericIdentityGenerator(String pattern) {
		invariant = pattern.indexOf("{0}")==-1;
		if (invariant) {
			this.sql=pattern;
		} else {
			mf=new MessageFormat(pattern);
		}
	}

	public int generate(Connection con, String name) throws SQLException {
		String esql;
		if (invariant) {
			esql=this.sql;
		} else {
			esql=mf.format(args, new StringBuffer(), null).toString();
		}
		
		return new SQLProcessor(con, null).projectSingleInt(esql, null);
	}

}
