/*
  * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.sql;

import java.sql.SQLException;
import java.util.Properties;

import org.w3c.dom.Element;

import biz.hammurapi.config.ConfigurationException;

/**
 * Interface of DatabaseObject. This interface is used for creating dynamic proxies backed
 * up with database obects.
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public interface IDatabaseObject {

	/**
	 * Updates row in a table by primary key
	 * @param processor SQLProcessor
	 * @param tableName Target table
	 * @return Number of updates
	 * @throws SQLException
	 */
	public abstract int update(SQLProcessor processor, String tableName) throws SQLException;

	/**
	 * Deletes row in a table by primary key.
	 * @param processor SQLProcessor
	 * @param tableName Target table
	 * @return Number of updates
	 * @throws SQLException
	 */
	public abstract int delete(SQLProcessor processor, String tableName) throws SQLException;

	public abstract int insert(SQLProcessor processor, String tableName) throws SQLException;

	/**
	 * Loads columns from XML element
	 * @param holder
	 * @throws ConfigurationException If loading fails
	 */
	public abstract void fromDom(Element holder) throws ConfigurationException;

	public abstract void fromDom(Element holder, Properties nameMap) throws ConfigurationException;

	/**
	 * Serializes to DOM.
	 * @param holder Holder element
	 * @param nameMap Name map
	 * @param originals Output original values if any.
	 */
	public abstract void toDom(Element holder, Properties nameMap, boolean originals);

	/**
	 * Sets current values as original values in primary key columns.
	 */
	public abstract void setOriginal();

	/**
	 * @return true if object was modified since last database operation.
	 */
	public abstract boolean isModified();

	/**
	 * isDeleted flag is cleared when primary key columns are modified
	 * @return true if delete() method was executed.
	 */
	public abstract boolean isDeleted();

	/**
	 * Sets all columns to default values and clears 
	 * modified and deleted flags.
	 *
	 */
	public abstract void clear();

	/**
	 * Copies values from source object to this object
	 * @param source
	 */
	public abstract void copy(DatabaseObject source);

	public abstract void setColumnAttribute(String columnName, Object key, Object value);

	public abstract Object getColumnAttribute(String columnName, Object key);

	public abstract Object removeColumnAttribute(String columnName, Object key);

}