/*
  * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz 
 */
package biz.hammurapi.sql;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.StringTokenizer;

import biz.hammurapi.legacy.persistence.PersistenceException;
import biz.hammurapi.legacy.persistence.Storage;

/**
 * @author Pavel Vlasov
 * @version $Revision: 1.1 $
 */
public class JdbcStorage implements Storage {
	
	private SQLProcessor processor;

	public JdbcStorage(SQLProcessor processor) {
		this.processor=processor;
	}
	
	public SQLProcessor getProcessor() {
		return processor;
	}

	/**
	 * Key format should be:
	 * {table}:{key field}:{key value}:{projector}
	 */
	public String put(Object o) throws PersistenceException {
		if (o instanceof JdbcPersistable) {
			return ((JdbcPersistable) o).store(this);
		}
		return null;
	}

	public Object get(String key) throws PersistenceException {
		final StringTokenizer st=new StringTokenizer(key, ":");
		try {
			return processor.projectSingleObject(
						"DELETE FROM "+st.nextToken()+" WHERE "+st.nextToken()+"=?",
						new Parameterizer() {
							public void parameterize(PreparedStatement ps) throws SQLException {
								ps.setString(1, st.nextToken());
							}
						}, 
						(Projector) Class.forName(st.nextToken()).newInstance());
		} catch (InstantiationException e) {
			throw new PersistenceException(e);
		} catch (IllegalAccessException e) {
			throw new PersistenceException(e);
		} catch (ClassNotFoundException e) {
			throw new PersistenceException(e);
		} catch (SQLException e) {
			throw new PersistenceException(e);
		}
	}

	public void remove(String key) throws PersistenceException {
		final StringTokenizer st=new StringTokenizer(key, ":");
		try {
			processor.processUpdate(
					"DELETE FROM "+st.nextToken()+" WHERE "+st.nextToken()+"=?",
					new Parameterizer() {
						public void parameterize(PreparedStatement ps) throws SQLException {
							ps.setString(1, st.nextToken());
						}
					});
		} catch (SQLException e) {
			throw new PersistenceException(e);
		}
	}

}
