package biz.hammurapi.sql;

import java.sql.Connection;
import java.sql.SQLException;
import java.text.MessageFormat;

/**
 * Generates identities using SQLProcessor nextPK method
 * @author Pavel Vlasov
 *
 */
public class OracleSequenceIdentityGenerator implements IdentityGenerator {
	
	/**
	 * Default constructor with "{0}_SEQ" pattern
	 */
	public OracleSequenceIdentityGenerator() {
		this("{0}_SEQ");
	}
	
	private int mode; // 0 - passthrough 1 - invariant 2 - format
	private String seqName;
	private MessageFormat mf;
	private Object[] args=new Object[] {null};
	
	/**
	 * Constructor
	 * @param pattern - Key name pattern. {0} stands for table name.
	 */
	public OracleSequenceIdentityGenerator(String pattern) {
		if ("{0}".equals(pattern)) {
			mode=0;
		} else if (pattern.indexOf("{0}")==-1) {
			mode=1;
			this.seqName=pattern;
		} else {
			mode=2;
			mf=new MessageFormat(pattern);
		}
	}

	public int generate(Connection con, String name) throws SQLException {
		String ename;
		switch (mode) {
		case 0: 
			ename=name+"_SEQ";
			break;
		case 1:
			ename=seqName;
			break;
		case 2:
			synchronized (mf) {
				args[0]=name;
				ename=mf.format(args, new StringBuffer(), null).toString();
			}
		default:
			throw new IllegalStateException("Invalid mode: "+mode);
		}
		
		return new SQLProcessor(con, null).projectSingleInt("SELECT "+ename+".NEXTVAL FROM DUAL", null);
	}

}
