/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.sql;

import java.io.InputStream;
import java.io.Reader;
import java.math.BigDecimal;
import java.net.URL;
import java.sql.Array;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Date;
import java.sql.Ref;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Statement;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.Calendar;
import java.util.Map;


class ResultSetProxy implements ResultSet {
	private final SQLProcessor sqlProcessor;
	ResultSet master;
	
	ResultSetProxy(SQLProcessor processor, ResultSet master) {
		this.master=master;
		sqlProcessor = processor;
	}
	/**
	 * @param row
	 * @return
	 * @throws java.sql.SQLException
	 */
	public boolean absolute(int row) throws SQLException {
		return master.absolute(row);
	}
	/**
	 * @throws java.sql.SQLException
	 */
	public void afterLast() throws SQLException {
		master.afterLast();
	}
	/**
	 * @throws java.sql.SQLException
	 */
	public void beforeFirst() throws SQLException {
		master.beforeFirst();
	}
	/**
	 * @throws java.sql.SQLException
	 */
	public void cancelRowUpdates() throws SQLException {
		master.cancelRowUpdates();
	}
	/**
	 * @throws java.sql.SQLException
	 */
	public void clearWarnings() throws SQLException {
		master.clearWarnings();
	}
	/**
	 * @throws java.sql.SQLException
	 */
	public void close() throws SQLException {
		master.close();
	}
	/**
	 * @throws java.sql.SQLException
	 */
	public void deleteRow() throws SQLException {
		master.deleteRow();
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj) {
		return master.equals(obj);
	}
	/**
	 * @param columnName
	 * @return
	 * @throws java.sql.SQLException
	 */
	public int findColumn(String columnName) throws SQLException {
		return master.findColumn(sqlProcessor.parse(columnName));
	}
	/**
	 * @return
	 * @throws java.sql.SQLException
	 */
	public boolean first() throws SQLException {
		return master.first();
	}
	/**
	 * @param i
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Array getArray(int i) throws SQLException {
		return master.getArray(i);
	}
	/**
	 * @param colName
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Array getArray(String colName) throws SQLException {
		return master.getArray(sqlProcessor.parse(colName));
	}
	/**
	 * @param columnIndex
	 * @return
	 * @throws java.sql.SQLException
	 */
	public InputStream getAsciiStream(int columnIndex) throws SQLException {
		return master.getAsciiStream(columnIndex);
	}
	/**
	 * @param columnName
	 * @return
	 * @throws java.sql.SQLException
	 */
	public InputStream getAsciiStream(String columnName)
			throws SQLException {
		return master.getAsciiStream(sqlProcessor.parse(columnName));
	}
	/**
	 * @param columnIndex
	 * @return
	 * @throws java.sql.SQLException
	 */
	public BigDecimal getBigDecimal(int columnIndex) throws SQLException {
		return master.getBigDecimal(columnIndex);
	}
	/**
	 * @param columnIndex
	 * @param scale
	 * @return
	 * @throws java.sql.SQLException
	 */
	public BigDecimal getBigDecimal(int columnIndex, int scale)
			throws SQLException {
		return master.getBigDecimal(columnIndex, scale);
	}
	/**
	 * @param columnName
	 * @return
	 * @throws java.sql.SQLException
	 */
	public BigDecimal getBigDecimal(String columnName) throws SQLException {
		return master.getBigDecimal(sqlProcessor.parse(columnName));
	}
	/**
	 * @param columnName
	 * @param scale
	 * @return
	 * @throws java.sql.SQLException
	 */
	public BigDecimal getBigDecimal(String columnName, int scale)
			throws SQLException {
		return master.getBigDecimal(sqlProcessor.parse(columnName), scale);
	}
	/**
	 * @param columnIndex
	 * @return
	 * @throws java.sql.SQLException
	 */
	public InputStream getBinaryStream(int columnIndex) throws SQLException {
		return master.getBinaryStream(columnIndex);
	}
	/**
	 * @param columnName
	 * @return
	 * @throws java.sql.SQLException
	 */
	public InputStream getBinaryStream(String columnName)
			throws SQLException {
		return master.getBinaryStream(sqlProcessor.parse(columnName));
	}
	/**
	 * @param i
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Blob getBlob(int i) throws SQLException {
		return master.getBlob(i);
	}
	/**
	 * @param colName
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Blob getBlob(String colName) throws SQLException {
		return master.getBlob(sqlProcessor.parse(colName));
	}
	/**
	 * @param columnIndex
	 * @return
	 * @throws java.sql.SQLException
	 */
	public boolean getBoolean(int columnIndex) throws SQLException {
		return master.getBoolean(columnIndex);
	}
	/**
	 * @param columnName
	 * @return
	 * @throws java.sql.SQLException
	 */
	public boolean getBoolean(String columnName) throws SQLException {
		return master.getBoolean(sqlProcessor.parse(columnName));
	}
	/**
	 * @param columnIndex
	 * @return
	 * @throws java.sql.SQLException
	 */
	public byte getByte(int columnIndex) throws SQLException {
		return master.getByte(columnIndex);
	}
	/**
	 * @param columnName
	 * @return
	 * @throws java.sql.SQLException
	 */
	public byte getByte(String columnName) throws SQLException {
		return master.getByte(sqlProcessor.parse(columnName));
	}
	/**
	 * @param columnIndex
	 * @return
	 * @throws java.sql.SQLException
	 */
	public byte[] getBytes(int columnIndex) throws SQLException {
		return master.getBytes(columnIndex);
	}
	/**
	 * @param columnName
	 * @return
	 * @throws java.sql.SQLException
	 */
	public byte[] getBytes(String columnName) throws SQLException {
		return master.getBytes(sqlProcessor.parse(columnName));
	}
	/**
	 * @param columnIndex
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Reader getCharacterStream(int columnIndex) throws SQLException {
		return master.getCharacterStream(columnIndex);
	}
	/**
	 * @param columnName
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Reader getCharacterStream(String columnName) throws SQLException {
		return master.getCharacterStream(sqlProcessor.parse(columnName));
	}
	/**
	 * @param i
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Clob getClob(int i) throws SQLException {
		return master.getClob(i);
	}
	/**
	 * @param colName
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Clob getClob(String colName) throws SQLException {
		return master.getClob(sqlProcessor.parse(colName));
	}
	/**
	 * @return
	 * @throws java.sql.SQLException
	 */
	public int getConcurrency() throws SQLException {
		return master.getConcurrency();
	}
	/**
	 * @return
	 * @throws java.sql.SQLException
	 */
	public String getCursorName() throws SQLException {
		return master.getCursorName();
	}
	/**
	 * @param columnIndex
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Date getDate(int columnIndex) throws SQLException {
		return master.getDate(columnIndex);
	}
	/**
	 * @param columnIndex
	 * @param cal
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Date getDate(int columnIndex, Calendar cal) throws SQLException {
		return master.getDate(columnIndex, cal);
	}
	/**
	 * @param columnName
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Date getDate(String columnName) throws SQLException {
		return master.getDate(sqlProcessor.parse(columnName));
	}
	/**
	 * @param columnName
	 * @param cal
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Date getDate(String columnName, Calendar cal)
			throws SQLException {
		return master.getDate(sqlProcessor.parse(columnName), cal);
	}
	/**
	 * @param columnIndex
	 * @return
	 * @throws java.sql.SQLException
	 */
	public double getDouble(int columnIndex) throws SQLException {
		return master.getDouble(columnIndex);
	}
	/**
	 * @param columnName
	 * @return
	 * @throws java.sql.SQLException
	 */
	public double getDouble(String columnName) throws SQLException {
		return master.getDouble(sqlProcessor.parse(columnName));
	}
	/**
	 * @return
	 * @throws java.sql.SQLException
	 */
	public int getFetchDirection() throws SQLException {
		return master.getFetchDirection();
	}
	/**
	 * @return
	 * @throws java.sql.SQLException
	 */
	public int getFetchSize() throws SQLException {
		return master.getFetchSize();
	}
	/**
	 * @param columnIndex
	 * @return
	 * @throws java.sql.SQLException
	 */
	public float getFloat(int columnIndex) throws SQLException {
		return master.getFloat(columnIndex);
	}
	/**
	 * @param columnName
	 * @return
	 * @throws java.sql.SQLException
	 */
	public float getFloat(String columnName) throws SQLException {
		return master.getFloat(sqlProcessor.parse(columnName));
	}
	/**
	 * @param columnIndex
	 * @return
	 * @throws java.sql.SQLException
	 */
	public int getInt(int columnIndex) throws SQLException {
		return master.getInt(columnIndex);
	}
	/**
	 * @param columnName
	 * @return
	 * @throws java.sql.SQLException
	 */
	public int getInt(String columnName) throws SQLException {
		return master.getInt(sqlProcessor.parse(columnName));
	}
	/**
	 * @param columnIndex
	 * @return
	 * @throws java.sql.SQLException
	 */
	public long getLong(int columnIndex) throws SQLException {
		return master.getLong(columnIndex);
	}
	/**
	 * @param columnName
	 * @return
	 * @throws java.sql.SQLException
	 */
	public long getLong(String columnName) throws SQLException {
		return master.getLong(sqlProcessor.parse(columnName));
	}
	/**
	 * @return
	 * @throws java.sql.SQLException
	 */
	public ResultSetMetaData getMetaData() throws SQLException {
		return master.getMetaData();
	}
	/**
	 * @param columnIndex
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Object getObject(int columnIndex) throws SQLException {
		return master.getObject(columnIndex);
	}
	/**
	 * @param i
	 * @param map
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Object getObject(int i, Map map) throws SQLException {
		return master.getObject(i, map);
	}
	/**
	 * @param columnName
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Object getObject(String columnName) throws SQLException {
		return master.getObject(sqlProcessor.parse(columnName));
	}
	/**
	 * @param colName
	 * @param map
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Object getObject(String colName, Map map) throws SQLException {
		return master.getObject(sqlProcessor.parse(colName), map);
	}
	/**
	 * @param i
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Ref getRef(int i) throws SQLException {
		return master.getRef(i);
	}
	/**
	 * @param colName
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Ref getRef(String colName) throws SQLException {
		return master.getRef(sqlProcessor.parse(colName));
	}
	/**
	 * @return
	 * @throws java.sql.SQLException
	 */
	public int getRow() throws SQLException {
		return master.getRow();
	}
	/**
	 * @param columnIndex
	 * @return
	 * @throws java.sql.SQLException
	 */
	public short getShort(int columnIndex) throws SQLException {
		return master.getShort(columnIndex);
	}
	/**
	 * @param columnName
	 * @return
	 * @throws java.sql.SQLException
	 */
	public short getShort(String columnName) throws SQLException {
		return master.getShort(sqlProcessor.parse(columnName));
	}
	/**
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Statement getStatement() throws SQLException {
		return master.getStatement();
	}
	/**
	 * @param columnIndex
	 * @return
	 * @throws java.sql.SQLException
	 */
	public String getString(int columnIndex) throws SQLException {
		return master.getString(columnIndex);
	}
	/**
	 * @param columnName
	 * @return
	 * @throws java.sql.SQLException
	 */
	public String getString(String columnName) throws SQLException {
		return master.getString(sqlProcessor.parse(columnName));
	}
	/**
	 * @param columnIndex
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Time getTime(int columnIndex) throws SQLException {
		return master.getTime(columnIndex);
	}
	/**
	 * @param columnIndex
	 * @param cal
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Time getTime(int columnIndex, Calendar cal) throws SQLException {
		return master.getTime(columnIndex, cal);
	}
	/**
	 * @param columnName
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Time getTime(String columnName) throws SQLException {
		return master.getTime(sqlProcessor.parse(columnName));
	}
	/**
	 * @param columnName
	 * @param cal
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Time getTime(String columnName, Calendar cal)
			throws SQLException {
		return master.getTime(sqlProcessor.parse(columnName), cal);
	}
	/**
	 * @param columnIndex
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Timestamp getTimestamp(int columnIndex) throws SQLException {
		return master.getTimestamp(columnIndex);
	}
	/**
	 * @param columnIndex
	 * @param cal
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Timestamp getTimestamp(int columnIndex, Calendar cal)
			throws SQLException {
		return master.getTimestamp(columnIndex, cal);
	}
	/**
	 * @param columnName
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Timestamp getTimestamp(String columnName) throws SQLException {
		return master.getTimestamp(sqlProcessor.parse(columnName));
	}
	/**
	 * @param columnName
	 * @param cal
	 * @return
	 * @throws java.sql.SQLException
	 */
	public Timestamp getTimestamp(String columnName, Calendar cal)
			throws SQLException {
		return master.getTimestamp(sqlProcessor.parse(columnName), cal);
	}
	/**
	 * @return
	 * @throws java.sql.SQLException
	 */
	public int getType() throws SQLException {
		return master.getType();
	}
	/**
	 * @param columnIndex
	 * @return
	 * @throws java.sql.SQLException
	 */
	public InputStream getUnicodeStream(int columnIndex)
			throws SQLException {
		return master.getUnicodeStream(columnIndex);
	}
	/**
	 * @param columnName
	 * @return
	 * @throws java.sql.SQLException
	 */
	public InputStream getUnicodeStream(String columnName)
			throws SQLException {
		return master.getUnicodeStream(sqlProcessor.parse(columnName));
	}
	/**
	 * @param columnIndex
	 * @return
	 * @throws java.sql.SQLException
	 */
	public URL getURL(int columnIndex) throws SQLException {
		return master.getURL(columnIndex);
	}
	/**
	 * @param columnName
	 * @return
	 * @throws java.sql.SQLException
	 */
	public URL getURL(String columnName) throws SQLException {
		return master.getURL(sqlProcessor.parse(columnName));
	}
	/**
	 * @return
	 * @throws java.sql.SQLException
	 */
	public SQLWarning getWarnings() throws SQLException {
		return master.getWarnings();
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return master.hashCode();
	}
	/**
	 * @throws java.sql.SQLException
	 */
	public void insertRow() throws SQLException {
		master.insertRow();
	}
	/**
	 * @return
	 * @throws java.sql.SQLException
	 */
	public boolean isAfterLast() throws SQLException {
		return master.isAfterLast();
	}
	/**
	 * @return
	 * @throws java.sql.SQLException
	 */
	public boolean isBeforeFirst() throws SQLException {
		return master.isBeforeFirst();
	}
	/**
	 * @return
	 * @throws java.sql.SQLException
	 */
	public boolean isFirst() throws SQLException {
		return master.isFirst();
	}
	/**
	 * @return
	 * @throws java.sql.SQLException
	 */
	public boolean isLast() throws SQLException {
		return master.isLast();
	}
	/**
	 * @return
	 * @throws java.sql.SQLException
	 */
	public boolean last() throws SQLException {
		return master.last();
	}
	/**
	 * @throws java.sql.SQLException
	 */
	public void moveToCurrentRow() throws SQLException {
		master.moveToCurrentRow();
	}
	/**
	 * @throws java.sql.SQLException
	 */
	public void moveToInsertRow() throws SQLException {
		master.moveToInsertRow();
	}
	/**
	 * @return
	 * @throws java.sql.SQLException
	 */
	public boolean next() throws SQLException {
		return master.next();
	}
	/**
	 * @return
	 * @throws java.sql.SQLException
	 */
	public boolean previous() throws SQLException {
		return master.previous();
	}
	/**
	 * @throws java.sql.SQLException
	 */
	public void refreshRow() throws SQLException {
		master.refreshRow();
	}
	/**
	 * @param rows
	 * @return
	 * @throws java.sql.SQLException
	 */
	public boolean relative(int rows) throws SQLException {
		return master.relative(rows);
	}
	/**
	 * @return
	 * @throws java.sql.SQLException
	 */
	public boolean rowDeleted() throws SQLException {
		return master.rowDeleted();
	}
	/**
	 * @return
	 * @throws java.sql.SQLException
	 */
	public boolean rowInserted() throws SQLException {
		return master.rowInserted();
	}
	/**
	 * @return
	 * @throws java.sql.SQLException
	 */
	public boolean rowUpdated() throws SQLException {
		return master.rowUpdated();
	}
	/**
	 * @param direction
	 * @throws java.sql.SQLException
	 */
	public void setFetchDirection(int direction) throws SQLException {
		master.setFetchDirection(direction);
	}
	/**
	 * @param rows
	 * @throws java.sql.SQLException
	 */
	public void setFetchSize(int rows) throws SQLException {
		master.setFetchSize(rows);
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return master.toString();
	}
	/**
	 * @param columnIndex
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateArray(int columnIndex, Array x) throws SQLException {
		master.updateArray(columnIndex, x);
	}
	/**
	 * @param columnName
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateArray(String columnName, Array x) throws SQLException {
		master.updateArray(sqlProcessor.parse(columnName), x);
	}
	/**
	 * @param columnIndex
	 * @param x
	 * @param length
	 * @throws java.sql.SQLException
	 */
	public void updateAsciiStream(int columnIndex, InputStream x, int length)
			throws SQLException {
		master.updateAsciiStream(columnIndex, x, length);
	}
	/**
	 * @param columnName
	 * @param x
	 * @param length
	 * @throws java.sql.SQLException
	 */
	public void updateAsciiStream(String columnName, InputStream x,
			int length) throws SQLException {
		master.updateAsciiStream(sqlProcessor.parse(columnName), x, length);
	}
	/**
	 * @param columnIndex
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateBigDecimal(int columnIndex, BigDecimal x)
			throws SQLException {
		master.updateBigDecimal(columnIndex, x);
	}
	/**
	 * @param columnName
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateBigDecimal(String columnName, BigDecimal x)
			throws SQLException {
		master.updateBigDecimal(sqlProcessor.parse(columnName), x);
	}
	/**
	 * @param columnIndex
	 * @param x
	 * @param length
	 * @throws java.sql.SQLException
	 */
	public void updateBinaryStream(int columnIndex, InputStream x,
			int length) throws SQLException {
		master.updateBinaryStream(columnIndex, x, length);
	}
	/**
	 * @param columnName
	 * @param x
	 * @param length
	 * @throws java.sql.SQLException
	 */
	public void updateBinaryStream(String columnName, InputStream x,
			int length) throws SQLException {
		master.updateBinaryStream(sqlProcessor.parse(columnName), x, length);
	}
	/**
	 * @param columnIndex
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateBlob(int columnIndex, Blob x) throws SQLException {
		master.updateBlob(columnIndex, x);
	}
	/**
	 * @param columnName
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateBlob(String columnName, Blob x) throws SQLException {
		master.updateBlob(sqlProcessor.parse(columnName), x);
	}
	/**
	 * @param columnIndex
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateBoolean(int columnIndex, boolean x)
			throws SQLException {
		master.updateBoolean(columnIndex, x);
	}
	/**
	 * @param columnName
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateBoolean(String columnName, boolean x)
			throws SQLException {
		master.updateBoolean(sqlProcessor.parse(columnName), x);
	}
	/**
	 * @param columnIndex
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateByte(int columnIndex, byte x) throws SQLException {
		master.updateByte(columnIndex, x);
	}
	/**
	 * @param columnName
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateByte(String columnName, byte x) throws SQLException {
		master.updateByte(sqlProcessor.parse(columnName), x);
	}
	/**
	 * @param columnIndex
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateBytes(int columnIndex, byte[] x) throws SQLException {
		master.updateBytes(columnIndex, x);
	}
	/**
	 * @param columnName
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateBytes(String columnName, byte[] x)
			throws SQLException {
		master.updateBytes(sqlProcessor.parse(columnName), x);
	}
	/**
	 * @param columnIndex
	 * @param x
	 * @param length
	 * @throws java.sql.SQLException
	 */
	public void updateCharacterStream(int columnIndex, Reader x, int length)
			throws SQLException {
		master.updateCharacterStream(columnIndex, x, length);
	}
	/**
	 * @param columnName
	 * @param reader
	 * @param length
	 * @throws java.sql.SQLException
	 */
	public void updateCharacterStream(String columnName, Reader reader,
			int length) throws SQLException {
		master.updateCharacterStream(sqlProcessor.parse(columnName), reader, length);
	}
	/**
	 * @param columnIndex
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateClob(int columnIndex, Clob x) throws SQLException {
		master.updateClob(columnIndex, x);
	}
	/**
	 * @param columnName
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateClob(String columnName, Clob x) throws SQLException {
		master.updateClob(sqlProcessor.parse(columnName), x);
	}
	/**
	 * @param columnIndex
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateDate(int columnIndex, Date x) throws SQLException {
		master.updateDate(columnIndex, x);
	}
	/**
	 * @param columnName
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateDate(String columnName, Date x) throws SQLException {
		master.updateDate(sqlProcessor.parse(columnName), x);
	}
	/**
	 * @param columnIndex
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateDouble(int columnIndex, double x) throws SQLException {
		master.updateDouble(columnIndex, x);
	}
	/**
	 * @param columnName
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateDouble(String columnName, double x)
			throws SQLException {
		master.updateDouble(sqlProcessor.parse(columnName), x);
	}
	/**
	 * @param columnIndex
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateFloat(int columnIndex, float x) throws SQLException {
		master.updateFloat(columnIndex, x);
	}
	/**
	 * @param columnName
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateFloat(String columnName, float x) throws SQLException {
		master.updateFloat(sqlProcessor.parse(columnName), x);
	}
	/**
	 * @param columnIndex
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateInt(int columnIndex, int x) throws SQLException {
		master.updateInt(columnIndex, x);
	}
	/**
	 * @param columnName
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateInt(String columnName, int x) throws SQLException {
		master.updateInt(sqlProcessor.parse(columnName), x);
	}
	/**
	 * @param columnIndex
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateLong(int columnIndex, long x) throws SQLException {
		master.updateLong(columnIndex, x);
	}
	/**
	 * @param columnName
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateLong(String columnName, long x) throws SQLException {
		master.updateLong(sqlProcessor.parse(columnName), x);
	}
	/**
	 * @param columnIndex
	 * @throws java.sql.SQLException
	 */
	public void updateNull(int columnIndex) throws SQLException {
		master.updateNull(columnIndex);
	}
	/**
	 * @param columnName
	 * @throws java.sql.SQLException
	 */
	public void updateNull(String columnName) throws SQLException {
		master.updateNull(sqlProcessor.parse(columnName));
	}
	/**
	 * @param columnIndex
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateObject(int columnIndex, Object x) throws SQLException {
		master.updateObject(columnIndex, x);
	}
	/**
	 * @param columnIndex
	 * @param x
	 * @param scale
	 * @throws java.sql.SQLException
	 */
	public void updateObject(int columnIndex, Object x, int scale)
			throws SQLException {
		master.updateObject(columnIndex, x, scale);
	}
	/**
	 * @param columnName
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateObject(String columnName, Object x)
			throws SQLException {
		master.updateObject(sqlProcessor.parse(columnName), x);
	}
	/**
	 * @param columnName
	 * @param x
	 * @param scale
	 * @throws java.sql.SQLException
	 */
	public void updateObject(String columnName, Object x, int scale)
			throws SQLException {
		master.updateObject(sqlProcessor.parse(columnName), x, scale);
	}
	/**
	 * @param columnIndex
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateRef(int columnIndex, Ref x) throws SQLException {
		master.updateRef(columnIndex, x);
	}
	/**
	 * @param columnName
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateRef(String columnName, Ref x) throws SQLException {
		master.updateRef(sqlProcessor.parse(columnName), x);
	}
	/**
	 * @throws java.sql.SQLException
	 */
	public void updateRow() throws SQLException {
		master.updateRow();
	}
	/**
	 * @param columnIndex
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateShort(int columnIndex, short x) throws SQLException {
		master.updateShort(columnIndex, x);
	}
	/**
	 * @param columnName
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateShort(String columnName, short x) throws SQLException {
		master.updateShort(sqlProcessor.parse(columnName), x);
	}
	/**
	 * @param columnIndex
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateString(int columnIndex, String x) throws SQLException {
		master.updateString(columnIndex, x);
	}
	/**
	 * @param columnName
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateString(String columnName, String x)
			throws SQLException {
		master.updateString(sqlProcessor.parse(columnName), x);
	}
	/**
	 * @param columnIndex
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateTime(int columnIndex, Time x) throws SQLException {
		master.updateTime(columnIndex, x);
	}
	/**
	 * @param columnName
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateTime(String columnName, Time x) throws SQLException {
		master.updateTime(sqlProcessor.parse(columnName), x);
	}
	/**
	 * @param columnIndex
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateTimestamp(int columnIndex, Timestamp x)
			throws SQLException {
		master.updateTimestamp(columnIndex, x);
	}
	/**
	 * @param columnName
	 * @param x
	 * @throws java.sql.SQLException
	 */
	public void updateTimestamp(String columnName, Timestamp x)	throws SQLException {
		master.updateTimestamp(sqlProcessor.parse(columnName), x);
	}
	/**
	 * @return
	 * @throws java.sql.SQLException
	 */
	public boolean wasNull() throws SQLException {
		return master.wasNull();
	}	
}