/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.sql;

import java.io.IOException;
import java.io.StringReader;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;

import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.config.DomConfigurable;
import biz.hammurapi.config.GenericContainer;
import biz.hammurapi.convert.CompositeConverter;
import biz.hammurapi.xml.dom.DOMUtils;


/**
 * Container, which reads component definitions from rowset.
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public class RowsetConfigurableContainer extends GenericContainer implements RowProcessor {
	public RowsetConfigurableContainer() {
		// Default constructor
	}
	
	public boolean process(ResultSet rs) throws SQLException {
		String value=rs.getString("PARAMETER_VALUE");
		String componentName = rs.getString("NAME");
		String componentClass = rs.getString("CLASS_NAME");
		try {
			// Default class is String
			if (componentClass==null || componentClass.trim().length()==0) {
				addComponent(componentName, value);
			} else {
				Class clazz = Class.forName(componentClass);
				if (value==null || value.trim().length()==0) {
					addComponent(componentName,	clazz.newInstance());
				} else {
					// Attempt to construct class from value
					Constructor[] constructors=clazz.getConstructors();
					for (int i=0; i<constructors.length; i++) {
						if (constructors[i].getParameterTypes().length==1 && constructors[i].getParameterTypes()[0].equals(String.class)) {
							addComponent(componentName,	constructors[i].newInstance(new Object[] {value}));
							return true;
						}
					}
					
					Object instance = clazz.newInstance();
	
					if (DomConfigurable.class.isAssignableFrom(clazz)) {
						((DomConfigurable) instance).configure(DOMUtils.parse(new StringReader(value)).getDocumentElement(), this);
					} else {
						instance = CompositeConverter.getDefaultConverter().convert(instance, clazz, false);
					}
					addComponent(componentName, instance);									
				}
			}
		} catch (InstantiationException e) {
			throw new SQLExceptionEx("Cannot instantiate parameter '"+componentName+"' -> "+componentClass, e);
		} catch (IllegalAccessException e) {
			throw new SQLExceptionEx("Cannot instantiate parameter '"+componentName+"' -> "+componentClass, e);
		} catch (ClassNotFoundException e) {
			throw new SQLExceptionEx("Cannot instantiate parameter '"+componentName+"' -> "+componentClass, e);
		} catch (SecurityException e) {
			throw new SQLExceptionEx("Cannot instantiate parameter '"+componentName+"' -> "+componentClass, e);
		} catch (InvocationTargetException e) {
			throw new SQLExceptionEx("Cannot instantiate parameter '"+componentName+"' -> "+componentClass, e);
		} catch (ConfigurationException e) {
			throw new SQLExceptionEx("Cannot instantiate parameter '"+componentName+"'", e);
		} catch (SAXException e) {
			throw new SQLExceptionEx("Cannot instantiate parameter '"+componentName+"'", e);
		} catch (IOException e) {
			throw new SQLExceptionEx("Cannot instantiate parameter '"+componentName+"'", e);
		} catch (ParserConfigurationException e) {
			throw new SQLExceptionEx("Cannot instantiate parameter '"+componentName+"'", e);
		} catch (FactoryConfigurationError e) {
			throw new SQLExceptionEx("Cannot instantiate parameter '"+componentName+"'", e);
		}
		return true;
	}
}
