/*
  * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
  */
package biz.hammurapi.sql;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.sql.ResultSet;
import java.sql.SQLException;

import biz.hammurapi.config.RuntimeConfigurationException;
import biz.hammurapi.convert.CompositeConverter;
import biz.hammurapi.convert.Converter;


/**
 * Base class for SQLC generated projectors.
 * @author Pavel Vlasov
 *
 * @version $Revision: 1.3 $
 */
public class SmartProjector implements Projector {
    private Converter converter;
	private Class targetClass;
	private Class sourceClass;
	private Constructor sourceClassConstructor;

	public Object project(ResultSet rs) throws SQLException {
		Object ret;
		if (targetClass==null) {
			ret = defaultProject(rs);
		} else {
			if (sourceClass.isAssignableFrom(targetClass)) {
				ret=project(rs, targetClass);
			} else {
				ret=CompositeConverter.getDefaultConverter().convert(defaultProject(rs), targetClass, false);
			}
		}
		
		if (converter==null) {
			return ret;
		}
		
		return converter.convert(ret);
    }

    /**
	 * @param rs
	 * @param ret
	 * @return
	 * @throws SQLException
	 */
	private Object defaultProject(ResultSet rs) throws SQLException {
		try {
			return sourceClassConstructor.newInstance(new Object[] {rs});
		} catch (IllegalArgumentException e) {
			throw new SQLExceptionEx(e);
		} catch (InstantiationException e) {
			throw new SQLExceptionEx(e);
		} catch (IllegalAccessException e) {
			throw new SQLExceptionEx(e);
		} catch (InvocationTargetException e) {
			throw new SQLExceptionEx(e);
		}
	}

	/**
	 * @param rs
	 * @return
     * @throws SQLException
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 * @throws NoSuchMethodException
	 */
	private Object project(ResultSet rs, Class clazz) throws SQLException {			
		try {
			return clazz.getConstructor(new Class[] {ResultSet.class}).newInstance(new Object[] {rs});
		} catch (IllegalArgumentException e) {
			throw new SQLExceptionEx(e);
		} catch (SecurityException e) {
			throw new SQLExceptionEx(e);
		} catch (InstantiationException e) {
			throw new SQLExceptionEx(e);
		} catch (IllegalAccessException e) {
			throw new SQLExceptionEx(e);
		} catch (InvocationTargetException e) {
			throw new SQLExceptionEx(e);
		} catch (NoSuchMethodException e) {
			throw new SQLExceptionEx(e);
		}
	}

	public SmartProjector(Class sourceClass) {
    	this.sourceClass=sourceClass;
    	try {
			this.sourceClassConstructor=sourceClass.getConstructor(new Class[] {ResultSet.class});
		} catch (SecurityException e) {
			throw new RuntimeConfigurationException("Caused by: "+e, e);
		} catch (NoSuchMethodException e) {
			throw new RuntimeConfigurationException("Constructor from ResultSet not found in "+sourceClass+": "+e, e);
		}
    }
    
    public SmartProjector(Class sourceClass, Class targetClass, Converter converter) {
    	this.converter=converter;
    	this.targetClass=targetClass;
    	this.sourceClass=sourceClass;
    	try {
			this.sourceClassConstructor=sourceClass.getConstructor(new Class[] {ResultSet.class});
		} catch (SecurityException e) {
			throw new RuntimeConfigurationException("Caused by: "+e, e);
		} catch (NoSuchMethodException e) {
			throw new RuntimeConfigurationException("Constructor from ResultSet not found in "+sourceClass+": "+e, e);
		}
    }	    
}
