package biz.hammurapi.sql;

import java.sql.Connection;
import java.sql.SQLException;
import java.text.MessageFormat;

/**
 * Generates identities using SQLProcessor nextPK method
 * @author Pavel Vlasov
 *
 */
public class TableIdentityGenerator implements IdentityGenerator {
	
	/**
	 * Default constructor with "{0}" pattern
	 */
	public TableIdentityGenerator(String primaryKeysTable) {
		this(primaryKeysTable, "{0}");
	}
	
	private int mode; // 0 - passthrough 1 - invariant 2 - format
	private MessageFormat mf;
	private String keyName;
	private String primaryKeysTable;
	private Object[] args=new Object[] {null};
	
	/**
	 * Constructor
	 * @param primaryKeysTable - Table which hold primary keys. See SQLProcessor documentation.
	 * @param pattern - Key name pattern. {0} stands for table name.
	 */
	public TableIdentityGenerator(String primaryKeysTable, String pattern) {
		if ("{0}".equals(pattern)) {
			mode=0;
		} else if (pattern.indexOf("{0}")==-1) {
			mode=1;
			this.keyName=pattern;
		} else {
			mode=2;
			mf=new MessageFormat(pattern);
		}
		this.primaryKeysTable=primaryKeysTable;
	}

	public int generate(Connection con, String name) throws SQLException {
		String ename;
		switch (mode) {
		case 0: 
			ename=name;
			break;
		case 1:
			ename=keyName;
			break;
		case 2:
			synchronized (mf) {
				args[0]=name;
				ename=mf.format(args, new StringBuffer(), null).toString();
			}
		default:
			throw new IllegalStateException("Invalid mode: "+mode);
		}
		
		return new SQLProcessor(con, null).nextPK(primaryKeysTable, ename);
	}

}
