/*
 * hgee 2
 * Hammurapi Group Enterprise Extensions 
 * Copyright (C) 2006  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/products/enterpriseextensions/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.sql.cloudscape;

import java.io.File;
import java.io.IOException;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.Date;


/**
 * Hypersonic temporary data source. It is similar to standalone datasource, but
 * data files are created in system temporary directory and are scheduled to deletion 
 * on JVM exit. Use this datasource if you need to keep large amount of temporary data,
 * for example data pumping applications can benefit from such data source.
 * @author Pavel Vlasov
 * @version $Revision: 1.2 $
 */
public class CloudscapeTmpDataSource extends CloudscapeDataSource {
	
	/** 
	 * @param initScript Fully qualified name of database initialization script to be loaded by 
	 * classloader. Can be null.
	 * @throws ClassNotFoundException
	 * @throws IOException
	 * @throws SQLException
	 */
	public CloudscapeTmpDataSource(String initScript) throws ClassNotFoundException, IOException, SQLException {		
		super("org.apache.derby.jdbc.EmbeddedDriver", "jdbc:derby:"+createTmpDir()+";create=true", "sa", "sa", null);
		initDB(initScript);
	}
	
	private static String createTmpDir() throws IOException {
		String tmpDir=System.getProperty("java.io.tmpdir");
		SimpleDateFormat sdf=new SimpleDateFormat("yyyyMMddHHmmss");
		String prefix = "CloudscapeDB_"+sdf.format(new Date())+"_";
		String suffix = "_TMP";
		final File tmpDbDir = tmpDir==null ? File.createTempFile(prefix, suffix) : File.createTempFile(prefix, suffix, new File(tmpDir));
		if (tmpDbDir.delete()) {
			if (tmpDbDir.mkdirs()) {
				Runtime.getRuntime().addShutdownHook(	new Thread() {
					public void run() {						
						
						File[] dbFiles=tmpDbDir.listFiles();
						for (int i=0; i<dbFiles.length; i++) {
							dbFiles[i].deleteOnExit();
						}
						
						tmpDbDir.deleteOnExit();
					}
				});
				return tmpDbDir.getAbsolutePath()+File.separator+"CloudscapeTmpDB";
			} else { 
				throw new IOException("Cannot create directory "+tmpDbDir.getAbsolutePath());
			}
		} else {
			throw new IOException("Cannot delete file "+tmpDbDir.getAbsolutePath());
		}
	}	
}
