/*
  * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
  */
package biz.hammurapi.sql.columns;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

import biz.hammurapi.config.Context;
import biz.hammurapi.convert.CompositeConverter;


/**
 * @author Pavel Vlasov
 *
 * @version $Revision: 1.12 $
 */
public class BooleanColumn extends Column {
	private boolean value;
	
	// Original stuff	
	private boolean originalValue;
	private boolean isOriginalValueSet;
	
	public boolean getOriginalValue() {
		return isOriginalValueSet ? originalValue : value;
	}
	
	public void parameterizeOriginal(PreparedStatement ps, int idx) throws SQLException {
		ps.setBoolean(idx, getOriginalValue());
	}
	
	public void setOriginal() {
		originalValue=value;
		isOriginalValueSet=true;
	}
	// End of original stuff
	
	public boolean getValue() {
		return value;
	}
	
	public void setValue(boolean value) {
		if (force || this.value!=value) {
			this.value = value;
			onChange();
		}
	}
	
	public BooleanColumn(String name, boolean isPrimaryKey) {
		super(name, isPrimaryKey);
	}

	public BooleanColumn(String name, boolean isPrimaryKey, ResultSet rs) throws SQLException {
		super(name, isPrimaryKey);
		ResultSetMetaData metaData = rs.getMetaData();
		for (int i=1, c=metaData.getColumnCount(); i<=c; i++) {
			if (name.equals(metaData.getColumnName(i))) {
				this.value=rs.getBoolean(i);
				break;
			}
		}
	}
	
	public BooleanColumn(String name, boolean isPrimaryKey, boolean value) {
		super(name, isPrimaryKey);
		this.value=value;
	}
	
	protected void parameterizeInternal(PreparedStatement ps, int idx) throws SQLException {
		ps.setBoolean(idx, value);
	}

	public Object getObjectValue(boolean ov) {
		if (ov) {
			return isOriginalValueSet ? new Boolean(originalValue) : null;
		}
		return new Boolean(value);
	}
	
	public String toString() {
		return getName()+(isModified() ? "*" : "")+"="+value;
	}
	
    public boolean equals(Object otherColumn) {
        if (otherColumn instanceof BooleanColumn) {
        	return value==((BooleanColumn) otherColumn).value;
        }
        
		return false;
    }	
    
	public int hashCode() {
		return getName().hashCode() ^ (value ? 0 : 1);
	}

	/**
	 * @param textValue "YES", "yes", "TRUE", "true" yield true. All other values yield false.
	 */
	public void load(String textValue) {
		setValue("yes".equalsIgnoreCase(textValue) || "true".equalsIgnoreCase(textValue));
	}

	public void clear() {
		setValue(false);
		clearModified();
	}

	public void configure(Context context, CompositeConverter converter) {
		Object o=context.get(getName());
		if (o!=null) {
			setValue(((Boolean) converter.convert(o, boolean.class, false)).booleanValue());
		}
		
	}

	protected String getType() {
		return "boolean";
	}

	public void set(Column source) {
		setValue(((BooleanColumn) source).getValue());
	}
	
    /**
     * Clears modified flag and sets original value to current value.
     */
    public void clearModified() {
    	super.clearModified();
    	originalValue=value;
    }
	
}
