/*
  * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
  */
package biz.hammurapi.sql.columns;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

import biz.hammurapi.config.Context;
import biz.hammurapi.convert.CompositeConverter;


/**
 * @author Pavel Vlasov
 *
 * @version $Revision: 1.12 $
 */
public class FloatColumn extends Column {
	private float value;
	
	// Original stuff	
	private float originalValue;
	private boolean isOriginalValueSet;
	
	public float getOriginalValue() {
		return isOriginalValueSet ? originalValue : value;
	}
	
	public void parameterizeOriginal(PreparedStatement ps, int idx) throws SQLException {
		ps.setFloat(idx, getOriginalValue());
	}
	
	public void setOriginal() {
		originalValue=value;
		isOriginalValueSet=true;
	}
	// End of original stuff
	
	public float getValue() {
		return value;
	}
	
	public void setValue(float value) {
		if (force || this.value!=value) {
			this.value = value;
			onChange();
		}
	}
	
	public FloatColumn(String name, boolean isPrimaryKey) {
		super(name, isPrimaryKey);
	}

	public FloatColumn(String name, boolean isPrimaryKey, float value) {
		super(name, isPrimaryKey);
		this.value=value;
	}
	
	public FloatColumn(String name, boolean isPrimaryKey, ResultSet rs) throws SQLException {
		super(name, isPrimaryKey);
		ResultSetMetaData metaData = rs.getMetaData();
		for (int i=1, c=metaData.getColumnCount(); i<=c; i++) {
			if (name.equals(metaData.getColumnName(i))) {
				this.value=rs.getFloat(i);
				break;
			}
		}
	}	
	
	protected void parameterizeInternal(PreparedStatement ps, int idx) throws SQLException {
		ps.setFloat(idx, value);
	}

	public Object getObjectValue(boolean ov) {
		if (ov) {
			return isOriginalValueSet ? new Float(originalValue) : null;
		}
		return new Float(value);
	}
	
	public String toString() {
		return getName()+(isModified() ? "*" : "")+"="+value;
	}
	
    public boolean equals(Object otherColumn) {
        if (otherColumn instanceof FloatColumn) {
        	return value==((FloatColumn) otherColumn).value;
        }
        
		return false;
    }	

	public int hashCode() {
		return getName().hashCode() ^ new Float(value).hashCode();
	}

	/* (non-Javadoc)
	 * @see biz.hammurapi.sql.columns.Column#load(java.lang.String)
	 */
	public void load(String textValue) {
		setValue(Float.parseFloat(textValue));
	}		
	
	public void clear() {
		setValue(0);
		clearModified();
	}
	
	public void configure(Context context, CompositeConverter converter) {
		Object o=context.get(getName());
		if (o!=null) {
			setValue(((Number) converter.convert(o, float.class, false)).floatValue());
		}		
	}		
	
	protected String getAlignment() {
		return "right";
	}	

	protected String getType() {
		return "float";
	}	

	public void set(Column source) {
		setValue(((FloatColumn) source).getValue());
	}		
	
    /**
     * Clears modified flag and sets original value to current value.
     */
    public void clearModified() {
    	super.clearModified();
    	originalValue=value;
    }
}
