/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.sql.hypersonic;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URL;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import biz.hammurapi.config.Component;
import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.config.Context;
import biz.hammurapi.config.DomConfigurable;
import biz.hammurapi.config.Wrapper;
import biz.hammurapi.util.StreamPumper;
import biz.hammurapi.xml.dom.DOMUtils;


public abstract class HypersonicDataSourceComponent implements Wrapper, DomConfigurable, Component {
	
	private byte[] scriptBuf;
	
	protected Reader getReader() {
		try {
			return new InputStreamReader(new ByteArrayInputStream(scriptBuf));
		} finally {
			scriptBuf=null;
		}
	}

	public void configure(Node configNode, Context context) throws ConfigurationException {
		ByteArrayOutputStream baos=new  ByteArrayOutputStream();
		NodeList children=configNode.getChildNodes();
		for (int i=0, length=children.getLength(); i<length; i++) {
			Node child=children.item(i);
			if (child instanceof Element) {
				try {
					String value=DOMUtils.getElementText((Element) child);
					InputStream in;
					
					if ("file".equals(child.getNodeName())) {
						 in = new FileInputStream(new File(value));
					} else if ("resource".equals(child.getNodeName())) {
						 in = getClass().getClassLoader().getResourceAsStream(value);
						 if (in==null) {
							 throw new ConfigurationException("Resource not found: "+value);
						 }
					} else if ("url".equals(child.getNodeName())) {
						 in = new URL(value).openStream();
					} else {
						continue;
					}
					
					new StreamPumper(in, baos, null, false).run();
					in.close();
					
				} catch (Exception e) {
					throw new ConfigurationException("Could not load script: "+e, e);
				}				
			}
		}
		try {
			baos.close();
		} catch (IOException e) {
			throw new ConfigurationException("Should never happen: "+e, e);
		}
		scriptBuf=baos.toByteArray();
	}

	public void setOwner(Object owner) {
		// No functionality
	}
}
