/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.sql.hypersonic;

import java.io.File;
import java.io.IOException;
import java.sql.SQLException;

import biz.hammurapi.sql.SQLProcessor;
import biz.hammurapi.sql.Transaction;



/**
 * Hypersonic standalone data source.
 * @author Pavel Vlasov	
 * @version $Revision: 1.5 $
 */
public class HypersonicStandaloneDataSource extends HypersonicDataSource {

	/**
	 * Constructor. Uses "sa" as user name and blank password. Creates a database if one doesn't exist.
	 * @param dbName - Database name. E.g. C:\myproject\myDB.
	 * @throws ClassNotFoundException
	 * @throws IOException
	 * @throws SQLException
	 */
	public HypersonicStandaloneDataSource(String dbName, Transaction databaseInitTransaction) throws ClassNotFoundException, SQLException {
		this(dbName, "sa", "", databaseInitTransaction, null);
	}
	
	/**
	 * Constructor. Creates a database if one doesn't exist.
	 * @param dbName - Database name. E.g. C:\myproject\myDB.
	 * @param initTransaction - Transaction to initialize database if it doesn't exist
	 * @throws ClassNotFoundException
	 * @throws IOException
	 * @throws SQLException
	 */
	public HypersonicStandaloneDataSource(String dbName, String user, String password, Transaction databaseInitTransaction, Transaction connectionInitTransaction) throws ClassNotFoundException, SQLException {
		super("jdbc:hsqldb:"+dbName, user, password, connectionInitTransaction);
		File dbFile=new File(dbName+".properties");
		if (databaseInitTransaction!=null && !dbFile.exists()) {
			new SQLProcessor(this, null).executeTransaction(databaseInitTransaction);
		}
	}			
}
