/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.sql.hypersonic;

import java.io.IOException;
import java.sql.SQLException;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import biz.hammurapi.CarryOverException;
import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.config.Context;
import biz.hammurapi.sql.SQLProcessor;
import biz.hammurapi.sql.Transaction;


public class HypersonicStandaloneDataSourceComponent extends HypersonicDataSourceComponent {

	private HypersonicStandaloneDataSource master;

	public HypersonicStandaloneDataSourceComponent() {
		super();
	}

	public Object getMaster() {
		if (master==null) {
			throw new IllegalStateException("Not yet started");
		}
		return master;
	}
	
	private String location;

	public void configure(Node configNode, Context context) throws ConfigurationException {
		super.configure(configNode, context);
		location=((Element) configNode).getAttribute("location");
	}

	public void start() throws ConfigurationException {
		try {
			master=new HypersonicStandaloneDataSource(
					location,
					new Transaction() {

						public boolean execute(SQLProcessor processor) throws SQLException {
							try {
								processor.executeScript(getReader());
							} catch (IOException e) {
								throw new CarryOverException(e);
							}
							return true;
						}
						
					}); 
		} catch (ClassNotFoundException e) {
			throw new ConfigurationException("Driver class not found", e);
		} catch (SQLException e) {
			throw new ConfigurationException("Could not initialize database: "+e, e);
		} catch (CarryOverException e) {
			throw new ConfigurationException("Could not initialize databse: "+e.getCause(), e.getCause());
		}
	}

	public void stop() throws ConfigurationException {
		if (master!=null) {
			master.shutdown();
		}
	}
}
