/*
  * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz 
 */
package biz.hammurapi.sql.xml;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.config.Context;
import biz.hammurapi.config.DomConfigFactory;
import biz.hammurapi.sql.SQLProcessor;
import biz.hammurapi.xml.dom.DOMUtils;
import biz.hammurapi.xml.dom.AbstractDomObject;


/**
 * @author Pavel Vlasov
 * @version $Revision: 1.2 $
 */
public class CompositeCommand extends AbstractDomObject implements Command {
	private Collection commands=new ArrayList();

	public void addCommand(Command command) {
		commands.add(command);	
	}
	
	private SQLProcessor processor;
	
	public Document execute(Document document, SQLProcessor processor, Context context) {
		Element element=document.createElement("sql2xml");
		document.appendChild(element);
		try {
			execute(element, processor, context);
		} catch (SQL2XMLException e) {
			log(e);
			Element errorElement=element.getOwnerDocument().createElement("error");
			toDom(e, errorElement);
			element.setAttribute("result", "error");
			element.appendChild(errorElement);
		}
		return document;
	}
	
	/**
	 * Override for proper logging
	 * @param e
	 */
	public void log(SQL2XMLException e) {
		e.printStackTrace();
	}
	
	public void execute(Element holder, SQLProcessor processor, Context context) throws SQL2XMLException {
		Iterator it=commands.iterator();
		while (it.hasNext()) {
			((Command) it.next()).execute(holder, this.processor==null ? processor : this.processor, context);
		}
	}
	
	
	public void configure(Node configNode, Context context) throws ConfigurationException {
		try {
			// TODO Handle <transaction> and <update> here as well
			NodeList nl = DOMUtils.selectNodeList(configNode, "query|update|transaction|message|java");
			for (int i=0, l=nl.getLength(); i<l; ++i) {
				Node n = (Element) nl.item(i);
				if ("query".equals(n.getNodeName())) {
					Select select=new Select();
					select.configure(n, context);
					addCommand(select);
				} else if ("update".equals(n.getNodeName())) {
					Update update=new Update();
					update.configure(n, context);
					addCommand(update);					
				} else if ("transaction".equals(n.getNodeName())) {
					Transaction transaction=new Transaction();
					transaction.configure(n, context);
					addCommand(transaction);
				} else if ("message".equals(n.getNodeName())) {
					Message message=new Message();
					message.configure(n, context);
					addCommand(message);
				} else if ("java".equals(n.getNodeName())) {
					DomConfigFactory factory=new DomConfigFactory(this.getClass().getClassLoader(), context);
					factory.create(n);
				}
				
				processor=SQLCommand.initProcessor(configNode);
			}
		} catch (Exception e) {
			throw new ConfigurationException(e);
		}
	}
	
	public void toDom(Throwable throwable, Element holder) {
		holder.setAttribute("type", throwable.getClass().getName());
		if (throwable.getMessage()!=null) {
			holder.setAttribute("message", throwable.getMessage());
		}
		
		Document ownerDocument = holder.getOwnerDocument();
		StackTraceElement[] frames=throwable.getStackTrace();
		for (int i=0; i<frames.length; i++) {
			Element frameElement=ownerDocument.createElement("frame");
			holder.appendChild(frameElement);
			frameElement.appendChild(ownerDocument.createTextNode(frames[i].toString()));
		}
		
		if (throwable.getCause()!=null) {
			Element causeElement=ownerDocument.createElement("cause");
			holder.appendChild(causeElement);
			toDom(throwable.getCause(), causeElement);
		}
	}
}
