/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.sql.xml;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.naming.InitialContext;
import javax.sql.DataSource;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.config.Context;
import biz.hammurapi.config.DomConfigurable;
import biz.hammurapi.sql.SQLProcessor;
import biz.hammurapi.xml.dom.AbstractDomObject;


/**
 * @author Pavel Vlasov
 * @version $Revision: 1.2 $
 */
public class SQLCommand extends AbstractDomObject implements DomConfigurable {

	protected static class Parameterizer implements biz.hammurapi.sql.Parameterizer {
		private static class Parameter {
			String name;
			Object value;
		}
		
		Parameter[] parameters;
		
		Parameterizer(Collection parameterNames) {
			parameters=new Parameter[parameterNames.size()];			
			Iterator it=parameterNames.iterator();
			for (int i=0; it.hasNext(); i++) {
				parameters[i]=new Parameter();
				parameters[i].name = (String) it.next();
			}
		}

		public void parameterize(PreparedStatement ps) throws SQLException {
			for (int i=0; i<parameters.length; i++) {
				ps.setObject(i+1, parameters[i].value);
			}
			
		}
		
		boolean setContext(Context context) {
			for (int i=0; i<parameters.length; i++) {
				Object o = context.get(parameters[i].name);
				parameters[i].value = o;
				if (parameters[i].value==null) {
					return false;
				}
			}
			return true;
		}		
	}

	protected String name;
	protected String sql;
	protected Parameterizer parameterizer;
	protected String errorMessage;
	private SQLProcessor processor;
	
	protected SQLProcessor getProcessor() {
		return processor;
	}

	public void configure(Node configNode, Context context) throws ConfigurationException {
		Element configElement=(Element) configNode;
		if (configElement.hasAttribute("name")) {
			name=configElement.getAttribute("name");
		}
		
		if (configElement.hasAttribute("error-message")) {
			errorMessage=configElement.getAttribute("error-message");
		}
		
		processor=initProcessor(configNode); 
		
		try {
			sql=getElementText(configElement, "sql");
			if (sql==null) {
				throw new ConfigurationException("<sql> element not found");
			}
			
			Collection parameters=new ArrayList();
			StringBuffer sqlBuf=new StringBuffer(sql);
			for (int i=sqlBuf.indexOf("?["); i!=-1; i=sqlBuf.indexOf("?[")) {
				int j=sqlBuf.indexOf("]?", i+2);
				if (j==-1) {
					break;
				} else {
					parameters.add(sqlBuf.substring(i+2, j));
					sqlBuf.delete(i+1, j+2);
				}
			}
			sql=sqlBuf.toString();
			if (!parameters.isEmpty()) {
				parameterizer=new Parameterizer(parameters);
			}
		} catch (Exception e) {
			throw new ConfigurationException(e);
		}		
	}

	/**
	 * @param configNode
	 * @throws ConfigurationException
	 */
	static SQLProcessor initProcessor(Node configNode) throws ConfigurationException {
		try {
			String jndiName=getElementText((Element) configNode, "data-source");
			if (jndiName==null) {
				return null;
			} else {
				DataSource ds=(DataSource) new InitialContext().lookup(jndiName);
				return new SQLProcessor(ds, null);
			}
		} catch (Exception e) {
			throw new ConfigurationException("Cannot read datasource JNDI name", e);			
		}
	}

}
