/*
  * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz 
 */
package biz.hammurapi.sql.xml;

import java.io.ByteArrayInputStream;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import biz.hammurapi.CarryOverException;
import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.config.Context;
import biz.hammurapi.config.DomConfigFactory;
import biz.hammurapi.config.PropertyParser;
import biz.hammurapi.sql.MetadataAwareRowProcessor;
import biz.hammurapi.sql.SQLProcessor;
import biz.hammurapi.xml.dom.DOMUtils;


/**
 * @author Pavel Vlasov
 * @version $Revision: 1.2 $
 */
public class Select extends SQLCommand implements Command {
	
	//	private String pageSize;
//	private String pageNumParameter;
	private CompositeCommand onRow;
	private CompositeCommand onEmptySet;
	
	public void configure(Node configNode, Context context) throws ConfigurationException {
		super.configure(configNode, context);
		try {
			Node n=DOMUtils.selectSingleNode(configNode, "on-row");
			if (n!=null) {
				onRow=new CompositeCommand();
				onRow.configure(n, context);
			}
			
			n=DOMUtils.selectSingleNode(configNode, "on-empty-result-set");
			if (n!=null) {
				onEmptySet=new CompositeCommand();
				onEmptySet.configure(n, context);
			}
		} catch (Exception e) {
			throw new ConfigurationException(e);
		}		
	}
	
	public void execute(final Element holder, SQLProcessor pProcessor, final Context context) throws SQL2XMLException {
		final SQLProcessor processor = getProcessor()==null ? pProcessor : getProcessor();
		// TODO Paged processing
		try {
			if (parameterizer==null || parameterizer.setContext(context)) {
				final Document ownerDocument = holder.getOwnerDocument();
				final Element resultSetElement=ownerDocument.createElement("result-set");
				holder.appendChild(resultSetElement);
				if (name!=null) {
					resultSetElement.setAttribute("name", name);
				}

				processor.processSelect(
						sql,
						parameterizer,
						new MetadataAwareRowProcessor() {
							public boolean process(final ResultSet rs) throws SQLException {
								Element rowElement=ownerDocument.createElement("row");
								resultSetElement.appendChild(rowElement);
								for (int i=1, j=rs.getMetaData().getColumnCount(); i<=j; i++) {
									Element columnElement=ownerDocument.createElement("column");
									rowElement.appendChild(columnElement);
									columnElement.setAttribute("name", rs.getMetaData().getColumnName(i));
									columnElement.setAttribute("type", rs.getMetaData().getColumnTypeName(i));
									switch (rs.getMetaData().isNullable(i)) {
										case ResultSetMetaData.columnNoNulls:
											columnElement.setAttribute("nullable", "no");
											break;
										case ResultSetMetaData.columnNullable:
											columnElement.setAttribute("nullable", "yes");
											break;
										case ResultSetMetaData.columnNullableUnknown:
											columnElement.setAttribute("nullable", "unknown");
											break;																						
									}
									String value = rs.getString(i);
									if (value!=null) {
										columnElement.appendChild(ownerDocument.createTextNode(value));
									}
									
								}
								
								try {
									if (onRow!=null) {
										onRow.execute(
											rowElement, 
											processor, 
											new Context() {
												public Object get(String name) {
													//System.out.println("Context.get("+name+")");
													if (Select.this.name!=null && name.startsWith(Select.this.name+".")) {
														name=name.substring(Select.this.name.length()+1);
														return findColumn(name);
													} else if (name.indexOf('.')!=-1) {
														return context==null ? null : context.get(name);
													} else {
														Object ret=findColumn(name);
														return ret==null ? context==null ? null : context.get(name) : ret; 
													}
												}
	
												/**
												 * @param rs
												 * @param name
												 * @return
												 */
												private Object findColumn(String name) {
													try {
														for (int i=1; i<=rs.getMetaData().getColumnCount(); i++) {
															if (name.equals(rs.getMetaData().getColumnName(i))) {
																return rs.getObject(i);
															}
														}
														return null;
													} catch (SQLException e) {
														throw new CarryOverException(e);
													}
												}
												
											});
									}
								} catch (SQL2XMLException e) {
									throw new CarryOverException(e);
								}
	
								return true;
							}

							public void processMetadata(ResultSetMetaData metadata) throws DOMException, SQLException {
								Element metadataElement=ownerDocument.createElement("metadata");
								resultSetElement.appendChild(metadataElement);
								for (int i=1; i<=metadata.getColumnCount(); i++) {
									Element columnElement=ownerDocument.createElement("column");
									metadataElement.appendChild(columnElement);
									columnElement.setAttribute("name", metadata.getColumnName(i));
									columnElement.setAttribute("type", metadata.getColumnTypeName(i));
								}
							}

							public void onEmptyResultSet() throws SQLException {
								try {
									if (onEmptySet!=null) {
										onEmptySet.execute(holder, processor, context);
									}
								} catch (SQL2XMLException e) {
									throw new CarryOverException(e);
								}	
							}
						});
			}
		} catch (final SQLException e) {
			throw new SQL2XMLException(
					new PropertyParser(
							new Context() {
								public Object get(String name) {
									if ("error.message".equals(name)) {
										return e.getMessage();
									} else if ("error.class".equals(name)) {
										return e.getClass().getName();
									} else {
										return context.get(name);
									}
								}
							}, false).parse(errorMessage), e);
		} catch (final CarryOverException e) {
			if (e.getCause() instanceof SQL2XMLException) {
				throw (SQL2XMLException) e.getCause();
			} else {
				throw new SQL2XMLException(
						new PropertyParser(
								new Context() {
									public Object get(String name) {
										if ("error.message".equals(name)) {
											return e.getCause().getMessage();
										} else if ("error.class".equals(name)) {
											return e.getCause().getClass().getName();
										} else {
											return context.get(name);
										}
									}
								}, false).parse(errorMessage), e.getCause());
			}
		}
	}
	
	public static void main(String[] args) throws Exception {
		DomConfigFactory factory=new DomConfigFactory();
		String xml = "<query type=\"biz.hammurapi.sql.xml.Select\">" +
				"<sql>select * from people where last_name=?[lastName]?</sql>" +
				"</query>";
		Command select= (Command) factory.create(new ByteArrayInputStream(xml.getBytes()), "/query");
		
	}
}
