/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.sql.xml;

import java.sql.Connection;
import java.sql.SQLException;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.config.Context;
import biz.hammurapi.config.PropertyParser;
import biz.hammurapi.sql.SQLProcessor;


/**
 * @author Pavel Vlasov
 * @version $Revision: 1.2 $
 */
public class Transaction extends CompositeCommand {

	private String errorMessage;

	public void configure(Node configNode, Context context)	throws ConfigurationException {
		super.configure(configNode, context);
		Element configElement=(Element) configNode;
		
		if (configElement.hasAttribute("error-message")) {
			errorMessage=configElement.getAttribute("error-message");
		}
		
	}
	
	public void execute(Element holder, SQLProcessor processor, final Context context) throws SQL2XMLException {
		try {
			Connection connection = processor.getConnection();
			boolean autoCommit=connection.getAutoCommit();
			try {
				connection.setAutoCommit(false);
				super.execute(holder, new SQLProcessor(connection, null), context);
				connection.commit();
			} catch (SQLException e) {
				connection.rollback();
				throw e;
			} catch (SQL2XMLException e) {
				connection.rollback();
				throw e;
			} catch (RuntimeException e) {
				connection.rollback();
				throw e;
			} finally {
				connection.setAutoCommit(autoCommit);
				connection.close();
			}
		} catch (final SQLException e) {
			throw new SQL2XMLException(
					new PropertyParser(
							new Context() {
								public Object get(String name) {
									if ("error.message".equals(name)) {
										return e.getMessage();
									} else if ("error.class".equals(name)) {
										return e.getClass().getName();
									} else {
										return context.get(name);
									}
								}
							}, false).parse(errorMessage), e);
		}		
	}
}
