/*
  * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
  */
package biz.hammurapi.swing;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.MutableTreeNode;


/**
 * @author Pavel Vlasov
 *
 * @version $Revision: 1.1 $
 */
public class BeanVisualizer {
	
	public Visualizable toVisualizable(final Object obj) {
		return new Visualizable() {

			public MutableTreeNode toTree(final String title) {
				DefaultMutableTreeNode ret=new DefaultMutableTreeNode(obj) {
					public String toString() {
						return title+" ["+obj.getClass().getName()+"] "+obj;
					}
				}; 
				
				return ret;
			}

			public TableModel toTable() {
				int rowCount=2;
				DefaultTableModel tm=new DefaultTableModel(rowCount,4);			
				tm.setColumnIdentifiers(new String[] {"Property", "Declared type", "Runtime type", "Value"});
				
			    Class beanClass=obj.getClass();
			    
				tm.setValueAt("this", 0, 0);
				tm.setValueAt(beanClass.getName(), 0, 2);
				tm.setValueAt(obj, 0, 3);
				
				tm.setValueAt("Hash code", 1, 0);
				tm.setValueAt("int", 1, 1);
				tm.setValueAt("int", 1, 2);
				tm.setValueAt(Integer.toString(obj.hashCode(), Character.MAX_RADIX), 1, 3);
				
			    Method[] methods = beanClass.getMethods();
			    for (int i=0; i<methods.length; i++) {
			    	// getXXX() methods. Object.getClass() is not included.
			    	if (!(methods[i].getDeclaringClass().equals(Object.class)) && methods[i].getName().startsWith("get") && methods[i].getParameterTypes().length==0 ) {
			    		try {
							Object value=methods[i].invoke(obj, null);
							tm.setRowCount(++rowCount);
							int idx = rowCount-1;
							tm.setValueAt(methods[i].getName().substring(3), idx, 0);
							tm.setValueAt(methods[i].getReturnType().getName(), idx, 1);
							if (value==null) {
								tm.setValueAt("(null)", idx, 2);
								tm.setValueAt("(null)", idx, 3);
							} else {
								tm.setValueAt(value.getClass().getName(), idx, 2);
								tm.setValueAt(value, idx, 3);
							}
						} catch (IllegalArgumentException e) {
							e.printStackTrace();								
						} catch (IllegalAccessException e) {
							e.printStackTrace();
						} catch (InvocationTargetException e) {
							e.printStackTrace();
						}
			    	}					    						    	
			    }
			    
			    Field[] fields = beanClass.getFields();
			    for (int i=0; i<fields.length; i++) {
		    		try {
						Object value=fields[i].get(obj);								
						tm.setRowCount(++rowCount);
						int idx = rowCount-1;
						tm.setValueAt(fields[i].getName().substring(3), idx, 0);
						tm.setValueAt(fields[i].getType().getName(), idx, 1);
						if (value==null) {
							tm.setValueAt("(null)", idx, 2);
							tm.setValueAt("(null)", idx, 3);
						} else {
							tm.setValueAt(value.getClass().getName(), idx, 2);
							tm.setValueAt(value, idx, 3);
						}
					} catch (IllegalArgumentException e) {
						e.printStackTrace();								
					} catch (IllegalAccessException e) {
						e.printStackTrace();
					}
			    }
			    
				return tm;
			}
			
		};
		
	}
}
