/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.swing;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.WindowConstants;
import javax.swing.table.TableModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
/**
 * @author Pavel Vlasov
 * 
 * @version $Revision: 1.2 $
 */
public class Browser extends JFrame {

	private javax.swing.JPanel jContentPane = null;

	private JSplitPane jSplitPane = null;

	private JScrollPane jScrollPane = null;

	private JScrollPane jDetailsPane = null;

	private JTree jTree = null;

	private JTable jTable = null;
	/**
	 * This method initializes jSplitPane
	 * 
	 * @return javax.swing.JSplitPane
	 */
	private JSplitPane getJSplitPane() {
		if (jSplitPane == null) {
			jSplitPane = new JSplitPane();
			jSplitPane.setLeftComponent(getJScrollPane());
			jSplitPane.setRightComponent(getJScrollPane1());
			jSplitPane.setDividerLocation(400);
		}
		return jSplitPane;
	}

	/**
	 * This method initializes jScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setViewportView(getJTree());
		}
		return jScrollPane;
	}

	/**
	 * This method initializes jScrollPane1
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPane1() {
		if (jDetailsPane == null) {
			jDetailsPane = new JScrollPane();
			jDetailsPane.setViewportView(getJTable());
		}
		return jDetailsPane;
	}

	/**
	 * This method initializes jTree
	 * 
	 * @return javax.swing.JTree
	 */
	private JTree getJTree() {
		if (jTree == null) {
			jTree = new JTree();
			jTree.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() { 
				public void valueChanged(javax.swing.event.TreeSelectionEvent e) {
					TreePath newLeadSelectionPath = e.getNewLeadSelectionPath();
					if (newLeadSelectionPath==null) {
						jTable.setVisible(false);
					} else {
						DefaultMutableTreeNode lastPathComponent = (DefaultMutableTreeNode) newLeadSelectionPath.getLastPathComponent();
						Visualizable v = CompositeVisualizer.getThreadInstance().toVisualizable(lastPathComponent.getUserObject());
						TableModel toTable = v.toTable();
						if (toTable!=null) {
							jTable.setModel(toTable);
						}
						jTable.setVisible(toTable!=null);
						//jTable.getColumnModel().getColumn(0).setMaxWidth(150);
					}
				}
			});
		}
		return jTree;
	}

	/**
	 * This method initializes jTable	
	 * 	
	 * @return javax.swing.JTable	
	 */    
	private JTable getJTable() {
		if (jTable == null) {
			jTable = new JTable();
		}
		return jTable;
	}
	

	/**
	 * @param root
	 */
	public Browser(String title, TreeNode root) {
		super();
		initialize();
		setTitle(title);
		getJTree().setModel(new DefaultTreeModel(root));
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(921, 616);
		this.setContentPane(getJContentPane());
		this.setTitle("JFrame");
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getJContentPane() {
		if (jContentPane == null) {
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			jContentPane = new javax.swing.JPanel();
			jContentPane.setLayout(new GridBagLayout());
			gridBagConstraints1.gridx = 0;
			gridBagConstraints1.gridy = 0;
			gridBagConstraints1.weightx = 1.0;
			gridBagConstraints1.weighty = 1.0;
			gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
			jContentPane.add(getJSplitPane(), gridBagConstraints1);
		}
		return jContentPane;
	}
	
	/**
	 * Shows objects in Browser. Blocks current thread until browser closes.
	 * @param o
	 * @param title
	 */
	public static void show(Object o, String title) {
		MutableTreeNode node = CompositeVisualizer.getThreadInstance().toVisualizable(o).toTree(title);
		final Object monitor = new Object();
		Browser browser = new Browser(title, node) {
			protected void processWindowEvent(WindowEvent e) {
				super.processWindowEvent(e);
				if (e.getID()==WindowEvent.WINDOW_CLOSED) {
					synchronized (monitor) {
						monitor.notifyAll();
					}
				}
			}
		};
		
		browser.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		browser.setVisible(true);
		
		synchronized (monitor) {
			try {
				monitor.wait();
			} catch (InterruptedException ex) {
				// Ignore
			}
		}
	}
	
	/**
	 * Shows object in browser with default close operation <code>WindowConstants.EXIT_ON_CLOSE</code>
	 * @param o
	 * @param title
	 */
	public static void showAndExitOnClose(Object o, String title) {
		MutableTreeNode node = CompositeVisualizer.getThreadInstance().toVisualizable(o).toTree(title);		
		Browser browser = new Browser(title, node);
		browser.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
		browser.setVisible(true);
	}	 	
	
} //  @jve:decl-index=0:visual-constraint="10,10"
