/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */

package biz.hammurapi.util;

import java.util.Collection;
import java.util.LinkedList;

import biz.hammurapi.RuntimeException;

/**
 * @author Pavel Vlasov	
 * @version $Revision: 1.3 $
 */
public abstract class AbstractSearchable implements Visitable, Searchable {

	public Object find(final Acceptor acceptor) {
		/**
		 * Doesn't indicate problem, but carries found object.
		 */
		class ReturnException extends RuntimeException {
			/**
			 * Comment for <code>serialVersionUID</code>
			 */
			private static final long serialVersionUID = 358344825019870509L;
			Object found;
			
			ReturnException(Object found) {
				this.found=found;
			}
		}
		
		try {
			accept(new Visitor() {
				public boolean visit(Object target) {
					if (acceptor.accept(target)) {
						throw new ReturnException(target);
					}
					return true;
				}			
			});
			return null;
		} catch (ReturnException e) {
			return e.found;
		}
	}

	public Collection findAll(final Acceptor acceptor) {
		final Collection ret=new LinkedList();
		accept(new Visitor() {
			public boolean visit(Object target) {
				if (acceptor.accept(target)) {
					ret.add(target);
				}
				return true;
			}			
		});
		return ret;
	}
}
