/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.util;

import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * @author Pavel Vlasov	
 * @version $Revision: 1.1 $
 */
public class AccumulatingExceptionSink implements ExceptionSink {
	private List exceptions=new LinkedList();
	
	public class Entry {
		Entry(Object source, Exception exception) {
			super();
			this.source = source;
			this.exception = exception;
		}
		Object source;
		Exception exception;
		
		public Exception getException() {
			return exception;
		}
		public Object getSource() {
			return source;
		}
	}

	public synchronized void consume(Object source, Exception e) {
		exceptions.add(new Entry(source, e));
	}
	
	public synchronized List getExceptions() {
		return Collections.unmodifiableList(exceptions);
	}
	
	public synchronized void reset() {
		exceptions.clear();
	}
	
	/**
	 * Invokes printStackTrace() for all accumulated exceptions 
	 */
	public synchronized void dump() {
		Iterator it=exceptions.iterator();
		while (it.hasNext()) {
			Entry entry=(Entry) it.next();
			System.err.println("Source: "+entry.getSource());
			entry.getException().printStackTrace();
		}
	}
}
