/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.util;

import java.lang.reflect.Method;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * @author Pavel Vlasov	
 * @version $Revision: 1.1 $
 */
public class AccumulatingVisitorExceptionSink implements VisitorExceptionSink {
	private List exceptions=new LinkedList();
	
	public class Entry {
		private Object visitor;
		private Exception exception;
		private Object visitee;
		private Method method;

		Entry(Object visitor, Method method, Object visitee, Exception exception) {
			this.visitor = visitor;
			this.exception = exception;
			this.visitee = visitee;
			this.method = method;
		}
		
		public Exception getException() {
			return exception;
		}
		
		public Method getMethod() {
			return method;
		}
		
		public Object getVisitee() {
			return visitee;
		}
		
		public Object getVisitor() {
			return visitor;
		}
	}

	public void consume(DispatchingVisitor dispatcher, Object visitor, Method method, Object visitee, Exception e) {
		exceptions.add(new Entry(visitor, method, visitee, e));
	}
	
	public synchronized List getExceptions() {
		return Collections.unmodifiableList(exceptions);
	}
	
	public synchronized void reset() {
		exceptions.clear();
	}
	
	/**
	 * Invokes printStackTrace() for all accumulated exceptions 
	 */
	public synchronized void dump() {
		Iterator it=exceptions.iterator();
		while (it.hasNext()) {
			Entry entry=(Entry) it.next();
			System.err.println("Visitor: "+entry.getVisitor());
			System.err.println("Method: "+entry.getMethod());
			System.err.println("Visitee: "+entry.getVisitee());
			entry.getException().printStackTrace();
		}
	}
}
