/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.util;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.Set;

/**
 * Visits class hierarchy in the order: class, implemented
 * interfaces, superclass, superclass implemented interfaces, ...
 * java.lang.Object is always visited last.
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public class ClassHierarchyVisitable implements Visitable {
	private Class clazz;
	
	public ClassHierarchyVisitable(Class clazz) {
		this.clazz=clazz;
	}
	
	public boolean accept(Visitor visitor) {
		LinkedList queue=new LinkedList();
		queue.add(clazz);
		Set visited=new HashSet();
		visited.add(Object.class);
		
		while (!queue.isEmpty()) {
			Class currentClazz=(Class) queue.removeFirst();
			if (visitor.visit(currentClazz)) {
				Class[] interfaces = currentClazz.getInterfaces();
				for (int i=0; i<interfaces.length; i++) {
					if (visited.add(interfaces[i])) {
						queue.add(interfaces[i]);
					}
				}
				
				Class superclass = currentClazz.getSuperclass();
				if (superclass!=null && visited.add(superclass)) {
					queue.add(superclass);
				}
			} else {
				return false;
			}
		}
		
		return visitor.visit(Object.class);
	}

}
