/*
  * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
  */
package biz.hammurapi.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import biz.hammurapi.convert.Converter;

/**
 * Collection that contains elements of one type but appears as
 * containing elements of another type. Conversion is done by a 
 * Converter. Usage - keep collection of 'keys' and retrieve
 * 'values' from secondary storage on demand. 
 * @author Pavel Vlasov
 *
 * @version $Revision: 1.3 $
 */
public class ConvertingCollection implements Collection {
	
	private Collection master;
	private Converter key2ValueConverter;
	private Converter value2KeyConverter;

	/**
	 * @param master
	 * @param key2ValueConverter
	 * @param value2KeyConverter
	 */
	public ConvertingCollection(Collection master, Converter key2ValueConverter, Converter value2KeyConverter) {
		super();
		this.master = master;
		this.key2ValueConverter = key2ValueConverter;
		this.value2KeyConverter = value2KeyConverter;
	}
	
	public int size() {
		return master.size();
	}

	public void clear() {
		master.clear();
	}

	public boolean isEmpty() {
		return master.isEmpty();
	}

	public Object[] toArray() {
		Object[] ret = master.toArray();
		for (int i=0; i<ret.length; i++) {
			ret[i]=key2ValueConverter.convert(ret[i]);
		}
		return ret;
	}

	public boolean add(Object o) {
		if (value2KeyConverter==null) {
			throw new IllegalStateException("Cannot add - value2KeyConverter is null");
		}
		
		return master.add(value2KeyConverter.convert(o));
	}

	public boolean contains(Object o) {
		if (value2KeyConverter==null) {
			throw new IllegalStateException("Cannot execute - value2KeyConverter is null");
		}
		
		return master.contains(value2KeyConverter.convert(o));
	}

	public boolean remove(Object o) {
		if (value2KeyConverter==null) {
			throw new IllegalStateException("Cannot remove - value2KeyConverter is null");
		}
		
		return master.remove(value2KeyConverter.convert(o));
	}

	public boolean addAll(Collection c) {
		if (value2KeyConverter==null) {
			throw new IllegalStateException("Cannot add - value2KeyConverter is null");
		}
		
		Collection converted=new ArrayList();
		Iterator it=c.iterator();
		while (it.hasNext()) {
			converted.add(value2KeyConverter.convert(it.next()));
		}
		return master.addAll(converted);
	}

	public boolean containsAll(Collection c) {
		if (value2KeyConverter==null) {
			throw new IllegalStateException("Cannot execute - value2KeyConverter is null");
		}
		
		Collection converted=new ArrayList();
		Iterator it=c.iterator();
		while (it.hasNext()) {
			converted.add(value2KeyConverter.convert(it.next()));
		}
		return master.containsAll(converted);
	}

	public boolean removeAll(Collection c) {
		if (value2KeyConverter==null) {
			throw new IllegalStateException("Cannot remove - value2KeyConverter is null");
		}
		
		Collection converted=new ArrayList();
		Iterator it=c.iterator();
		while (it.hasNext()) {
			converted.add(value2KeyConverter.convert(it.next()));
		}
		return master.removeAll(converted);
	}

	public boolean retainAll(Collection c) {
		if (value2KeyConverter==null) {
			throw new IllegalStateException("Cannot retain - value2KeyConverter is null");
		}
		
		Collection converted=new ArrayList();
		Iterator it=c.iterator();
		while (it.hasNext()) {
			converted.add(value2KeyConverter.convert(it.next()));
		}
		return master.retainAll(converted);
	}

	public Iterator iterator() {
		final Iterator masterIterator=master.iterator();
		return new Iterator() {

			public void remove() {
				masterIterator.remove();
			}

			public boolean hasNext() {
				return masterIterator.hasNext();
			}

			public Object next() {
				return key2ValueConverter.convert(masterIterator.next());
			}			
		};
	}

	public Object[] toArray(Object[] a) {
        if (a.length < size()) {
            a = (Object[]) java.lang.reflect.Array.newInstance(
                                a.getClass().getComponentType(), size());
        }

        
        Iterator it=master.iterator();
        for (int i=0; it.hasNext(); i++) {
        	a[i]=key2ValueConverter.convert(it.next());
        }

        if (a.length > size()) {
            a[size()] = null;
        }

        return a;
	}

}
