/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */

package biz.hammurapi.util;

import java.util.LinkedList;

/**
 * Has internal invocation queue and pumps all invocation to be made 
 * from a single thread. A workaround to use thread-unsafe classes in
 * multithreaded environment.
 * @author Pavel Vlasov	
 * @version $Revision: 1.3 $
 */
public class InvocationPump {
	private LinkedList queue=new LinkedList();
	
	protected abstract class Command {
		private Object returnValue;
		private Throwable exception;
		
		/**
		 * Put single-threaded functionality here
		 */
		protected abstract Object execute() throws Throwable;
		
		/**
		 * Posts the command to the queue and waits
		 * until it is executed
		 */
		public synchronized Object post() throws Throwable {
			synchronized (queue) {
				queue.add(this);
				queue.notifyAll();
			}
			
			wait();
			
			if (exception!=null) {
				throw exception;
			}
			
			return returnValue;
		}
	}
	
	public void pump() {
		synchronized (queue) {
			if (!queue.isEmpty()) {
				Command command=(Command) queue.removeFirst();
				try {
					command.returnValue=command.execute();
				} catch (Throwable th) {
					command.exception=th;
				}
				
				synchronized(command) {
					command.notifyAll();
				}
			}
		}
	}
}
