/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.util;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

/**
 * Load balancer distributes work to other workers
 * according to weights. The less the weight the more
 * job worker is getting.
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public class LoadBalancer implements Worker {

	private List workers=new ArrayList();
	
	private void descore() {
		synchronized (workers) {
			Iterator it=workers.iterator();
			while (it.hasNext()) {
				((WorkerEntry) it.next()).score-=Integer.MAX_VALUE;
			}
		}
	}
	
	private class WorkerEntry implements Comparable {
		Worker worker;
		int weight;
		int score;
		
		/**
		 * @param worker
		 * @param weight Must be >0
		 */
		WorkerEntry(Worker worker, int weight) {
			super();
			if (weight<1) {
				throw new IllegalArgumentException("Worker weight must be >0");
			}
			this.worker = worker;
			this.weight = weight;
		}

		public int compareTo(Object o) {
			if (o instanceof WorkerEntry) {
				return score-((WorkerEntry) o).score;
			}
				
			return hashCode()-o.hashCode();
		}
		
		boolean post(Runnable job) {
			if (worker.post(job)) {
				// reduce all scores to avoid overflow.
				if (score>Integer.MAX_VALUE-weight) {
					descore();
				}
				
				score+=weight;
				return true;
			}
			return false;
		}
	}
	
	public boolean post(Runnable job) {
		synchronized (workers) {
			Collections.sort(workers);
			Iterator it=workers.iterator();
			while (it.hasNext()) {
				WorkerEntry we=(WorkerEntry) it.next();
				if (we.post(job)) {
					return true;
				}
			}
		}
		
		return false;
	}

	public void addWorker(Worker worker, int weight) {
		synchronized (workers) {
			workers.add(new WorkerEntry(worker, weight));
		}
	}
}
