/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.util;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Aggregates multiple classpath resources in one resource stream.
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public class ResourceAggregator {
	private List resourceList=new ArrayList();
	
	/**
	 * Adds resource to the list of resources to be aggregated. 
	 * @param resourceName
	 */
	public void addResource(String resourceName) {
		resourceList.add(resourceName);
	}
	
	/**
	 * Reads each line from 'resourceListName' resource and adds
	 * to the list of resources to be aggregated.
	 * Directive #include allows to include another resource
	 * list. If line has # as its first non-blank character then this 
	 * line is treated as comment line. 
	 * @param resourceListName
	 * @throws IOException 
	 */
	public void addResourceList(String resourceListName) throws IOException {
		InputStream resourceStream = getClass().getClassLoader().getResourceAsStream(resourceListName);
		if (resourceStream==null) {
			throw new IOException("Script list resource not found: "+resourceListName);
		}
		
		BufferedReader br=new BufferedReader(new InputStreamReader(resourceStream));
		String s;					
		while ((s=br.readLine())!=null) {
			String strim = s.trim();
			if (strim.startsWith("#include")) {
				addResourceList(strim.substring("#include".length()).trim());
			} else if (strim.length()>0 && !strim.startsWith("#")) {
				resourceList.add(strim);
			}
		}
		br.close();
	}
	
	/**
	 * Aggregates all resources in one resource.
	 * @return Aggregated input stream.
	 * @throws IOException
	 */
	public InputStream aggregate() throws IOException {
		ByteArrayOutputStream baos=new ByteArrayOutputStream();
		Iterator it=resourceList.iterator();
		ClassLoader classLoader=getClass().getClassLoader();
		while (it.hasNext()) {
			String resourceName=(String) it.next();
			InputStream in=classLoader.getResourceAsStream(resourceName);
			if (in==null) {
				throw new IOException("Resource not found: "+resourceName);
			}
			byte[] buf=new byte[4096];
			int l;
			while ((l=in.read(buf))!=-1) {
				baos.write(buf, 0, l);
			}
			in.close();
		}
		baos.close();
		
		return new ByteArrayInputStream(baos.toByteArray());
	}

	/**
	 * Aggregates resource lists listed in command line arguments and outputs them to console. 
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			ResourceAggregator aggregator=new ResourceAggregator();
			for (int i=0; i<args.length; i++) {
				aggregator.addResourceList(args[i]);
			}
			
			BufferedReader br=new BufferedReader(new InputStreamReader(aggregator.aggregate()));
			String line;
			while ((line=br.readLine())!=null) {
				System.out.println(line);
			}
		} catch (IOException e) {
			System.err.println("ERROR: "+e.getMessage());
			System.exit(1);
		}
		

	}

}
