/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.util;

import java.util.LinkedList;

/**
 * Processes tasks from differen threads in a single thread.
 * @author Pavel Vlasov	
 * @version $Revision: 1.6 $
 */
public class SerialProcessor {
	
	private LinkedList tasks=new LinkedList();
	
	{
		new Thread() {
			{
				setName("Task processing thread");
				start();
			}
	
			public void run() {
				try {
					while (true) {
						Runnable task;
						synchronized (tasks) {
							while (tasks.isEmpty()) {
								try {
									tasks.wait();
								} catch (InterruptedException e) {
									return;
								}
							}
							
							task=(Runnable) tasks.removeFirst();
						}
						
						if (task==null) {
							return;
						} 
	
						try {
						    task.run();
						} finally {
						    task.notifyAll();
						}
					}
				} catch (Exception e) {
				    e.printStackTrace();
				} finally {
				    synchronized (tasks) {
				        terminated=true;
				        tasks.notifyAll();
				    }
				}
			}
		};
	
	    Runtime.getRuntime().addShutdownHook(new Thread() {
            public void run() {
                synchronized (tasks) {
    				tasks.add(null);
    				tasks.notifyAll();                    
                }
            }
	    });
	}
	
	private void checkTerminated() {
	    synchronized (tasks) {
	        if (terminated) {
	            throw new IllegalStateException("Task processing thread was terminated prematurely");
	        }
	    }
	}	
	
	private boolean terminated;

	/**
	 * Executes task in a separate thread
	 * @throws InterruptedException
	 */
	public void execute(Runnable task) throws InterruptedException {
		if (task!=null) {
			synchronized (tasks) {
			    checkTerminated();			    
				tasks.add(task);
				tasks.notifyAll();
			}	
			
			synchronized (task) {
			    task.wait();
			}
		}
	}
	
	private static SerialProcessor defaultInstance;
	
	public static SerialProcessor getDefaultInstance() {
	    synchronized (SerialProcessor.class) {
	        if (defaultInstance==null) {
	            defaultInstance=new SerialProcessor();
	        }
	    }
	    return defaultInstance;
	}
}
