/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.util;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import biz.hammurapi.config.Component;
import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.config.Context;
import biz.hammurapi.config.DomConfigurable;
import biz.hammurapi.config.Wrapper;

public class ThreadPoolComponentWrapper implements Wrapper, Component, DomConfigurable {
	private ThreadPool master;
	private int numberOfThreads=20;
	private int priority=Thread.NORM_PRIORITY;
	private int maxQueue=1000;
	private String exceptionSinkName;
	private Object owner;
	private String name;
	
	public Object getMaster() {
		return master;
	}

	public void start() throws ConfigurationException {
		ExceptionSink exceptionSink=null;
		if (exceptionSinkName!=null) {
			if (!(owner instanceof Context)) {
				throw new ConfigurationException("Cannot lookup exception sink - owner doesn't implement Context");
			}
			
			exceptionSink=(ExceptionSink) ((Context) owner).get(exceptionSinkName);
			
			if (exceptionSink==null) {
				throw new ConfigurationException("Lookup failed for exception sink '"+exceptionSinkName+"'");
			}
		}
		
		master=new ThreadPool(numberOfThreads, priority, maxQueue, exceptionSink, name);
		master.start();
	}

	public void stop() throws ConfigurationException {
		if (master!=null) {
			master.stop();
		}
	}

	public void setOwner(Object owner) {
		this.owner=owner;
	}

	public void configure(Node configNode, Context context) throws ConfigurationException {
		Element e=(Element) configNode;
		if (e.hasAttribute("threads")) {
			numberOfThreads=Integer.parseInt(e.getAttribute("threads"));
			if (numberOfThreads<1) {
				throw new ConfigurationException("Number of threads shall be >=1");
			}
		}
		
		if (e.hasAttribute("max-queue")) {
			maxQueue=Integer.parseInt(e.getAttribute("max-queue"));
		}
		
		if (e.hasAttribute("exception-sink")) {
			exceptionSinkName=e.getAttribute("exception-sink");
		}

		if (e.hasAttribute("pool-name")) {
			name=e.getAttribute("pool-name");
		}
		
		if (e.hasAttribute("priority")) {
			String pStr=e.getAttribute("priority");
			if ("min".equalsIgnoreCase(pStr)) {
				priority=Thread.MIN_PRIORITY;
			} else if ("max".equalsIgnoreCase(pStr)) {
				priority=Thread.MAX_PRIORITY;
			} else if ("normal".equalsIgnoreCase(pStr)) {
				priority=Thread.NORM_PRIORITY;
			} else {
				throw new ConfigurationException("Invalid priority value: "+pStr);
			}
		}
	}

}
