/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.util;

import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * Simple class to build tree structures
 * @author Pavel Vlasov	
 * @version $Revision: 1.4 $
 */
public class TreeNode extends VisitableBase implements Attributable {
	private TreeNode parent;

	public TreeNode() {
		super();
	}

	/* (non-Javadoc)
	 * @see biz.hammurapi.util.Visitable#accept(biz.hammurapi.util.Visitor)
	 */
	protected void acceptChildren(Visitor visitor) {
		childrenVisitable.accept(visitor);
	}

	private Map attributes=new HashMap();
	
	public void setAttribute(Object key, Object value) {
		if (value==null) {
			attributes.remove(key);
		} else {
			attributes.put(key, value);
		}
	}
	
	public Object getAttribute(Object key) {
		return attributes.get(key);
	}
	
	private List children=new LinkedList();
	private Visitable childrenVisitable=new CollectionVisitable(children, false);
	
	public List getChildren() {
		return Collections.unmodifiableList(children);		
	}
	
	public void addChild(Object child) {
		children.add(child);
		if (child instanceof TreeNode) {
			((TreeNode) child).parent=this;
		}
	}
	
	public boolean hasAttribute(String name) {
		return attributes.containsKey(name);
	}
	/**
	 * @return Returns the parent.
	 */
	public TreeNode getParent() {
		return parent;
	}

	public Object removeAttribute(Object key) {
		return attributes.remove(key);
	}

}
