/*
  * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.util;

import java.util.StringTokenizer;

/**
 * Represents 3-digit version
 * @author Pavel Vlasov
 * @version $Revision: 1.2 $
 */
public final class Version implements Comparable {
	private int major;
	private int minor;
	private int micro;

	/**
	 * Create a new instance of a <code>Version</code> object with the
	 * specified version numbers.
	 *
	 * @param major Major version number.
	 * @param minor Minor version number.
	 * @param rev Micro version number.
	 */
	public Version(int major, int minor, int micro) {
		this.major = major;
		this.minor = minor;
		this.micro = micro;
	}
	
	public Version(String version) {
		StringTokenizer st=new StringTokenizer(version, ".");
		if (st.countTokens()==3) {
			major=Integer.parseInt(st.nextToken());
			minor=Integer.parseInt(st.nextToken());
			micro=Integer.parseInt(st.nextToken());
		} else {
			throw new IllegalArgumentException("Invalid version format: "+version);
		}
	}

	/**
	 * 
	 * @param other
	 * @return
	 */
	public boolean equals(Object o) {
		if (o instanceof Version) {
			Version v=(Version) o;
			return v.major==major && v.minor==minor && v.micro==micro;
		} else {
			return super.equals(o);
		}
	}

	/**
	 * Overload toString to report version correctly.
	 *
	 * @return the dot seperated version string
	 */
	public String toString() {
		return major + "." + minor + "." + micro;
	}

	public int compareTo(Object o) {
		if (o instanceof Version) {
			Version v=(Version) o;
			if (v.major==major) {
				if (v.minor==minor) {
					return micro-v.micro;
				} else {
					return minor-v.minor;
				}
			} else {
				return major-v.major;
			}
		} else {
			return hashCode()-o.hashCode();
		}
	}
	
	public int hashCode() {
		return major ^ minor ^ micro;
	}

	public int getMajor() {
		return major;
	}

	public int getMicro() {
		return micro;
	}

	public int getMinor() {
		return minor;
	}
}