/*
  * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz 
 */
package biz.hammurapi.util;


/**
 * Implements Visitable contract. Subclasses shall implement visitChildren() method.
 * @author Pavel Vlasov
 * @version $Revision: 1.4 $
 */
public abstract class VisitableBase implements Visitable {
	
	public boolean accept(Visitor visitor) {
		if (visitor==null) {
			return false;
		}
		
		if (visitor.visit(this)) {
			acceptChildren(visitor);
			if (visitor instanceof PoliteVisitor) {
				((PoliteVisitor) visitor).leave(this);
			}
			return true;
		}
		
		return false;
	}

	/**
	 * Override this method to navigate visitor through object constituents.
	 * @param visitor
	 */
	protected void acceptChildren(Visitor visitor) {
		// No functionality
	}
	
	/**
	 * Properly handles passing visitor to a child - takes into account that child is visitable and visitor may
	 * be an instance of PoliteVisitor.
	 * @param visitor
	 * @param child
	 */
	public static void object2visitor(Object object, Visitor visitor) {
		if (visitor!=null && object!=null) {
			if (object instanceof Visitable) {
				((Visitable) object).accept(visitor);
			} else {
				if (visitor.visit(object) && visitor instanceof PoliteVisitor) {
					((PoliteVisitor) visitor).leave(object);
				}
			}
		}
	}
}
