/*
  * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
  */
package biz.hammurapi.util;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Stack;

/**
 * Provides visiting stack information.
 * @author Pavel Vlasov
 * @version $Revision: 1.3 $
 */
public class VisitorStack {
	VisitorStack() {
		// Default constructor
		super();
	}
	
	private Stack stack=new Stack();
	private List unmodifiableStack = Collections.unmodifiableList(stack);
	
	void push(Object o) {
//		tab();
//		System.out.println("-> "+o);
		stack.push(o);		
	}

	/**
	 * 
	 */
//	private void tab() {
//		for (int i=0; i<stack.size(); i++) {
//			System.out.print("  ");
//		}
//	}
	
	/**
	 * Removes objects before this object and the object itself from the stack. 
	 * == operator used for objects comparison
	 */
	void pop(Object o) {
		Iterator it=stack.iterator();
		while (it.hasNext()) {
			if (o==it.next()) {
				while (!stack.isEmpty() && stack.pop()!=o);
				break;
			}
		}
//		tab();
//		System.out.println("<- "+o);
	}
	
	/**
	 * 
	 * @return Object from the top of the stack.
	 */	
	public Object peek() {
		return stack.peek();		
	}
	
	/**
	 * 
	 * @return Unmodifiable stack
	 */
	public List getStack() {
		return unmodifiableStack;
	}
	
	/**
	 * @param clazz
	 * @return true if stack contains objects of type of the class.
	 */
	public boolean isIn(Class clazz) {
		Iterator it=stack.iterator();
		while (it.hasNext()) {
			if (clazz.isInstance(it.next())) {
				return true;
			}
		}
		return false;
	}

	/**
	 * @param clases
	 * @return true if stack contains objects of type of one of the classes.
	 */
	public boolean isIn(Class[] classes) {
		Iterator it=stack.iterator();
		while (it.hasNext()) {
			Object next = it.next();
			for (int i=0; i<classes.length; i++) {
				if (classes[i].isInstance(next)) {
					return true;
				}
			}
		}
		return false;
	}
	
	/**
	 * @param clazz
	 * @return list of stack entries of given type.
	 */
	public List getStack(Class clazz) {
		List ret=new ArrayList();
		Iterator it=stack.iterator();
		while (it.hasNext()) {
			Object next = it.next();
			if (clazz.isInstance(next)) {
				ret.add(next);
			}
		}
		return ret;
	}

	/**
	 * @param clases
	 * @return list of stack entries of given types.
	 */
	public List getStack(Class[] classes) {
		List ret=new ArrayList();
		Iterator it=stack.iterator();
		while (it.hasNext()) {
			for (int i=0; i<classes.length; i++) {
				Object next = it.next();
				if (classes[i].isInstance(next)) {
					ret.add(next);
				}
			}
		}
		return ret;
	}
	
	public void print() {
		Iterator it=stack.iterator();
		while (it.hasNext()) {
			System.out.println(it.next());
		}
	}	
	
}
