/*
  * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
  */
package biz.hammurapi.xml.dom;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.w3c.dom.Element;

import biz.hammurapi.xml.dom.CompositeDomSerializer.Member;

/**
 * @author Pavel Vlasov
 *
 * @version $Revision: 1.4 $
 */
public class BeanDomSerializer implements Member {
    private class StackEntry {
        long counter;
        
        private class IdentityWrapper {
        	Object o;
        	
        	public IdentityWrapper(Object o) {
        		this.o=o;
			}
        	
        	public int hashCode() {
        		return o.hashCode();
        	}
        	
        	public boolean equals(Object obj) {
        		return obj instanceof IdentityWrapper && o==((IdentityWrapper) obj).o;
        	}
        }
        
        Map identityMap=new HashMap();
        Map referenceMap=new HashMap();
        
        Long getReference(Object o) {
            if (o instanceof Number || o instanceof String) {
                return (Long) referenceMap.get(o);
            }
            
            return (Long) identityMap.get(new IdentityWrapper(o));
        }
        
        long addReference(Object o) {
            if (o instanceof Number || o instanceof String) {
                Long ref=(Long) referenceMap.get(o);
                if (ref==null) {
                    long next=counter++;
                    referenceMap.put(o, new Long(next));
                    return next;
                }
                
                return ref.longValue();
            }
            
            IdentityWrapper iw = new IdentityWrapper(o);
            
            Long ref=(Long) identityMap.get(iw);
            if (ref==null) {
                long next=counter++;
                identityMap.put(iw, new Long(next));
                return next;
            }
            
            return ref.longValue();
        }
    }
    
	private static ThreadLocal stack=new ThreadLocal();

	private CompositeDomSerializer owner;	
	
	private Set badMethods=new HashSet();

	public DomSerializable toDomSerializable(final Object object) {
		if (object!=null) {
			return new DomSerializable() {

				public void toDom(Element holder) {
				    if (object instanceof Class) {
				        holder.appendChild(holder.getOwnerDocument().createTextNode(((Class) object).getName()));
				        return;
				    }
				    
				    StackEntry sEntry=(StackEntry) stack.get();
				    boolean clean = sEntry==null;
				    if (clean) {
				        sEntry=new StackEntry();
				        stack.set(sEntry);
				    }
				    
				    try {
				        
				    	Long refId=sEntry.getReference(object);
				    	if (refId!=null) {
				    		holder.setAttribute("refid", refId.toString());
				    		return;
				    	} 
				    	
					    long nextCounter = sEntry.addReference(object);
					    //System.out.println("New id: "+nextCounter);
						holder.setAttribute("id", String.valueOf(nextCounter));
					    
					    holder.setAttribute("type", object.getClass().getName());
					    holder.appendChild(holder.getOwnerDocument().createTextNode(object.toString()));
					    
					    if (owner!=null) {
						    Class beanClass=object.getClass();
							final Object[] args = new Object[] {};
						    Method[] methods = beanClass.getMethods();
						    for (int i=0; i<methods.length; i++) {
						    	// getXXX() methods. Object.getClass() is not included.
						    	Method method = methods[i];
								if (!(void.class.equals(method.getReturnType()))
										&& Modifier.isPublic(method.getModifiers())
						    			&& !Modifier.isAbstract(method.getModifiers())
						    			&& !(method.getDeclaringClass().equals(Object.class)) 
						    			&& !Modifier.isStatic(method.getModifiers()) 
						    			&& method.getName().startsWith("get") 
						    			&& method.getParameterTypes().length==0
						    			&& !isBad(method)) {
						    		try {
										Object value=method.invoke(object, args);
										if (value!=object) {
											DomSerializable vds=owner.getStackHead().toDomSerializable(value);
											if (vds!=null) {
										        Element ve=holder.getOwnerDocument().createElement(method.getName().substring(3));		        
										        holder.appendChild(ve);
										        vds.toDom(ve);
											}
										}
									} catch (Exception e) {
										synchronized (badMethods) {
											badMethods.add(method);
										}
								        handleAccessError(holder, method, e);
									}
						    	}					    						    	
						    }
						    
						    Field[] fields = beanClass.getFields();
						    for (int i=0; i<fields.length; i++) {
						    	Field field = fields[i];
								if (!Modifier.isStatic(field.getModifiers()) && Modifier.isPublic(field.getModifiers())) {
						    		try {
										Object value=field.get(object);								
										DomSerializable vds=owner.getStackHead().toDomSerializable(value);
										if (vds!=null) {
									        Element ve=holder.getOwnerDocument().createElement(field.getName());		        
									        holder.appendChild(ve);
									        vds.toDom(ve);
										}
									} catch (Exception e) {
								        handleAccessError(holder, field, e);
									}
						    	}
						    }
					    } 
				    } finally {
				        if (clean) {
				            stack.set(null);
				        }
				    }
				}

				private boolean isBad(Method method) {
					synchronized (badMethods) {
						return badMethods.contains(method);
					}
				}				
			};
		}
		
		return null;
	}

	/**
	 * Callback method
	 */
	public void setOwner(CompositeDomSerializer owner) {
		this.owner=owner;
	}

	private void handleAccessError(Element holder, java.lang.reflect.Member member, Throwable e) {
		Element ee=holder.getOwnerDocument().createElement("access-error");
		holder.appendChild(ee);
		ee.setAttribute("member", member.toString());
		DomSerializable eds=owner.getStackHead().toDomSerializable(e);
		eds.toDom(ee);
	}
}
