/*
 * hgcommons 7
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.xml.dom;

import java.lang.reflect.Field;

import org.w3c.dom.Element;

import biz.hammurapi.RuntimeException;
import biz.hammurapi.convert.CompositeConverter;


/**
 * @author Pavel Vlasov
 * @version $Revision: 1.4 $
 */
public class XmlBeanBase implements DomSerializable {

	public void toDom(Element holder) {
		holder.setAttribute("type", getClass().getName());
	    for (Class clazz=getClass(); !clazz.equals(XmlBeanBase.class); clazz=clazz.getSuperclass()) {
			try {
				Field[] fields = clazz.getDeclaredFields();
				for (int i=0, j=fields.length; i<j; i++) {
					boolean accessible=fields[i].isAccessible();
					fields[i].setAccessible(true);
					Object fieldValue=fields[i].get(this);
					fields[i].setAccessible(accessible);
					if (fieldValue!=null) {
						CompositeDomSerializer
							.getThreadInstance()
							.toDomSerializable(fieldValue)
							.toDom(AbstractDomObject.addElement(holder,fields[i].getName()));
					}
				}
			} catch (IllegalAccessException iae) {
				throw new RuntimeException("Cannot access field: "+iae, iae);
			}
	    }
	}
	
	public String toString() {
		StringBuffer ret=new StringBuffer(getClass().getName());
		ret.append("[");
	    for (Class clazz=getClass(); !clazz.equals(XmlBeanBase.class); clazz=clazz.getSuperclass()) {
			try {
				Field[] fields = clazz.getDeclaredFields();
				for (int i=0, j=fields.length; i<j; i++) {
				    if (i>0) {
				        ret.append(", ");
				    }
					ret.append(fields[i].getName());
					ret.append("=");
					boolean accessible=fields[i].isAccessible();
					fields[i].setAccessible(true);
					Object fieldValue=fields[i].get(this);
					fields[i].setAccessible(accessible);
					if (fieldValue==null) {
						ret.append("(null)");
					} else {
						ret.append(fieldValue);
					}				
				}
			} catch (IllegalAccessException iae) {
				throw new RuntimeException("Cannot access field: "+iae, iae);
			}
	    }
		ret.append("]");
		return ret.toString();
	}	
	
	
//	/**
//	 * @param rs
//	 * @throws SQLException
//	 */
//	protected static void checkObject(Object o, String column, String targetType) throws SQLException {
//		System.out.println(column+" -> ("+targetType+") "+(o==null ? "null" : o.getClass().getName()));
//	}
	
	/**
	 * Converts one type to another if possible
	 * @return Converted object.
	 */
	public static Object convert(Object arg, String targetClass) {
		try {
			return CompositeConverter.getDefaultConverter().convert(arg, Class.forName(targetClass), false);
		} catch (ClassNotFoundException e) {
			throw new ClassCastException("Target class not found: "+targetClass);			
		}
	}

	/**
	 * Two objects are considered equal and all their fields are equal.
	 */
    public boolean equals(Object otherBean) {
        if (otherBean==null) {
            return false;
        } else if (getClass().equals(otherBean.getClass())) {
			Field[] fields = getClass().getFields();
			for (int i=0, j=fields.length; i<j; i++) {
			    try {
					Object fieldValue=fields[i].get(this);
					Object otherFieldValue=fields[i].get(otherBean);
					if (fieldValue==null) {
					    if (otherFieldValue!=null) {
					        return false;
					    } 
					} else {
				        if (otherFieldValue==null) {
				            return false;
				        } else if (!fieldValue.equals(otherFieldValue)) {
				            return false;
				        }
					}
			    } catch (IllegalAccessException e) {
			        throw new RuntimeException("Cannot compare objects", e);
			    }
			}            
			return true;
        } else {
            return false;
        }
    }
    
	public int hashCode() {
		int ret=0;
		Field[] fields = getClass().getFields();
		for (int i=0, j=fields.length; i<j; i++) {
		    try {
		    	ret^=fields[i].getName().hashCode();
				Object fieldValue=fields[i].get(this);
				if (fieldValue!=null) {
					ret^=fieldValue.hashCode();
				}
		    } catch (IllegalAccessException e) {
		        throw new RuntimeException("Cannot calculate hash code", e);
		    }
		}            
		return ret;
	}
}
